package com.main.campustream;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

/**
 * @author Bryan Crabtree
 * 
 */
public class APIs {

	/**
	 * Returns a user's profile as a json string.
	 * 
	 * @param username
	 * @return
	 */
	public static String GetUserProfile(String username) {
		String statusList = "";
		String url = ("http://api.campustream.com/1/users/show.json?username=" + username);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			statusList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return statusList;
	}

	/**
	 * Returns a list of users that the current user is following as a json string.
	 * 
	 * @return
	 */
	public static String GetFriendList() {
		String friendList = "";
		String url = ("http://api.campustream.com/1/friendships/follows.json?username=" + Campustream.username);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			friendList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return friendList;
	}

	/**
	 * Returns the specified user's most recent status message as a json string.
	 * 
	 * @param userName
	 * @return
	 */
	public static String GetUserStatus(String userName) {
		String friendStatus = "";
		String url = ("http://api.campustream.com/1/statuses/user_timeline.json?username=" + userName + "&count=1");
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			friendStatus = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return friendStatus;
	}

	/**
	 * Returns a user's 50 most recent status messages as a json string.
	 * 
	 * @param userName
	 * @return
	 */
	public static String GetUserActivity(String userName) {
		String friendStatus = "";
		String url = ("http://api.campustream.com/1/statuses/user_timeline.json?username=" + userName + "&count=50");
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			friendStatus = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return friendStatus;
	}

	/**
	 * Returns a list of users with the specified name (Partial Matching) as a json string,
	 * 
	 * @param friendUsername
	 * @return
	 */
	public static String FindFriends(String friendUsername) {
		String friendList = "";
		String url = ("http://apidev.campustream.com/1/search/users.json?q=" + friendUsername);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			friendList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return friendList;
	}

	/**
	 * Returns the 50 most recent public status messages as a json string.
	 * 
	 * @return
	 */
	public static String GetStream() {
		String statusList = "";
		String url = ("http://api.campustream.com/1/statuses/public_timeline.json?count=50");
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			statusList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return statusList;
	}

	/**
	 * Returns a list of questions in the specified section with the specified sorting method as a json string.
	 * 
	 * @param section
	 * @param sorting
	 * @return
	 */
	public static String GetQuestionList(String section, String sorting) {
		String questionList = "";
		String url = ("http://api.campustream.com/1/collaborate/list.json?section=" + section + "&sorting=" + sorting);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			questionList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return questionList;
	}

	/**
	 * Returns the responses for the specified question as a json string.
	 * 
	 * @param id
	 * @return
	 */
	public static String GetQuestionResponses(String id) {
		String questionList = "";
		String url = ("http://api.campustream.com/1/collaborate/show.json?id=" + id);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			questionList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return questionList;
	}

	/**
	 * Returns the user with the specified ID as a json string.
	 * 
	 * @param id
	 * @return
	 */
	public static String GetUserWithID(String id) {
		String questionList = "";
		String url = ("http://api.campustream.com/1/friendships/show.json?source_id=" + id + "&target_id=" + id);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			questionList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return questionList;
	}

	/**
	 * Rates the specified question response up or down.
	 * 
	 * @param direction
	 *            up OR down
	 * @param id
	 */
	public static void rateQuestion(String direction, String id) {
		String url = "http://api.campustream.com/1/collaborate/vote";
		HttpClient client = new DefaultHttpClient();
		HttpPost post = new HttpPost(url);
		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair("access_token", Campustream.accessToken));
		params.add(new BasicNameValuePair("id", id));
		params.add(new BasicNameValuePair("dir", direction));
		try {
			UrlEncodedFormEntity ent = new UrlEncodedFormEntity(params);
			post.setEntity(ent);
			client.execute(post);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns the specified section of news and events as a json string.
	 * 
	 * @param section
	 * @return
	 */
	public static String GetNewsList(String section) {
		String newsList = "";
		String url = ("http://api.campustream.com/1/news/list.json?type=" + section);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			newsList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return newsList;
	}

	/**
	 * Returns the specified section of news and events as a json string.
	 * 
	 * @param section
	 * @param category
	 * @return
	 */
	public static String GetNewsListSection(String section, String category) {
		String newsList = "";
		String url = ("http://api.campustream.com/1/news/list.json?type=" + section + "&slug=" + category);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			newsList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return newsList;
	}

	/**
	 * Returns the comments on the specified new or event item as a json string.
	 * 
	 * @param id
	 * @return
	 */
	public static String GetNewsComments(String id) {
		String newsList = "";
		String url = ("http://api.campustream.com/1/news/show.json?id=" + id);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			newsList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return newsList;
	}

	/**
	 * Rates the specified news or event item up or down.
	 * 
	 * @param direction
	 *            up OR down
	 * @param id
	 */
	public static void rateNews(String direction, String id) {
		String url = "http://api.campustream.com/1/news/vote";
		HttpClient client = new DefaultHttpClient();
		HttpPost post = new HttpPost(url);
		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair("access_token", Campustream.accessToken));
		params.add(new BasicNameValuePair("id", id));
		params.add(new BasicNameValuePair("dir", direction));
		try {
			UrlEncodedFormEntity ent = new UrlEncodedFormEntity(params);
			post.setEntity(ent);
			client.execute(post);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns the comments for the specified status update as a json string.
	 * 
	 * @param id
	 * @return
	 */
	public static String GetStreamComments(String id) {
		String commentList = "";
		String url = ("http://api.campustream.com/1/statuses/comments.json?status_id=" + id);
		BufferedReader in = null;
		HttpClient client = new DefaultHttpClient();
		HttpGet get = new HttpGet(url);
		try {
			HttpResponse response = client.execute(get);
			in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			StringBuffer sb = new StringBuffer("");
			String line = "";
			String NL = System.getProperty("line.separator");
			while ((line = in.readLine()) != null) {
				sb.append(line + NL);
			}
			in.close();
			sb.deleteCharAt(sb.length() - 1);
			commentList = sb.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return commentList;
	}
}
