package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class CommentList extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<StatusComment> m_statuses = null;
	private StatusAdapter m_adapter;
	private Runnable viewStatuses;

	int numComments = 0;
	String commentList = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.commentlistview);
		m_statuses = new ArrayList<StatusComment>();
		this.m_adapter = new StatusAdapter(this, R.layout.commentlistviewrows, m_statuses);
		setListAdapter(this.m_adapter);

		Button commentButton = (Button) findViewById(R.id.commentButton);
		commentButton.setOnClickListener(this);

		Button refreshButton = (Button) findViewById(R.id.refreshButton);
		refreshButton.setOnClickListener(newRefreshListener);

		viewStatuses = new Runnable() {
			@Override
			public void run() {
				getComments();
			}
		};
		Thread thread = new Thread(null, viewStatuses, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(CommentList.this, "Please Wait", "Retrieving Comments", true);
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_statuses != null && m_statuses.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_statuses.size(); i++)
					m_adapter.add(m_statuses.get(i));
			}
			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * Retrieves the comments for the current status.
	 */
	private void getComments() {
		try {
			String statusName = "";
			String statusUser = "";
			String statusIcon = "";
			String statusMessage = "";
			String statusID = "";
			m_statuses = new ArrayList<StatusComment>();
			commentList = APIs.GetStreamComments(Campustream.statusID);
			JSONObject streamItem = new JSONObject(commentList);
			JSONArray comments = streamItem.getJSONArray("comments");
			numComments = comments.length();

			for (int x = 0; x < numComments; x++) {
				StatusComment currentStatus = new StatusComment();

				JSONObject jsonComment = comments.getJSONObject(x);
				JSONObject jsonUser = jsonComment.getJSONObject("user");

				statusName = jsonUser.getString("name");
				statusUser = jsonUser.getString("username");
				statusIcon = jsonUser.getString("avatar");
				statusMessage = jsonComment.getString("message");
				statusID = jsonComment.getString("id");
				statusIcon = statusIcon.replaceFirst("medium", "small");

				statusName = Campustream.correctString(statusName);
				statusMessage = Campustream.correctString(statusMessage);

				currentStatus.setCommentName(statusName);
				currentStatus.setCommentUser(statusUser);
				currentStatus.setCommentIcon(statusIcon);
				currentStatus.setCommentMessage(statusMessage);
				currentStatus.setCommentID(statusID);

				m_statuses.add(currentStatus);
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class StatusAdapter extends ArrayAdapter<StatusComment> {

		private ArrayList<StatusComment> items;

		public StatusAdapter(Context context, int textViewResourceId, ArrayList<StatusComment> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.commentlistviewrows, null);
			}
			StatusComment o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				final ImageView pic = (ImageView) v.findViewById(R.id.statusIcon);
				if (tt != null) {
					tt.setText(o.getCommentName());
				}
				if (bt != null) {
					bt.setText(o.getCommentMessage());
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getCommentIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Refreshes the comment list for the current status.
	 */
	public void refreshList() {
		m_adapter.clear();
		viewStatuses = new Runnable() {
			@Override
			public void run() {
				getComments();
			}
		};
		Thread thread = new Thread(null, viewStatuses, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(CommentList.this, "Please Wait", "Retrieving Comments", true);
	}

	/**
	 * Opens the profile of the acted upon user.
	 */
	public void OpenProfile() {
		Intent myIntent = new Intent(this, Profile.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Click listener for the refresh button.
	 */
	private OnClickListener newRefreshListener = new OnClickListener() {
		public void onClick(View v) {
			refreshList();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		Intent myIntent = new Intent(this, CommentDialog.class);
		startActivityForResult(myIntent, 0);
	}
}