package com.main.campustream;

import java.util.ArrayList;
import java.util.Calendar;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TimePicker;
import android.widget.Toast;

/**
 * @author Bryan Crabtree
 * 
 */
public class EventsDialog extends Activity implements OnClickListener {
	String[] news_categories = { "8", "9", "10", "11", "12", "13" };
	String[] locations = { "Air Force and Aerospace Studies", "Alden Memorial", "Alumni Gym", "Atwater Kent Laboratories", "Bartlett Center", "Boynton Hall",
			"Campus Center", "Campus Police", "Collegiate Religious Center", "20 Trowbridge Road", "Fuller Laboratories",
			"WPI Life Sciences and Bioengineering Center", "Goddard Hall", "Gordon Library", "Harrington Auditorium", "Health Center", "Higgins House",
			"Higgins Laboratories", "Hughes House", "International House", "Jeppson House", "Kaven Hall", "Oasis House", "Olin Hall", "Plant Services",
			"Power House", "Project Center", "Salisbury Laboratories", "Skull Tomb", "Stratton Hall", "Student Development and Counseling Center",
			"Washburn Shops", "Stoddard Laboratories", "85 Prescott Street", "Daniels Hall", "16 Elbridge", "Ellsworth Apartments", "Founders Hall",
			"Fuller Apartments", "Unity House", "Institute Hall", "Morgan Hall", "Sanford Riley Hall", "22 Schussler", "Stoddard Complex", "25 Trowbridge",
			"East Hall", "Alpha Chi Rho", "Alpha Gamma Delta", "Alpha Tau Omega", "Alpha Xi Delta", "Lambda Chi Alpha", "Phi Gamma Delta", "Phi Kappa Theta",
			"Phi Sigma Kappa", "Phi Sigma Sigma", "SigmaAlpha Epsilon", "Sigma Phi Epsilon", "Sigma Pi", "Tau Kappa Epsilon", "Theta Chi", "Zeta Psi",
			"Goat's Head" };

	private int startMinute;
	private int startHour;
	private int endMinute;
	private int endHour;
	private int startDay;
	private int startMonth;
	private int startYear;
	private int endDay;
	private int endMonth;
	private int endYear;
	static final int START_DIALOG_ID = 0;
	static final int END_DIALOG_ID = 1;
	static final int START_DATE_DIALOG_ID = 2;
	static final int END_DATE_DIALOG_ID = 3;
	EditText startTime;
	EditText endTime;
	EditText startDate;
	EditText endDate;

	String startAMPM = "";
	String endAMPM = "";
	String formattedStartHour = "";
	String formattedStartMinute = "";
	String formattedEndHour = "";
	String formattedEndMinute = "";
	String formattedStartDate = "";
	String formattedEndDate = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.eventsdialog);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		Button submitButton = (Button) findViewById(R.id.submitButton);
		submitButton.setOnClickListener(submitListener);

		Button cancelButton = (Button) findViewById(R.id.cancelButton);
		cancelButton.setOnClickListener(cancelListener);

		ArrayAdapter<String> completeAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, locations);
		AutoCompleteTextView buildingView = (AutoCompleteTextView) findViewById(R.id.buildingField);
		buildingView.setAdapter(completeAdapter);

		Spinner spinner = (Spinner) findViewById(R.id.categoryField);
		ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.events_array, android.R.layout.simple_spinner_item);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinner.setAdapter(adapter);

		startTime = (EditText) findViewById(R.id.startTimeField);
		startTime.setOnClickListener(startTimeListener);

		endTime = (EditText) findViewById(R.id.endTimeField);
		endTime.setOnClickListener(endTimeListener);

		startDate = (EditText) findViewById(R.id.startDateField);
		startDate.setOnClickListener(startDateListener);

		endDate = (EditText) findViewById(R.id.endDateField);
		endDate.setOnClickListener(endDateListener);

		final Calendar c = Calendar.getInstance();
		startMinute = c.get(Calendar.MINUTE);
		startHour = c.get(Calendar.HOUR_OF_DAY);
		endMinute = startMinute;
		endHour = startHour;
		startDay = c.get(Calendar.DAY_OF_MONTH);
		startMonth = c.get(Calendar.MONTH);
		startYear = c.get(Calendar.YEAR);
		endDay = startDay;
		endMonth = startMonth;
		endYear = startYear;

		updateStartTimeDisplay();
		updateEndTimeDisplay();
		updateStartDateDisplay();
		updateEndDateDisplay();
	}

	/**
	 * Time picker for the starting time of the event.
	 */
	private TimePickerDialog.OnTimeSetListener mStartTimeSetListener = new TimePickerDialog.OnTimeSetListener() {
		public void onTimeSet(TimePicker view, int hourOfDay, int minute) {
			startHour = hourOfDay;
			startMinute = minute;
			updateStartTimeDisplay();
		}
	};

	/**
	 * Time picker for the ending time of the event.
	 */
	private TimePickerDialog.OnTimeSetListener mEndTimeSetListener = new TimePickerDialog.OnTimeSetListener() {
		public void onTimeSet(TimePicker view, int hourOfDay, int minute) {
			endHour = hourOfDay;
			endMinute = minute;
			updateEndTimeDisplay();
		}
	};

	/**
	 * Date picker for the starting date of the event.
	 */
	private DatePickerDialog.OnDateSetListener mStartDateSetListener = new DatePickerDialog.OnDateSetListener() {
		public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
			startYear = year;
			startMonth = monthOfYear;
			startDay = dayOfMonth;
			updateStartDateDisplay();
		}
	};

	/**
	 * Data picker for the ending date of the event.
	 */
	private DatePickerDialog.OnDateSetListener mEndDateSetListener = new DatePickerDialog.OnDateSetListener() {
		public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
			endYear = year;
			endMonth = monthOfYear;
			endDay = dayOfMonth;
			updateEndDateDisplay();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreateDialog(int)
	 */
	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
			case START_DIALOG_ID:
				return new TimePickerDialog(this, mStartTimeSetListener, startHour, startMinute, false);
			case END_DIALOG_ID:
				return new TimePickerDialog(this, mEndTimeSetListener, endHour, endMinute, false);
			case START_DATE_DIALOG_ID:
				return new DatePickerDialog(this, mStartDateSetListener, startYear, startMonth, startDay);
			case END_DATE_DIALOG_ID:
				return new DatePickerDialog(this, mEndDateSetListener, endYear, endMonth, endDay);
		}
		return null;
	}

	/**
	 * Click listener for the submit button.
	 */
	private OnClickListener submitListener = new OnClickListener() {
		public void onClick(View v) {
			submitNews();
			finish();
		}
	};

	/**
	 * Click listener for the cancel button.
	 */
	private OnClickListener cancelListener = new OnClickListener() {
		public void onClick(View v) {
			finish();
		}
	};

	/**
	 * Click listener for the start time text field.
	 */
	private OnClickListener startTimeListener = new OnClickListener() {
		public void onClick(View v) {
			showDialog(START_DIALOG_ID);
		}
	};

	/**
	 * Click listener for the end time text field.
	 */
	private OnClickListener endTimeListener = new OnClickListener() {
		public void onClick(View v) {
			showDialog(END_DIALOG_ID);
		}
	};

	/**
	 * Click listener for the start date text field.
	 */
	private OnClickListener startDateListener = new OnClickListener() {
		public void onClick(View v) {
			showDialog(START_DATE_DIALOG_ID);
		}
	};

	/**
	 * Click listener for the end date text field.
	 */
	private OnClickListener endDateListener = new OnClickListener() {
		public void onClick(View v) {
			showDialog(END_DATE_DIALOG_ID);
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
	}

	/**
	 * Sets the start time text field to the selected time.
	 */
	private void updateStartTimeDisplay() {
		String ampm = "am";
		int startHourFormatted = startHour;
		if (startHour >= 12) {
			ampm = "pm";
			startHourFormatted = startHourFormatted - 12;
		}
		if (startHourFormatted == 0) {
			startHourFormatted = 12;
		}
		startAMPM = ampm;
		formattedStartHour = Integer.toString(startHourFormatted);
		formattedStartMinute = (pad(startMinute));
		startTime.setText(new StringBuilder().append(startHourFormatted).append(" : ").append(pad(startMinute)).append(" " + ampm));
	}

	/**
	 * Sets the end time text field to the selected time.
	 */
	private void updateEndTimeDisplay() {
		String ampm = "am";
		int endHourFormatted = endHour;
		if (endHour >= 12) {
			ampm = "pm";
			endHourFormatted = endHourFormatted - 12;
		}
		if (endHourFormatted == 0) {
			endHourFormatted = 12;
		}
		endAMPM = ampm;
		formattedEndHour = Integer.toString(endHourFormatted);
		formattedEndMinute = (pad(endMinute));
		endTime.setText(new StringBuilder().append(endHourFormatted).append(" : ").append(pad(endMinute)).append(" " + ampm));
	}

	/**
	 * Sets the start date text field to the selected date.
	 */
	private void updateStartDateDisplay() {
		formattedStartDate = Integer.toString(startMonth + 1) + "/" + Integer.toString(startDay) + "/" + Integer.toString(startYear);
		startDate.setText(new StringBuilder().append(startMonth + 1).append(" - ").append(startDay).append(" - ").append(startYear));
	}

	/**
	 * Sets the end date text field to the selected date.
	 */
	private void updateEndDateDisplay() {
		formattedEndDate = Integer.toString(endMonth + 1) + "/" + Integer.toString(endDay) + "/" + Integer.toString(endYear);
		endDate.setText(new StringBuilder().append(endMonth + 1).append(" - ").append(endDay).append(" - ").append(endYear));
	}

	/**
	 * Formats time values for displaying.
	 * 
	 * @param c
	 * @return
	 */
	private static String pad(int c) {
		if (c >= 10)
			return String.valueOf(c);
		else
			return "0" + String.valueOf(c);
	}

	/**
	 * Submits a news or event item with the selected information.
	 */
	public void submitNews() {
		final EditText titleField = (EditText) findViewById(R.id.titleField);
		final EditText messageField = (EditText) findViewById(R.id.messageField);
		final EditText roomField = (EditText) findViewById(R.id.roomField);
		AutoCompleteTextView buildingField = (AutoCompleteTextView) findViewById(R.id.buildingField);
		final Spinner categoryField = (Spinner) findViewById(R.id.categoryField);
		String title = titleField.getText().toString().trim();
		String message = messageField.getText().toString().trim();
		String room = roomField.getText().toString().trim();
		String building = buildingField.getText().toString().trim();
		String category = news_categories[categoryField.getSelectedItemPosition()];
		String url = "http://api.campustream.com/1/news/create";
		HttpClient client = new DefaultHttpClient();
		HttpPost post = new HttpPost(url);
		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair("title", title));
		params.add(new BasicNameValuePair("content", message));
		params.add(new BasicNameValuePair("category", category));
		params.add(new BasicNameValuePair("type", "text"));
		params.add(new BasicNameValuePair("event_location", building));
		params.add(new BasicNameValuePair("event_location_area", room));
		params.add(new BasicNameValuePair("event_start_from_date", formattedStartDate));
		params.add(new BasicNameValuePair("event_start_from_hour", formattedStartHour));
		params.add(new BasicNameValuePair("event_start_from_minute", formattedStartMinute));
		params.add(new BasicNameValuePair("event_start_from_ampm", startAMPM));
		params.add(new BasicNameValuePair("event_start_to_date", formattedEndDate));
		params.add(new BasicNameValuePair("event_start_to_hour", formattedEndHour));
		params.add(new BasicNameValuePair("event_start_to_minute", formattedEndMinute));
		params.add(new BasicNameValuePair("event_start_to_ampm", endAMPM));
		params.add(new BasicNameValuePair("access_token", Campustream.accessToken));
		try {
			UrlEncodedFormEntity ent = new UrlEncodedFormEntity(params);
			post.setEntity(ent);
			client.execute(post);
		} catch (Exception e) {
			Toast.makeText(this, e.toString(), Toast.LENGTH_LONG).show();
			e.printStackTrace();
		}
	}

}