package com.main.campustream;

import java.util.ArrayList;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * @author Bryan Crabtree
 * 
 */
public class FriendDialog extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<Friend> m_friends = null;
	private FriendAdapter m_adapter;
	private Runnable viewFriends;

	int numFriends = 0;
	String friendList = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.frienddialog);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		m_friends = new ArrayList<Friend>();
		this.m_adapter = new FriendAdapter(this, R.layout.friendslistviewrows, m_friends);
		setListAdapter(this.m_adapter);

		Button searchButton = (Button) findViewById(R.id.friendSearchButton);
		searchButton.setOnClickListener(searchListener);

		Button clearButton = (Button) findViewById(R.id.friendClearButton);
		clearButton.setOnClickListener(clearListener);

		Button cancelButton = (Button) findViewById(R.id.friendCancelButton);
		cancelButton.setOnClickListener(cancelListener);

		ListView friendsList = getListView();
		friendsList.setTextFilterEnabled(true);
		friendsList.setOnItemClickListener(friendsListListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreateDialog(int)
	 */
	protected Dialog onCreateDialog(int position) {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "Add Friend", "View Profile", "Cancel" };

		builder.setTitle(m_friends.get(position).getFriendName());
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					addFriend();
					newFriendToast();
				} else if (item == 1) {
					OpenProfile();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Click listener for the search button.
	 */
	private OnClickListener searchListener = new OnClickListener() {
		public void onClick(View v) {
			InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.hideSoftInputFromWindow(findViewById(R.id.searchField).getWindowToken(), 0);

			final EditText searchField = (EditText) findViewById(R.id.searchField);

			if (searchField.getText().toString().trim().length() < 3) {
				errorToast();
				searchField.setText("");
			} else {
				m_adapter.clear();
				searchField.setText(searchField.getText().toString().trim().toLowerCase());
				viewFriends = new Runnable() {
					@Override
					public void run() {
						friendSearch(searchField.getText().toString());
					}
				};
				Thread thread = new Thread(null, viewFriends, "MagentoBackground");
				thread.start();
				m_ProgressDialog = ProgressDialog.show(FriendDialog.this, "Please Wait", "Searching For User", true);
			}
		}
	};

	/**
	 * Click listener for the clear button.
	 */
	private OnClickListener clearListener = new OnClickListener() {
		public void onClick(View v) {
			m_adapter.clear();

			final EditText searchField = (EditText) findViewById(R.id.searchField);
			searchField.setText("");

			final TextView resultView = (TextView) findViewById(R.id.friendSearchResults);
			resultView.setText("Matching Results: 0");
		}
	};

	/**
	 * Click listener for the cancel button.
	 */
	private OnClickListener cancelListener = new OnClickListener() {
		public void onClick(View v) {
			finish();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {

	}

	/**
	 * Creates an Android toast event.
	 */
	public void errorToast() {
		Toast.makeText(this, "Invalid username.", Toast.LENGTH_LONG).show();
	}

	/**
	 * Creates an Android toast event.
	 */
	public void newFriendToast() {
		Toast.makeText(this, ("You are now following: " + Campustream.currentProfileUsername), Toast.LENGTH_LONG).show();
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_friends != null && m_friends.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_friends.size(); i++)
					m_adapter.add(m_friends.get(i));
			}
			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	private class FriendAdapter extends ArrayAdapter<Friend> {

		private ArrayList<Friend> items;

		public FriendAdapter(Context context, int textViewResourceId, ArrayList<Friend> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.friendslistviewrows, null);
			}
			Friend o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				final ImageView pic = (ImageView) v.findViewById(R.id.friendIcon);
				if (tt != null) {
					tt.setText(o.getFriendName());
				}
				if (bt != null) {
					bt.setText("Username: " + o.getFriendUser());
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getFriendIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Searches for a user of the specified name (Partial Matching).
	 * 
	 * @param friendUsername
	 */
	private void friendSearch(String friendUsername) {
		try {
			String friendName = "";
			String friendUser = "";
			String friendIcon = "";
			String friendStatus = "";
			m_friends = new ArrayList<Friend>();
			TextView resultView = (TextView) findViewById(R.id.friendSearchResults);
			friendList = APIs.FindFriends(friendUsername);
			if (friendList.length() > 10) {

				JSONArray results = new JSONArray(friendList);
				numFriends = results.length();

				for (int x = 0; x < numFriends; x++) {
					Friend currentFriend = new Friend();

					JSONObject jsonFriend = results.getJSONObject(x);

					friendName = jsonFriend.getString("name");
					friendUser = jsonFriend.getString("username");

					friendIcon = jsonFriend.getString("avatar");
					friendIcon = friendIcon.replaceFirst("medium", "small");

					currentFriend.setFriendName(friendName);
					currentFriend.setFriendUser(friendUser);
					currentFriend.setFriendIcon(friendIcon);
					currentFriend.setFriendStatus(friendStatus);

					m_friends.add(currentFriend);
				}

				resultView.setText("Matching Results: " + Integer.toString(numFriends));
			} else {
				resultView.setText("Matching Results: 0");
			}

		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener friendsListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			Campustream.currentProfileUsername = m_friends.get(position).getFriendUser();
			onCreateDialog(position);
		}
	};

	/**
	 * Opens the profile of the acted upon user.
	 */
	public void OpenProfile() {
		Intent myIntent = new Intent(this, Profile.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Sets the default user to follow the acted upon user.
	 */
	public void addFriend() {
		String url = "http://api.campustream.com/1/friendships/create";
		HttpClient client = new DefaultHttpClient();
		HttpPost post = new HttpPost(url);
		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair("username", Campustream.currentProfileUsername));
		params.add(new BasicNameValuePair("access_token", Campustream.accessToken));
		try {
			UrlEncodedFormEntity ent = new UrlEncodedFormEntity(params);
			post.setEntity(ent);
			client.execute(post);
		} catch (Exception e) {
			Toast.makeText(this, e.toString(), Toast.LENGTH_LONG).show();
			e.printStackTrace();
		}
	}
}