package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class NewsList extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<NewsItem> m_news = null;
	private NewsAdapter m_adapter;
	private Runnable viewNews;

	int numNews = 0;
	String newsList = "";
	String newsVoteID = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.newslistview);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		m_news = new ArrayList<NewsItem>();
		this.m_adapter = new NewsAdapter(this, R.layout.newslistviewrows, m_news);
		setListAdapter(this.m_adapter);

		Button newsNewButton = (Button) findViewById(R.id.newsNewButton);
		newsNewButton.setOnClickListener(this);

		Button newsRefreshButton = (Button) findViewById(R.id.newsRefreshButton);
		newsRefreshButton.setOnClickListener(refreshListener);

		TextView categoryField = (TextView) findViewById(R.id.categoryField);
		categoryField.setText(Campustream.newsCategoryTitle);

		ListView newsList = getListView();
		newsList.setTextFilterEnabled(true);
		newsList.setOnItemClickListener(newsListListener);
		newsList.setOnItemLongClickListener(voteListListener);

		viewNews = new Runnable() {
			@Override
			public void run() {
				getNews();
			}
		};
		Thread thread = new Thread(null, viewNews, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(NewsList.this, "Please Wait", "Retrieving Listings", true);
	}

	/**
	 * The dialog created when long-pressing a news or event item.
	 * 
	 * @return
	 */
	protected Dialog onCreateDialog() {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "Vote Listing Up", "Vote Listing Down", "Cancel" };

		builder.setTitle("Select Action");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					APIs.rateNews("up", newsVoteID);
					refreshList();
				} else if (item == 1) {
					APIs.rateNews("down", newsVoteID);
					refreshList();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_news != null && m_news.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_news.size(); i++)
					m_adapter.add(m_news.get(i));
			}
			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * Retrieves the news or event items in the current category.
	 */
	private void getNews() {
		try {
			String newsTitle = "";
			String newsMessage = "";
			String newsUser = "";
			String newsUsername = "";
			String newsResponses = "";
			String newsVotes = "";
			String newsID = "";
			String newsIcon = "";
			String newsUserID = "";
			m_news = new ArrayList<NewsItem>();
			if (Campustream.newsCategory.equals("all")) {
				newsList = APIs.GetNewsList(Campustream.newsSection);
			} else {
				newsList = APIs.GetNewsListSection(Campustream.newsSection, Campustream.newsCategory);
			}
			JSONArray news = new JSONArray(newsList);
			numNews = news.length();

			for (int x = 0; x < numNews; x++) {
				NewsItem currentNewsItem = new NewsItem();

				JSONObject jsonNews = news.getJSONObject(x);
				newsUserID = jsonNews.getString("posted_by");

				JSONObject jsonProfile = new JSONObject();
				jsonProfile = new JSONObject(APIs.GetUserWithID(newsUserID));
				jsonProfile = jsonProfile.getJSONObject("target");
				newsUser = jsonProfile.getString("name");
				newsUsername = jsonProfile.getString("username");
				newsIcon = jsonProfile.getString("avatar");
				newsIcon = newsIcon.replaceFirst("medium", "small");

				newsTitle = jsonNews.getString("title");
				newsMessage = jsonNews.getString("post");
				newsResponses = jsonNews.getString("num_comments");
				newsID = jsonNews.getString("id");
				newsVotes = jsonNews.getString("votes");

				newsUser = Campustream.correctString(newsUser);
				newsTitle = Campustream.correctString(newsTitle);
				newsMessage = Campustream.correctString(newsMessage);

				currentNewsItem.setNewsTitle(newsTitle);
				currentNewsItem.setNewsMessage(newsMessage);
				currentNewsItem.setNewsUser(newsUser);
				currentNewsItem.setNewsUsername(newsUsername);
				currentNewsItem.setNewsResponses(newsResponses);
				currentNewsItem.setNewsID(newsID);
				currentNewsItem.setNewsVotes(newsVotes);
				currentNewsItem.setNewsIcon(newsIcon);

				m_news.add(currentNewsItem);
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class NewsAdapter extends ArrayAdapter<NewsItem> {

		private ArrayList<NewsItem> items;

		public NewsAdapter(Context context, int textViewResourceId, ArrayList<NewsItem> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.newslistviewrows, null);
			}
			NewsItem o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				TextView vt = (TextView) v.findViewById(R.id.votetext);
				final ImageView pic = (ImageView) v.findViewById(R.id.newsIcon);
				if (tt != null) {
					tt.setText(o.getNewsTitle());
				}
				if (bt != null) {
					if (o.getNewsResponses().equals("null")) {
						bt.setText(o.getNewsUser() + " : 0 comments");
					} else {
						if (Integer.parseInt(o.getNewsResponses()) == 1) {
							bt.setText(o.getNewsUser() + " : " + o.getNewsResponses() + " Comment");
						} else {
							bt.setText(o.getNewsUser() + " : " + o.getNewsResponses() + " Comments");
						}
					}
				}
				if (vt != null) {
					vt.setText(o.getNewsVotes());
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getNewsIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Click listener for the refresh button.
	 */
	private OnClickListener refreshListener = new OnClickListener() {
		public void onClick(View v) {
			refreshList();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		SubmitNews();
	}

	/**
	 * Refreshes the list.
	 */
	public void refreshList() {
		m_adapter.clear();
		viewNews = new Runnable() {
			@Override
			public void run() {
				getNews();
			}
		};
		Thread thread = new Thread(null, viewNews, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(NewsList.this, "Please Wait", "Retrieving Listings", true);
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener newsListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			Campustream.newsID = m_news.get(position).getNewsID();
			Campustream.newsTitle = m_news.get(position).getNewsTitle();
			Campustream.newsMessage = m_news.get(position).getNewsMessage();
			Campustream.newsAsker = m_news.get(position).getNewsUser();
			OpenNews();
		}
	};

	/**
	 * Long press listener for the list view.
	 */
	public OnItemLongClickListener voteListListener = new OnItemLongClickListener() {
		public boolean onItemLongClick(AdapterView<?> a, View v, int position, long id) {
			onCreateDialog();
			newsVoteID = m_news.get(position).getNewsID();
			return false;
		}
	};

	/**
	 * Opens the acted upon news or event item.
	 */
	public void OpenNews() {
		Intent myIntent = new Intent(this, NewsPage.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Opens a dialog for submitting news or event items.
	 */
	public void SubmitNews() {
		if (Campustream.newsSection.equals("news")) {
			Intent myIntent = new Intent(this, NewsDialog.class);
			startActivityForResult(myIntent, 0);
		} else {
			Intent myIntent = new Intent(this, EventsDialog.class);
			startActivityForResult(myIntent, 0);
		}
	}
}