package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.util.Linkify;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class NewsPage extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<Comment> m_comments = null;
	private CommentAdapter m_adapter;
	private Runnable viewQuestions;

	int numComments = 0;
	String commentList = "";

	String temporaryID = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.newspagelistview);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		ListView questionList = getListView();

		View header = (View) getLayoutInflater().inflate(R.layout.pageheader, null);
		questionList.addHeaderView(header);

		m_comments = new ArrayList<Comment>();
		this.m_adapter = new CommentAdapter(this, R.layout.questionlistviewrows, m_comments);
		setListAdapter(this.m_adapter);

		Button newsCommentButton = (Button) findViewById(R.id.newsCommentButton);
		newsCommentButton.setOnClickListener(this);

		Button newsRefreshButton = (Button) findViewById(R.id.newsRefreshButton);
		newsRefreshButton.setOnClickListener(refreshListener);

		questionList.setTextFilterEnabled(true);
		questionList.setOnItemClickListener(questionListListener);

		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(NewsPage.this, "Please Wait", "Retrieving Comments", true);
	}

	/**
	 * Dialog created upon selected a response to a news or event item.
	 * 
	 * @return
	 */
	protected Dialog onCreateDialog() {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "Comment", "View User Profile", "Cancel" };

		builder.setTitle("Select Action");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					Campustream.newsReply = temporaryID;
					addComment();
				} else if (item == 1) {
					OpenProfile();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_comments != null && m_comments.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_comments.size(); i++)
					m_adapter.add(m_comments.get(i));
			}

			TextView titleField = (TextView) findViewById(R.id.titleField);
			TextView messageField = (TextView) findViewById(R.id.messageField);
			TextView posterField = (TextView) findViewById(R.id.posterField);

			titleField.setText(Campustream.newsTitle);
			messageField.setText(Campustream.removeTags(Campustream.newsMessage));
			posterField.setText("Posted by: " + Campustream.newsAsker);

			android.text.util.Linkify.addLinks(messageField, Linkify.ALL);

			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * Retrieves the responses to a news or event item.
	 */
	private void getQuestions() {
		try {
			m_comments = new ArrayList<Comment>();
			commentList = APIs.GetNewsComments(Campustream.newsID);
			JSONObject news = new JSONObject(commentList);
			JSONArray comments = news.getJSONArray("comments");
			numComments = comments.length();

			for (int x = 0; x < numComments; x++) {
				JSONObject jsonComment = comments.getJSONObject(x);
				m_comments.add(getComments(jsonComment));
				JSONArray jsonChild = null;
				try {
					jsonChild = jsonComment.getJSONArray("children");
				} catch (Exception e) {
				}
				if ((jsonChild != null) && (jsonChild.length() > 0)) {
					checkChildren(jsonChild);
				}
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * Checks for nested responses.
	 * 
	 * @param jsonChild
	 */
	private void checkChildren(JSONArray jsonChild) {
		try {
			int numKids = jsonChild.length();
			for (int x = 0; x < numKids; x++) {
				JSONObject jsonComment = jsonChild.getJSONObject(x);
				m_comments.add(getComments(jsonComment));
				JSONArray jsonKid = null;
				try {
					jsonKid = jsonComment.getJSONArray("children");
				} catch (Exception e) {
				}
				if ((jsonKid != null) && (jsonKid.length() > 0)) {
					checkChildren(jsonKid);
				}
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Retrieves the responses to a news or event item.
	 * 
	 * @param jsonComment
	 * @return
	 */
	private Comment getComments(JSONObject jsonComment) {
		try {
			String commentMessage = "";
			String commentID = "";
			String commentUser = "";
			String commentUsername = "";
			String commentIcon = "";

			Comment currentComment = new Comment();

			commentMessage = jsonComment.getString("content");
			commentID = jsonComment.getString("id");
			commentUser = jsonComment.getString("posted_by");

			JSONObject jsonProfile = new JSONObject();
			jsonProfile = new JSONObject(APIs.GetUserWithID(commentUser));
			jsonProfile = jsonProfile.getJSONObject("target");
			commentUser = jsonProfile.getString("name");
			commentUsername = jsonProfile.getString("username");
			commentIcon = jsonProfile.getString("avatar");
			commentIcon = commentIcon.replaceFirst("medium", "small");

			commentUser = Campustream.correctString(commentUser);
			commentMessage = Campustream.correctString(commentMessage);

			currentComment.setCommentMessage(commentMessage);
			currentComment.setCommentID(commentID);
			currentComment.setCommentUser(commentUser);
			currentComment.setCommentUsername(commentUsername);
			currentComment.setCommentIcon(commentIcon);

			return currentComment;
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
		return null;
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class CommentAdapter extends ArrayAdapter<Comment> {

		private ArrayList<Comment> items;

		public CommentAdapter(Context context, int textViewResourceId, ArrayList<Comment> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.questionlistviewrows, null);
			}
			Comment o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				final ImageView pic = (ImageView) v.findViewById(R.id.questionIcon);
				if (tt != null) {
					tt.setText(o.getCommentMessage());
				}
				if (bt != null) {
					bt.setText("Posted by: " + o.getCommentUser());
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getCommentIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Click listener for the refresh button.
	 */
	private OnClickListener refreshListener = new OnClickListener() {
		public void onClick(View v) {
			refreshList();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		Campustream.newsReply = "";
		Intent myIntent = new Intent(this, NewsCommentDialog.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Refreshes the list.
	 */
	public void refreshList() {
		m_adapter.clear();
		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(NewsPage.this, "Please Wait", "Retrieving Comments", true);
	}

	/**
	 * Opens the profile of the acted upon user.
	 */
	public void OpenProfile() {
		Intent myIntent = new Intent(this, Profile.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener questionListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			position--;
			Campustream.currentProfileUsername = m_comments.get(position).getCommentUsername();
			temporaryID = m_comments.get(position).getCommentID();
			onCreateDialog();
		}
	};

	/**
	 * Opens a dialog to add a response to the acted upon item.
	 */
	public void addComment() {
		Intent myIntent = new Intent(this, NewsCommentDialog.class);
		startActivityForResult(myIntent, 0);
	}
}