package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class Profile extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<StreamStatus> m_statuses = null;
	private StatusAdapter m_adapter;
	private Runnable viewStatuses;

	int numStatuses = 0;
	String statusList = "";

	public static String profileURL = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.profile);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		ListView streamList = getListView();

		View header = (View) getLayoutInflater().inflate(R.layout.profileheader, null);
		streamList.addHeaderView(header);

		m_statuses = new ArrayList<StreamStatus>();
		this.m_adapter = new StatusAdapter(this, R.layout.streamlistviewrows, m_statuses);
		setListAdapter(this.m_adapter);

		if (Campustream.currentProfileUsername.length() <= 0) {
			finish();
		}

		ImageView iconField = (ImageView) findViewById(R.id.iconField);
		TextView nameField = (TextView) findViewById(R.id.nameField);
		TextView usernameField = (TextView) findViewById(R.id.usernameField);
		EditText emailField = (EditText) findViewById(R.id.emailField);
		EditText websiteField = (EditText) findViewById(R.id.websiteField);
		EditText bioField = (EditText) findViewById(R.id.bioField);
		EditText yearField = (EditText) findViewById(R.id.yearField);
		EditText majorField = (EditText) findViewById(R.id.majorField);
		EditText minorField = (EditText) findViewById(R.id.minorField);
		String avatarURL = "";

		try {
			JSONObject jsonProfile = new JSONObject(APIs.GetUserProfile(Campustream.currentProfileUsername));
			nameField.setText(Campustream.correctString(jsonProfile.getString("name")));
			usernameField.setText(Campustream.correctString(jsonProfile.getString("username")) + "@wpi.edu");
			emailField.setText(Campustream.correctString(jsonProfile.getString("username")) + "@wpi.edu");
			websiteField.setText(Campustream.correctString(jsonProfile.getString("website")));
			bioField.setText(Campustream.correctString(jsonProfile.getString("bio")));
			yearField.setText(Campustream.correctString(jsonProfile.getString("class_year")));
			majorField.setText(Campustream.correctString(jsonProfile.getString("college_majors")));
			minorField.setText(Campustream.correctString(jsonProfile.getString("college_minors")));
			avatarURL = jsonProfile.getString("avatar");
			avatarURL = avatarURL.replaceFirst("medium", "small");
		} catch (JSONException e) {
			e.printStackTrace();
		}

		Drawable picture = null;

		while (picture == null) {
			picture = GetUserInfo.GetAvatarIcon(avatarURL);
		}

		iconField.setImageDrawable(picture);

		streamList.setTextFilterEnabled(true);
		streamList.setOnItemClickListener(streamListListener);

		viewStatuses = new Runnable() {
			@Override
			public void run() {
				getStatuses();
			}
		};
		Thread thread = new Thread(null, viewStatuses, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(Profile.this, "Please Wait", "Retrieving User Activity", true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreateDialog(int)
	 */
	protected Dialog onCreateDialog(int position) {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "Add Comment", "View Comments", "Cancel" };

		builder.setTitle(m_statuses.get(position).getStatusName());
		Campustream.statusID = m_statuses.get(position).getStatusID();
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					NewComment();
				} else if (item == 1) {
					OpenComments();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_statuses != null && m_statuses.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_statuses.size(); i++)
					m_adapter.add(m_statuses.get(i));
			}
			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
			Campustream.currentProfileUsername = "";
		}
	};

	/**
	 * Retrieves the user's 50 most recent status updates.
	 */
	private void getStatuses() {
		try {
			String statusName = "";
			String statusUser = "";
			String statusIcon = "";
			String statusMessage = "";
			String statusComments = "";
			String statusType = "";
			String statusID = "";
			m_statuses = new ArrayList<StreamStatus>();
			statusList = APIs.GetUserActivity(Campustream.currentProfileUsername);
			JSONArray followers = new JSONArray(statusList);
			numStatuses = followers.length();

			for (int x = 0; x < numStatuses; x++) {
				StreamStatus currentStatus = new StreamStatus();

				JSONObject jsonStatus = followers.getJSONObject(x);
				JSONObject jsonUser = jsonStatus.getJSONObject("user");

				statusName = jsonUser.getString("name");
				statusUser = jsonUser.getString("username");
				statusIcon = jsonUser.getString("avatar");
				statusMessage = jsonStatus.getString("message");
				statusComments = jsonStatus.getString("num_comments");
				statusType = jsonStatus.getString("type");
				statusID = jsonStatus.getString("id");
				statusIcon = statusIcon.replaceFirst("medium", "small");

				statusName = Campustream.correctString(statusName);
				statusMessage = Campustream.correctString(statusMessage);

				currentStatus.setStatusName(statusName);
				currentStatus.setStatusUser(statusUser);
				currentStatus.setStatusIcon(statusIcon);
				currentStatus.setStatusMessage(statusMessage);
				currentStatus.setStatusComments(statusComments);
				currentStatus.setStatusType(statusType);
				currentStatus.setStatusID(statusID);

				m_statuses.add(currentStatus);
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class StatusAdapter extends ArrayAdapter<StreamStatus> {

		private ArrayList<StreamStatus> items;

		public StatusAdapter(Context context, int textViewResourceId, ArrayList<StreamStatus> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.streamlistviewrows, null);
			}
			StreamStatus o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				TextView lt = (TextView) v.findViewById(R.id.lowertext);
				final ImageView pic = (ImageView) v.findViewById(R.id.statusIcon);
				if (tt != null) {
					tt.setText(o.getStatusName());
				}
				if (bt != null) {
					if (o.getStatusType().equals("question")) {
						bt.setText("Asked: " + o.getStatusMessage());
					} else if (o.getStatusType().equals("newsevent")) {
						bt.setText("Posted: " + o.getStatusMessage());
					} else if (o.getStatusType().equals("response")) {
						bt.setText("Replied: " + o.getStatusMessage());
					} else {
						bt.setText(o.getStatusMessage());
					}
				}
				if (lt != null) {
					if (o.getStatusComments().equals("1")) {
						lt.setText(o.getStatusComments() + " Comment");
					} else {
						lt.setText(o.getStatusComments() + " Comments");
					}
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getStatusIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener streamListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			position--;
			Campustream.currentProfileUsername = m_statuses.get(position).getStatusUser();
			onCreateDialog(position);
		}
	};

	/**
	 * Opens the comment dialog.
	 */
	public void NewComment() {
		Intent myIntent = new Intent(this, CommentDialog.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Views the comments of the acted upon status update.
	 */
	public void OpenComments() {
		Intent myIntent = new Intent(this, CommentList.class);
		startActivityForResult(myIntent, 0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.content.DialogInterface.OnClickListener#onClick(android.content.DialogInterface, int)
	 */
	@Override
	public void onClick(DialogInterface dialog, int which) {
	}
}