package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.util.Linkify;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class QuestionPage extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<Response> m_responses = null;
	private ResponseAdapter m_adapter;
	private Runnable viewQuestions;

	int numResponses = 0;
	String questionList = "";

	String temporaryID = "";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.questionpagelistview);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		ListView questionList = getListView();

		View header = (View) getLayoutInflater().inflate(R.layout.pageheader, null);
		questionList.addHeaderView(header);

		m_responses = new ArrayList<Response>();
		this.m_adapter = new ResponseAdapter(this, R.layout.questionlistviewrows, m_responses);
		setListAdapter(this.m_adapter);

		Button questionReplyButton = (Button) findViewById(R.id.questionReplyButton);
		questionReplyButton.setOnClickListener(this);

		Button questionRefreshButton = (Button) findViewById(R.id.questionRefreshButton);
		questionRefreshButton.setOnClickListener(refreshListener);

		questionList.setTextFilterEnabled(true);
		questionList.setOnItemClickListener(questionListListener);

		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(QuestionPage.this, "Please Wait", "Retrieving Question Responses", true);
	}

	/**
	 * Dialog created when a question response is acted upon.
	 * 
	 * @return
	 */
	protected Dialog onCreateDialog() {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "View User Profile", "Vote Response Up", "Vote Response Down", "Cancel" };

		builder.setTitle("Select Action");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					OpenProfile();
				} else if (item == 1) {
					APIs.rateQuestion("up", temporaryID);
					refreshList();
				} else if (item == 2) {
					APIs.rateQuestion("down", temporaryID);
					refreshList();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_responses != null && m_responses.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_responses.size(); i++)
					m_adapter.add(m_responses.get(i));
			}

			TextView titleField = (TextView) findViewById(R.id.titleField);
			TextView questionField = (TextView) findViewById(R.id.messageField);
			TextView askerField = (TextView) findViewById(R.id.posterField);

			titleField.setText(Campustream.questionTitle);
			questionField.setText(Campustream.questionMessage);
			askerField.setText("Asked by: " + Campustream.questionAsker);

			android.text.util.Linkify.addLinks(questionField, Linkify.ALL);

			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * Retrieves the responses to the current question.
	 */
	private void getQuestions() {
		try {
			String responseMessage = "";
			String responseID = "";
			String responseVote = "";
			String responseUser = "";
			String responseUsername = "";
			String responseIcon = "";

			m_responses = new ArrayList<Response>();
			questionList = APIs.GetQuestionResponses(Campustream.questionID);
			JSONObject question = new JSONObject(questionList);
			JSONArray responses = question.getJSONArray("responses");
			numResponses = responses.length();

			for (int x = 0; x < numResponses; x++) {
				Response currentResponse = new Response();

				JSONObject jsonResponse = responses.getJSONObject(x);

				responseMessage = jsonResponse.getString("message");
				responseID = jsonResponse.getString("id");
				responseVote = jsonResponse.getString("vote_count");
				responseUser = jsonResponse.getString("user_id");

				JSONObject jsonProfile = new JSONObject();
				jsonProfile = new JSONObject(APIs.GetUserWithID(responseUser));
				jsonProfile = jsonProfile.getJSONObject("target");
				responseUser = jsonProfile.getString("name");
				responseUsername = jsonProfile.getString("username");
				responseIcon = jsonProfile.getString("avatar");
				responseIcon = responseIcon.replaceFirst("medium", "small");

				responseUser = Campustream.correctString(responseUser);
				responseMessage = Campustream.correctString(responseMessage);

				currentResponse.setResponseMessage(responseMessage);
				currentResponse.setResponseID(responseID);
				currentResponse.setResponseVote(responseVote);
				currentResponse.setResponseUser(responseUser);
				currentResponse.setResponseUsername(responseUsername);
				currentResponse.setResponseIcon(responseIcon);

				m_responses.add(currentResponse);
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class ResponseAdapter extends ArrayAdapter<Response> {

		private ArrayList<Response> items;

		public ResponseAdapter(Context context, int textViewResourceId, ArrayList<Response> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.questionlistviewrows, null);
			}
			Response o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				final ImageView pic = (ImageView) v.findViewById(R.id.questionIcon);
				if (tt != null) {
					tt.setText(o.getResponseMessage());
				}
				if (bt != null) {
					if (Integer.parseInt(o.getResponseVote()) == 1) {
						bt.setText(o.getResponseUser() + " : " + o.getResponseVote() + " Vote");
					} else {
						bt.setText(o.getResponseUser() + " : " + o.getResponseVote() + " Votes");
					}
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getResponseIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Click listener for the refresh button.
	 */
	private OnClickListener refreshListener = new OnClickListener() {
		public void onClick(View v) {
			refreshList();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		Intent myIntent = new Intent(this, ResponseDialog.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Refreshes the list.
	 */
	public void refreshList() {
		m_adapter.clear();
		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(QuestionPage.this, "Please Wait", "Retrieving Question Responses", true);
	}

	/**
	 * Opens the profile of the acted upon user.
	 */
	public void OpenProfile() {
		Intent myIntent = new Intent(this, Profile.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener questionListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			position--;
			Campustream.currentProfileUsername = m_responses.get(position).getResponseUsername();
			temporaryID = m_responses.get(position).getResponseID();
			onCreateDialog();
		}
	};
}