package com.main.campustream;

import java.util.ArrayList;
import org.json.JSONArray;
import org.json.JSONObject;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * @author Bryan Crabtree
 * 
 */
public class QuestionsList extends ListActivity implements OnClickListener {

	private ProgressDialog m_ProgressDialog = null;
	private ArrayList<Question> m_questions = null;
	private QuestionAdapter m_adapter;
	private Runnable viewQuestions;

	int numQuestions = 0;
	String questionList = "";
	String sorting = "last_updated";

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.questionlistview);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		m_questions = new ArrayList<Question>();
		this.m_adapter = new QuestionAdapter(this, R.layout.questionlistviewrows, m_questions);
		setListAdapter(this.m_adapter);

		Button questionSortButton = (Button) findViewById(R.id.questionSortButton);
		questionSortButton.setOnClickListener(this);

		Button questionActionButton = (Button) findViewById(R.id.questionActionButton);
		questionActionButton.setOnClickListener(actionListener);

		TextView categoryField = (TextView) findViewById(R.id.categoryField);
		categoryField.setText(Campustream.questionCategoryTitle);

		ListView questionList = getListView();
		questionList.setTextFilterEnabled(true);
		questionList.setOnItemClickListener(questionListListener);

		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(QuestionsList.this, "Please Wait", "Retrieving Question List", true);
	}

	/**
	 * Dialog created when acting upon a collaboration item.
	 * 
	 * @return
	 */
	protected Dialog onCreateDialog() {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		final CharSequence[] items = { "Last Updated", "Most Popular", "Recently Asked", "Cancel" };

		builder.setTitle("Sort By...");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) {
				if (item == 0) {
					sorting = "last_updated";
					refreshList();
				} else if (item == 1) {
					sorting = "most_popular";
					refreshList();
				} else if (item == 2) {
					sorting = "newest";
					refreshList();
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();

		return dialog;
	}

	/**
	 * Dialog created upon pressing the "Action" button.
	 * 
	 * @return
	 */
	protected Dialog actionDialog() {
		Dialog dialog = null;
		AlertDialog.Builder actionBuilder = new AlertDialog.Builder(this);

		final CharSequence[] actions = { "Refresh List", "Ask Question", "Cancel" };

		actionBuilder.setTitle("Select Action");
		actionBuilder.setItems(actions, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int action) {
				if (action == 0) {
					refreshList();
				} else if (action == 1) {
					NewQuestion();
				}
			}
		});
		AlertDialog actionAlert = actionBuilder.create();
		actionAlert.show();

		return dialog;
	}

	/**
	 * Dismisses the progress dialog and updates the list.
	 */
	private Runnable returnRes = new Runnable() {

		@Override
		public void run() {
			if (m_questions != null && m_questions.size() > 0) {
				m_adapter.notifyDataSetChanged();
				for (int i = 0; i < m_questions.size(); i++)
					m_adapter.add(m_questions.get(i));
			}
			m_ProgressDialog.dismiss();
			m_adapter.notifyDataSetChanged();
		}
	};

	/**
	 * Retrieves a list of questions in the current category.
	 */
	private void getQuestions() {
		try {
			String questionTitle = "";
			String questionMessage = "";
			String questionUser = "";
			String questionUsername = "";
			String questionResponses = "";
			String questionID = "";
			String questionIcon = "";
			m_questions = new ArrayList<Question>();
			questionList = APIs.GetQuestionList(Campustream.questionSection, sorting);
			JSONArray questions = new JSONArray(questionList);
			numQuestions = questions.length();

			for (int x = 0; x < numQuestions; x++) {
				Question currentQuestion = new Question();

				JSONObject jsonQuestion = questions.getJSONObject(x);
				JSONObject jsonUser = jsonQuestion.getJSONObject("user");

				questionTitle = jsonQuestion.getString("title");
				questionMessage = jsonQuestion.getString("message");
				questionUser = jsonUser.getString("name");
				questionUsername = jsonUser.getString("username");
				questionResponses = jsonQuestion.getString("num_responses");
				questionID = jsonQuestion.getString("id");
				questionIcon = jsonUser.getString("avatar");
				questionIcon = questionIcon.replaceFirst("medium", "small");

				questionUser = Campustream.correctString(questionUser);
				questionTitle = Campustream.correctString(questionTitle);
				questionMessage = Campustream.correctString(questionMessage);

				currentQuestion.setQuestionTitle(questionTitle);
				currentQuestion.setQuestionMessage(questionMessage);
				currentQuestion.setQuestionUser(questionUser);
				currentQuestion.setQuestionUsername(questionUsername);
				currentQuestion.setQuestionResponses(questionResponses);
				currentQuestion.setQuestionID(questionID);
				currentQuestion.setQuestionIcon(questionIcon);

				m_questions.add(currentQuestion);
			}
		} catch (Exception e) {
		}
		runOnUiThread(returnRes);
	}

	/**
	 * @author Bryan Crabtree
	 * 
	 */
	public class QuestionAdapter extends ArrayAdapter<Question> {

		private ArrayList<Question> items;

		public QuestionAdapter(Context context, int textViewResourceId, ArrayList<Question> items) {
			super(context, textViewResourceId, items);
			this.items = items;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.questionlistviewrows, null);
			}
			Question o = items.get(position);
			if (o != null) {
				TextView tt = (TextView) v.findViewById(R.id.toptext);
				TextView bt = (TextView) v.findViewById(R.id.bottomtext);
				final ImageView pic = (ImageView) v.findViewById(R.id.questionIcon);
				if (tt != null) {
					tt.setText(o.getQuestionTitle());
				}
				if (bt != null) {
					if (Integer.parseInt(o.getQuestionResponses()) == 1) {
						bt.setText(o.getQuestionUser() + " : " + o.getQuestionResponses() + " Response");
					} else {
						bt.setText(o.getQuestionUser() + " : " + o.getQuestionResponses() + " Responses");
					}
				}
				if (pic != null) {
					Drawable picture = null;
					Boolean pictureSet = false;
					final String icon = o.getQuestionIcon();

					for (int y = 0; y < 1000; y++) {
						if (Campustream.profileAvatars[y] != null) {
							if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Ready"))) {
								picture = Campustream.profileIcons[y];
								pic.setImageDrawable(picture);
								pictureSet = true;
								y = 10000;
							} else if ((Campustream.profileAvatars[y].equals(icon)) && (Campustream.profileAvatarStatuses[y].equals("Downloading"))) {
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								final Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileAvatarStatuses[x].equals("Downloading")) {
											try {
												Thread.sleep(100);
											} catch (InterruptedException e) {
												e.printStackTrace();
											}
										}
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								pictureSet = true;
								y = 10000;
							}
						} else {
							y = 10000;
						}
					}

					if (pictureSet == false) {
						for (int y = 0; y < 1000; y++) {
							if (Campustream.profileAvatars[y] == null) {
								Campustream.profileAvatars[y] = icon;
								Campustream.profileAvatarStatuses[y] = "Downloading";
								final int x = y;
								pic.setImageDrawable(null);
								final Handler handler = new Handler() {
									@Override
									public void handleMessage(Message message) {
										pic.setImageDrawable((Drawable) message.obj);
									}
								};

								Thread thread = new Thread() {
									@Override
									public void run() {
										while (Campustream.profileIcons[x] == null) {
											Campustream.profileIcons[x] = GetUserInfo.GetAvatarIcon(icon);
										}
										Campustream.profileAvatarStatuses[x] = "Ready";
										final Drawable newPicture = Campustream.profileIcons[x];
										Message message = handler.obtainMessage(1, newPicture);
										handler.sendMessage(message);
									}
								};
								thread.start();
								y = 10000;
							}
						}
					}
				}
			}
			return v;
		}
	}

	/**
	 * Click listener for the action button.
	 */
	private OnClickListener actionListener = new OnClickListener() {
		public void onClick(View v) {
			actionDialog();
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		onCreateDialog();
	}

	/**
	 * Refreshes the list.
	 */
	public void refreshList() {
		m_adapter.clear();
		viewQuestions = new Runnable() {
			@Override
			public void run() {
				getQuestions();
			}
		};
		Thread thread = new Thread(null, viewQuestions, "MagentoBackground");
		thread.start();
		m_ProgressDialog = ProgressDialog.show(QuestionsList.this, "Please Wait", "Retrieving Question List", true);
	}

	/**
	 * Click listener for the list view.
	 */
	public OnItemClickListener questionListListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> a, View v, int position, long id) {
			Campustream.questionID = m_questions.get(position).getQuestionID();
			Campustream.questionTitle = m_questions.get(position).getQuestionTitle();
			Campustream.questionMessage = m_questions.get(position).getQuestionMessage();
			Campustream.questionAsker = m_questions.get(position).getQuestionUser();
			OpenQuestion();
		}
	};

	/**
	 * Opens a question and shows its responses.
	 */
	public void OpenQuestion() {
		Intent myIntent = new Intent(this, QuestionPage.class);
		startActivityForResult(myIntent, 0);
	}

	/**
	 * Opens a dialog to submit a new question.
	 */
	public void NewQuestion() {
		Intent myIntent = new Intent(this, QuestionDialog.class);
		startActivityForResult(myIntent, 0);
	}
}