package com.main.campustream;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import android.app.TabActivity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.TabHost;

/**
 * @author Bryan Crabtree
 * 
 */
public class TabView extends TabActivity {

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.ActivityGroup#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		Resources res = getResources(); // Resource object to get Drawables
		TabHost tabHost = getTabHost(); // The activity TabHost
		TabHost.TabSpec spec; // Reusable TabSpec for each tab
		Intent intent; // Reusable Intent for each tab

		// Create the Stream tab
		intent = new Intent().setClass(this, StreamTab.class);
		spec = tabHost.newTabSpec("stream").setIndicator("Stream", res.getDrawable(R.drawable.streamicon)).setContent(intent);
		tabHost.addTab(spec);

		// Create the News tab
		intent = new Intent().setClass(this, NewsTab.class);
		spec = tabHost.newTabSpec("news").setIndicator("News", res.getDrawable(R.drawable.newsicon)).setContent(intent);
		tabHost.addTab(spec);

		// Create the Events tab
		intent = new Intent().setClass(this, EventsTab.class);
		spec = tabHost.newTabSpec("events").setIndicator("Events", res.getDrawable(R.drawable.eventsicon)).setContent(intent);
		tabHost.addTab(spec);

		// Create the Stream tab
		intent = new Intent().setClass(this, CollaborationTab.class);
		spec = tabHost.newTabSpec("collaboration").setIndicator("Collaborate", res.getDrawable(R.drawable.collaborationicon)).setContent(intent);
		tabHost.addTab(spec);

		tabHost.setCurrentTabByTag("stream");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.layout.menulayout, (android.view.Menu) menu);
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.profile:
				Campustream.currentProfileUsername = Campustream.username;
				Intent myIntent1 = new Intent(this, Profile.class);
				startActivityForResult(myIntent1, 0);
				break;
			case R.id.friends:
				Intent myIntent2 = new Intent(this, FriendsTab.class);
				startActivityForResult(myIntent2, 0);
				break;
			case R.id.settings:
				Intent myIntent3 = new Intent(this, SettingsDialog.class);
				startActivityForResult(myIntent3, 0);
				break;
		}
		return true;
	}

	/**
	 * Opens an HTTP connection at the specified URL.
	 * 
	 * @param urlString
	 * @return
	 * @throws IOException
	 */
	public static InputStream OpenHttpConnection(String urlString) throws IOException {
		InputStream in = null;
		int response = -1;

		URL url = new URL(urlString);
		URLConnection conn = url.openConnection();

		if (!(conn instanceof HttpURLConnection))
			throw new IOException("Not an HTTP Connection!");

		try {
			HttpURLConnection httpConn = (HttpURLConnection) conn;
			httpConn.setAllowUserInteraction(false);
			httpConn.setInstanceFollowRedirects(true);
			httpConn.setRequestMethod("GET");
			httpConn.connect();

			response = httpConn.getResponseCode();
			if (response == HttpURLConnection.HTTP_OK) {
				in = httpConn.getInputStream();
			}
		} catch (Exception ex) {
			throw new IOException("Error Connecting!");
		}
		return in;
	}
}
