<?php
/*
 * This file defines all routes for the framework. The array index is
 * a regex expression that matches an incoming URL, and the corresponding
 * array value is the controller/method to execute if that URL is matched.
 * You may also pass arguments to the controller method. See user/show below
 * for an example.
 */
 
// Debug routes
$config['debug/(flockdb|sphinx|phpinfo|rss|notify)'] = 'debug/$1';

// Default route for root directory
$config['_default'] = 'home/index';

// Default session routes
$config['session/new'] = 'session/new_action';
$config['session/create'] = 'session/create';
$config['session/destroy'] = 'session/destroy';
$config['session/twitter'] = 'session/twitter_login';
$config['session/twitter/finish'] = 'session/twitter_finish';

// Confirmation routes
$config['confirm/wait'] = 'confirm/wait';
$config['confirm/do'] = 'confirm/do_action';

// Twitter session routes
$config['twitter/create'] = 'twitter/create';
$config['twitter/new'] = 'twitter/new_action';
$config['twitter/destroy'] = 'twitter/destroy';

// Facebook session routes
$config['facebook/(authorize|deauthorize)'] = 'facebook/$1';

// Login or Register
$config['register'] = 'account.manage/register';
$config['login'] = 'account.manage/login';

// User routes
$config['user/([A-Za-z0-9_]+)'] = 'user/show/username:$1';
$config['user/avatar/(small|medium|large)/([A-Za-z0-9_]+)'] = 'user/avatar/size:$1/username:$2';
$config['user/([A-Za-z0-9_]+)/follows'] = 'friendship/follows/user:$1';
$config['user/([A-Za-z0-9_]+)/followers'] = 'friendship/followers/user:$1';
$config['users'] = 'user/list_users';

// Status routes
$config['s/([A-Za-z0-9]+)'] = 'status/show/id:$1';

// News & Events routes
$config['news_events'] = 'news/index/type:all';
$config['news'] = 'news/index/type:news';
$config['news/([A-Za-z0-9_]+)'] = 'news/index/type:news/slug:$1';
$config['events'] = 'news/index/type:event';
$config['event/([A-Za-z0-9_]+)'] = 'news/index/type:event/slug:$1';
$config['ne/([A-Za-z0-9]+)'] = 'news/show/short_id:$1';
$config['news/new'] = 'news/new_action';

// Collaborate routes
$config['collaborate'] = 'collaborate/index';
$config['collaborate/([A-Za-z_]+)'] = 'collaborate/show_section/section:$1';
$config['collaborate/q/([A-Za-z0-9]+)'] = 'collaborate/show/short_id:$1';

// Messaging
$config['messages'] = 'messaging/index';

// User settings
$config['settings/(profile|services)'] = 'account.settings/index/page:$1';
$config['settings/service_hooks'] = 'account.settings/service_hooks';
$config['settings/profile_photo/(update|remove)'] = 'account.settings/profile_photo/action:$1';
$config['settings/update_profile'] = 'account.settings/update_profile';

// Site statistics
$config['admin/stats'] = 'admin.stats/index';

// API Routing
$config['api/1/([A-Za-z_]+)/([A-Za-z_]+)'] = 'api.apiv1/dispatch/controller:$1/method:$2';

## API endpoints

// User APIs
$api['user/authorize'] = 'user/authorize';
$api['users/(show)'] = 'user/$1';
$api['users/list'] = 'user/show_list';

// Friendship APIs
$api['friendships/(show|create|destroy|followers|follows)'] = 'friendship/$1';

// Status APIs
$api['statuses/(public_timeline|user_timeline|friend_timeline|show|comments|update|delete)'] = 'status/$1';
$api['comments/(update)'] = 'comment/$1';

// Search APIs
$api['search/(statuses|users|tags)'] = 'search/$1';

// Collaborate APIs
$api['collaborate/(create|respond|vote|show|delete)'] = 'collaborate/$1';
$api['collaborate/list'] = 'collaborate/show_list';

// News and Events APIs
$api['news/list'] = 'news/index';
$api['news/(show|create|vote|create_comment|delete_comment|location_search)'] = 'news/$1';

// Notification APIs
$api['notifications/(show|mark_as_read)'] = 'notification/$1';

// Setting APIs
$api['settings/(service_hooks|email)'] = 'account.settings/$1';

// Debug routes
$config['debug/auth'] = 'debug/auth_test';
$api['debug/auth'] = 'debug/auth_test';