<?

/**
 * Handles all Collaboration routes and API endpoints.
 */
class Collaborate_Controller extends Controller implements REST {
	public $enable_session = true;
	public $template = 'template/main';
	
	/**
	 * Shows the Collaboration index page with the various sections.
	 */
	public function index() {
		sess::require_login();
		
		$view = new View('collaborate/index');
		$this->template->title = "Collaborate";
		$this->template->active = 'collaborate';
		$this->template->content = $view->render();
		echo $this->template->render();
	}
	
	/**
	 * Shows a single Collaboration section
	 */
	public function show_section($args) {
		$sections = Question_Model::$categories;
		
		sess::require_login();
		
		$section = $args['section'];
		
		if (!in_array($section, $sections)) {
			return Hub::redirect('/collaborate');
		}
		
		$view = new View('collaborate/section');
		$view->section = ucfirst(strtolower($section));
		$this->template->title = "Collaborate &raquo; {$view->section}";
		$this->template->active = 'collaborate';
		$this->template->content = $view->render();
		echo $this->template->render();
	}
	
	/**
	 * Loads and displays a list of Collaboration items.
	 */
	public function show_list($args) {
		$section = (isset($_GET['section']) ? $_GET['section'] : 'academic');
		$sorting = (isset($_GET['sorting']) ? $_GET['sorting'] : 'last_updated');
		$page = (isset($_GET['page']) ? $_GET['page'] : 1);
		
		$r = RedisManager::connection();
		
		$question_ids = array();
		$questions = null;
		
		$start = ($page - 1) * 20;
		
		switch ($sorting) {
			case 'last_updated':
				$questions = ActiveRecord::find_all('Question_Model', "SELECT * FROM questions WHERE category = '{$section}' AND deleted = 0 ORDER BY last_update DESC LIMIT $start, 20");
				break;
			case 'popular':
				break;
			case 'newest':
				$question_ids = $r->lrange("questions:stream:{$section}:newest", $start, $start + 20);
			case 'search':
				break;
		}
		
		if (!$questions) {
			$questions = array();
			foreach($question_ids as $qid) {
				$question = ActiveCache::find('Question_Model', $qid, 43200)->sql(
					"SELECT * FROM questions WHERE id = $qid"
				);
				
				if (!$question->is_loaded() || $question->deleted > 0) {
					continue;
				}
				
				$question->load_user();
				$question->load_tags();
				$question->user = $question->user->limited_object();
				$questions[] = $question->limited_object();
			}
		} else {
			foreach ($questions as $i=>$question) {
				$question->load_user();
				$question->load_tags();
				$question->user = $question->user->limited_object();
				$questions[$i] = $question->limited_object();
			}
		}
		
		View::respond_to(array('json', 'jsonp', 'xml'), function ($format) use($questions) {
			switch ($format) {
				case 'json': echo json_encode($questions); break;
				case 'xml': echo xml::encode_array($questions, 'questions', 'question');
			}
		});
	}
	
	/**
	 * Shows a single question in the Collaboration section
	 */
	public function show($args) {
		$id = null;
		
		if (isset($args['short_id'])) {
			$id = base_convert($args['short_id'], 36, 10);
		} elseif (isset($_GET['short_id'])) {
			$id = base_convert($_GET['short_id'], 36, 10);
		} elseif (isset($_GET['id']) && is_numeric($_GET['id'])) {
			$id = $_GET['id'];
		} else {
			return Hub::http_error(401, "Missing question ID");
		}
		
		$question = ActiveCache::find('Question_Model', $id, 43200)->sql(
			"SELECT * FROM questions WHERE id = $id LIMIT 1"
		);
		
		if (!$question->is_loaded() || $question->deleted) {
			View::respond_to(array('xml', 'json', 'jsonp'), function () {
				Hub::http_error(404, "Question not found");
			});
			
			// Should show an error page in the future
			View::respond_to('html', function () {
				Hub::redirect('/collaborate');
			});
			
			return false;
		}
		
		$question->load_user();
		$question->load_responses();
		$question->load_tags();
		
		View::respond_to(array('xml', 'json', 'jsonp'), function ($format) use($question) {
			foreach ($question->responses as $i=>$response) {
				$response->message = htmlspecialchars_decode($response->message, ENT_QUOTES);
				$question->responses[$i] = $response;
			}
			
			echo $question->{"to_$format"}();
		});
		
		$template = $this->template;
		View::respond_to('html', function ($format) use($question, $template) {
			$view = new View('collaborate/show');
			$view->question = $question;
			$template->title = "Collaborate &raquo; " . ucfirst($question->category) . " &raquo; " . $question->title;
			$template->active = 'collaborate';
			$template->content = $view->render();
			echo $template->render();
		});
	}
	
	/**
	 * Creates a new question for Collaboration
	 */
	public function create() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		if (!isset($_POST['title']) || !isset($_POST['message']) || !isset($_POST['category'])) {
			return Hub::http_error(401, "Missing required data");
		}
		
		if (!in_array($_POST['category'], Question_Model::$categories)) {
			return Hub::http_error(401, "Invalid category");
		}
		
		$public = 1;
		if (isset($_POST['public'])) {
			$public = ($_POST['public'] == '0' ? 0 : 1);
		}
		
		$title = htmlspecialchars(trim($_POST['title']), ENT_QUOTES, 'UTF-8');
		$message = htmlspecialchars(trim($_POST['message']), ENT_QUOTES, 'UTF-8');
		
		$question = new Question_Model();
		$question->user_id = sess::getUserID();
		$question->category = strtolower($_POST['category']);
		$question->title = $title;
		$question->message = nl2br($message);
		$question->post_date = ActiveRecord::NOW();
		$question->last_update = ActiveRecord::NOW();
		$question->vote_count = 0;
		$question->is_public = $public;
		$question->save();
		
		// and now to save the tags...
		$tags_list = isset($_POST['tags']) ? trim($_POST['tags']) : false;
		if ($tags_list) {
			$tags = explode(',', $tags_list);
		}
		
		foreach($tags as $tag) {
			// convert newlines and tabs to spaces
			$tag = trim(str_replace(array("\t", "\n", "\r"), " ", $tag));
			if (strlen($tag) > 0 && !preg_match('/[^A-Z0-9_-\s]+/i', $tag)) {
				$_tag = new Questiontag_Model();
				$_tag->question_id = $question->id;
				$_tag->tag = $tag;
				$_tag->save();
			}
		}
		
		View::respond_to(array('json', 'xml'), function ($format) use($question) {
			$question->load_user();
			$question->load_tags();
			echo $question->{"to_$format"}();
		});	
	}
	
	/**
	 * Creates a response to a Collaboration question.
	 */
	public function respond() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Unauthorized");
		}
		
		if (!isset($_POST['message'])) {
			return Hub::http_error(401, "Missing respond message");
		}
		
		$id = null;
		if (isset($_POST['short_id'])) {
			$id = base_convert($_POST['short_id'], 36, 10);
		} elseif (isset($_POST['id']) && is_numeric($_POST['id'])) {
			$id = $_POST['id'];
		} else {
			return Hub::http_error(401, "Question ID missing or invalid");
		}
		
		$message = htmlspecialchars(trim($_POST['message']), ENT_QUOTES, 'UTF-8');
		
		$public = 1;
		if (isset($_POST['public']) && is_numeric($_POST['public'])) {
			$public = $_POST['public'];
		}
		
		$response = new Response_Model();
		$response->question_id = $id;
		$response->user_id = sess::getUserID();
		$response->message = nl2br($message);
		$response->post_date = ActiveRecord::NOW();
		$response->vote_count = 0;
		$response->is_public = $public;
		$response->save();
		
		// chances are, you probably want to vote your own response up. digg
		// and reddit do it, we can too.
		$response->vote('up');
		
		// Now that we've responded, we need to move this question back to the top
		// of the 'last updated' questions
		$response->load_question();
		$response->question->last_update = ActiveRecord::NOW();
		$response->question->save(false);
		
		View::respond_to(array('json', 'xml'), function ($format) use($response) {
			echo $response->{"to_$format"}();
		});
	}
	
	/**
	 * Adds or removes a vote on a Collaboration response
	 */
	public function vote() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Unauthorized");
		}
		
		if (!isset($_POST['dir']) || !isset($_POST['id'])) {
			return Hub::http_error(401, "Missing data, invalid request");
		}
		
		if (($_POST['dir'] != 'up' && $_POST['dir'] != 'down') || !is_numeric($_POST['id'])) {
			return Hub::http_error(401, "Invalid data");
		}
		
		$response = ActiveCache::find('Response_Model', $_POST['id'], 43200)->sql(
			"SELECT * FROM responses WHERE id = {$_POST['id']} LIMIT 1"
		);
		
		if (!$response->is_loaded()) {
			return Hub::http_error(404, "Response not found");
		}
		
		$response->vote($_POST['dir']);
		
		View::respond_to(array('json', 'xml'), function ($format) use($response) {
			echo $response->{"to_$format"}();
		});
	}
	
	/**
	 * Deletes a Collaboration question.
	 */
	public function delete() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$id = $_POST['id'];
		
		if (!$id || !is_numeric($id)) {
			return Hub::http_error(401, "Invalid or missing ID");
		}
		
		$question = ActiveCache::find('Question_Model', $id, 43200)->sql(
			"SELECT * FROM questions WHERE id = $id LIMIT 1"
		);
		
		if (!$question->is_loaded()) {
			return Hub::http_error(404, "Question not found");
		}
		
		if ($question->user_id != sess::getUserID()) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$question->delete();
	}
}