<?

/**
 * Social graph methods. Shows connections in the social graph, as
 * well as handles adding/deleting connections.
 */
class Friendship_Controller extends Controller implements REST {
	public $enable_session = true;
	public $template = 'template/main';
	
	/**
	 * Shows the social relationship between two given users and shows
	 * their follow status with regards to each other.
	 */
	public function show() {
		if (
			(!isset($_GET['source_id']) && !isset($_GET['source_username'])) || 
			(!isset($_GET['target_id']) && !isset($_GET['target_username']))
		   ) {
			
			return Hub::http_error(401, "Missing required data");
		}
		
		$source = null;
		$target = null;
		
		if (isset($_GET['source_id'])) {
			if (!is_numeric($_GET['source_id'])) {
				return Hub::http_error(401, "Invalid data");
			}

			$source = ActiveCache::find('User_Model', $_GET['source_id'], 43200)->sql(
				"SELECT * FROM users WHERE id = {$_GET['source_id']} LIMIT 1"
			);
		} else {
			$username = DatabaseManager::escape($_GET['source_username']);
			
			$source = ActiveCache::find('User_Model', "name:$username", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}
		
		if (isset($_GET['target_id'])) {
			if (!is_numeric($_GET['target_id'])) {
				return Hub::http_error(401, "Invalid data");
			}

			$target = ActiveCache::find('User_Model', $_GET['target_id'], 43200)->sql(
				"SELECT * FROM users WHERE id = {$_GET['target_id']} LIMIT 1"
			);
		} else {
			$username = DatabaseManager::escape($_GET['target_username']);
			
			$target = ActiveCache::find('User_Model', "name:$username", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}
		
		
		if (!$source->is_loaded()) {
			return Hub::http_error(404, "Source user not found");
		} elseif (!$target->is_loaded()) {
			return Hub::http_error(404, "Target user not found");
		}
		
		$relationship = new Relationship_Model();
		$relationship->set($source, $target);
		
		View::respond_to(array('xml', 'json', 'jsonp'), function ($format) use($relationship) {
			echo $relationship->{"to_$format"}();
		});
	}
	
	public function create() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		if (!isset($_POST['user_id']) && !isset($_POST['username'])) {
			return Hub::http_error(401, "Missing required data");
		}
		
		$user = null;
		$curUser = sess::getUser();
		
		if (isset($_POST['user_id'])) {
			if (!is_numeric($_POST['user_id'])) {
				return Hub::http_error(401, "Invalid data");
			}
			
			$user_id = $_POST['user_id'];
			$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = $user_id LIMIT 1"
			);
			
		} else {
			$username = DatabaseManager::escape($_POST['username']);
			$user = ActiveCache::find('User_Model', "name:{$username}", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}
		
		if(!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		if ($curUser->is_following($user)) {
			return Hub::http_error(401, "User is already followed");
		}
		
		$curUser->follow($user);
		
		View::respond_to(array('json','xml'), function ($format) use($user) {
			echo $user->{"to_$format"}();
		});
	}
	
	public function destroy() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		if (!isset($_POST['user_id']) && !isset($_POST['username'])) {
			return Hub::http_error(401, "Missing required data");
		}
		
		$user = null;
		$curUser = sess::getUser();
		
		if (isset($_POST['user_id'])) {
			if (!is_numeric($_POST['user_id'])) {
				return Hub::http_error(401, "Invalid data");
			}
			
			$user_id = $_POST['user_id'];
			$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = $user_id LIMIT 1"
			);
			
		} else {
			$username = DatabaseManager::escape($_POST['username']);
			$user = ActiveCache::find('User_Model', "name:{$username}", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}
		
		if(!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		if (!$curUser->is_following($user)) {
			return Hub::http_error(401, "Authenticated user is not following the given user");
		}
		
		$curUser->unfollow($user);
		
		View::respond_to(array('json','xml'), function ($format) use($user) {
			echo $user->{"to_$format"}();
		});
	}
	
	public function followers($args) {
		if (!isset($_GET['user_id']) && !isset($_GET['username']) && !isset($args['user'])) {
			return Hub::http_error(401, "Missing required data");
		}
		
		$user = null;

		// this is incredibly hackish
		if (isset($args['user'])) {
			$_GET['username'] = $args['user'];
		}
		
		if (isset($_GET['user_id'])) {
			if (!is_numeric($_GET['user_id'])) {
				return Hub::http_error(401, "Invalid data");
			}
			
			$user_id = $_GET['user_id'];
			$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = $user_id LIMIT 1"
			);
			
		} else {
			$username = DatabaseManager::escape($_GET['username']);
			$user = ActiveCache::find('User_Model', "name:{$username}", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}

		if(!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		$result = $user->followers();
		$followers = array();
		if (count($result) > 0) {
			foreach ($result as $follower_id) {
				$follower = ActiveCache::find('User_Model', $follower_id, 43200)->sql(
					"SELECT * FROM users WHERE id = $follower_id LIMIT 1"
				);
				
				if ($follower->is_loaded()) {
					$followers[] = $follower->limited_object();
				}
			}
		}
		
		// HACKISH, BUT WHATEVS
		$user->columns[] = 'followers';
		$user->public_columns[] = 'followers';
		$user->followers = $followers;
		
		View::respond_to(array('xml', 'json', 'jsonp'), function ($format) use($user) {
		    echo $user->{"to_$format"}();
		});
		
		$template = $this->template;
		View::respond_to('html', function () use($user, $template) {
			$view = new View('user/friendlist');
			$view->type = 'followers';
			$view->user = $user;
			$template->content = $view->render();
			echo $template->render();
		});
	}
	
	public function follows($args) {
		if (!isset($_GET['user_id']) && !isset($_GET['username']) && !isset($args['user'])) {
			return Hub::http_error(401, "Missing required data");
		}
		
		$user = null;
		
		// this is incredibly hackish
		if (isset($args['user'])) {
			$_GET['username'] = $args['user'];
		}
		
		if (isset($_GET['user_id'])) {
			if (!is_numeric($_GET['user_id'])) {
				return Hub::http_error(401, "Invalid data");
			}
			
			$user_id = $_GET['user_id'];
			$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = $user_id LIMIT 1"
			);
			
		} else {
			$username = DatabaseManager::escape($_GET['username']);
			$user = ActiveCache::find('User_Model', "name:{$username}", 43200)->sql(
				"SELECT * FROM users WHERE username = '$username' LIMIT 1"
			);
		}
		
		if(!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		$result = $user->follows();
		$followers = array();
		if (count($result) > 0) {
			foreach ($result as $follower_id) {
				$follower = ActiveCache::find('User_Model', $follower_id, 43200)->sql(
					"SELECT * FROM users WHERE id = $follower_id LIMIT 1"
				);
				
				if ($follower->is_loaded()) {
					$followers[] = $follower->limited_object();
				}
			}
		}
		
		// HACKISH, BUT WHATEVS
		$user->columns[] = 'follows';
		$user->public_columns[] = 'follows';
		$user->follows = $followers;
		
		View::respond_to(array('xml', 'json', 'jsonp'), function ($format) use($user) {
		    echo $user->{"to_$format"}();
		});
		
		$template = $this->template;
		View::respond_to('html', function () use($user, $template) {
			$view = new View('user/friendlist');
			$view->type = 'follows';
			$view->user = $user;
			$template->content = $view->render();
			echo $template->render();
		});
	}

}