<?

/**
 * Handles all interactions with the News/Events section
 * of the site.
 */
class News_Controller extends Controller implements REST {
	public $enable_session = true;
	public $template = 'template/main';
	
	/**
	 * Loads the news/event page based on the given type and section.
	 */
	public function index($args) {
		if (isset($_GET['slug'])) {
			$args['slug'] = $_GET['slug'];
		}
		if (isset($_GET['type'])) {
			$args['type'] = $_GET['type'];
		}
		
		if ($args['type'] == 'news' && $args['slug'] == 'imported') {
			$args['type'] = 'imported';
		}
	
		$cat = 0;
		if (isset($args['slug'])) {
			$cat = Newseventcategory_Model::getBySlug($args['type'], $args['slug']);
		}
		
		$news = null;
		if (isset($args['slug'])) {
			// Type & Category
			$news = Newsevent_Model::load_popular(array('cat_id' => $cat->id));
		} elseif (isset($args['type']) && !isset($args['slug'])) {
			// Type & All Categories
			$news = Newsevent_Model::load_popular(array('type' => $args['type']));
		} else {
			// All Types & All Categories
			$news = Newsevent_Model::load_popular();
		}
		
		for ($i = 0; $i < count($news); $i++) {
			$news[$i]->load_category();
			$news[$i]->load_user();
		}

		$template = $this->template;	
		View::respond_to('html', function () use($news, $cat, $args, $template) {			
			$view = new View('news/index');
			$view->posts = $news;
			$view->type = (isset($args['type']) ? $args['type'] : 'all');
			$view->category = $cat;
			
			$template->active = 'newsevents';
			$template->content = $view->render();
			echo $template->render();
		});
		
		View::respond_to(array('json', 'jsonp', 'xml'), function ($format) use($news) {
			for ($i = 0; $i < count($news); $i++) {
				$news[$i] = $news[$i]->limited_object();
			}
			
			if ($format == 'json') {
				echo json_encode($news);
			} elseif ($format == 'jsonp') {
				return;
			} else {
				echo xml::encode_array($news, 'posts', 'post');
			}
		});
		
		View::respond_to('rss', function () use($news) {
		  $stories = array();
		  foreach ($news as $story) {
		    $stories[] = array(
		      'title' => $story->title,
		      'link' => 'http://campustream.com/ne/' . $story->short_id,
		      'description' => $story->post
		    );
		  }

		  echo xml::encode_rss($stories);
		});
	}
	
	/**
	 * Displays the page where users can create a new news/event item
	 */
	public function new_action() {
		sess::require_login();
		
		$view = new View('news/new');
		$this->template->active = 'newsevents';
		$this->template->content = $view->render();
		echo $this->template->render();
	}
	
	/**
	 * Shows the single given news/event item
	 */
	public function show($args) {
		$id = null;
		if (isset($args['short_id'])) {
			$id = base_convert($args['short_id'], 36, 10);
		} elseif (isset($_GET['id']) && is_numeric($_GET['id'])) {
			$id = $args['id'];
		} elseif (isset($_GET['short_id'])) {
			$id = base_convert($_GET['short_id'], 36, 10);
		} else {
			View::respond_to('html', function () {
				Hub::redirect('/news');
			});
			
			View::respond_to(array('json', 'jsonp', 'xml'), function () {
				Hub::http_error(400, "Missing or invalid ID");
			});
		}
		
		$news = ActiveCache::find('Newsevent_Model', $id, 43200)->sql(
			"SELECT * FROM newsevents WHERE id = $id LIMIT 1"
		);
		
		if (!$news->is_loaded()) {
			View::respond_to('html', function () {
				Hub::redirect('/news');
			});
			
			View::respond_to(array('json', 'jsonp', 'xml'), function () {
				Hub::http_error(400, "Missing or invalid ID");
			});
		}
		
		$news->load_category();
		$news->load_user();
		$news->load_comments();
		$news->load_eventmeta();
		
		$template = $this->template;
		View::respond_to('html', function () use($news, $template) {
			$view = new View('news/show');
			$view->news = $news;
			
			$template->active = 'newsevents';
			$template->content = $view->render();
			echo $template->render();
		});
		
		View::respond_to(array('json', 'jsonp', 'xml'), function ($format) use($news) {
			if ($news->type == 'text') {
				$news->post = bbparser::bb2html($news->post);
			}
			
			echo $news->{"to_$format"}();
		});
	}
	
	/**
	 * Creates a new news/event item
	 */
	public function create() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$title = htmlspecialchars(trim($_POST['title']), ENT_QUOTES, 'UTF-8');
		$content = htmlspecialchars(trim($_POST['content']), ENT_QUOTES, 'UTF-8');
		$category = $_POST['category'];
		$type = strtolower(trim($_POST['type']));
		
		/* Data validation */
		if (strlen($title) == 0 || strlen($content) == 0) {
			return Hub::http_error(400, "Missing title or content");
		}
		
		if (!is_numeric($category) || $category <= 0) {
			return Hub::http_error(400, "Invalid or missing category");
		}
		
		if ($type != 'text' && $type != 'link') {
			return Hub::http_error(400, "Invalid post type");
		}
		
		$cat = new Newseventcategory_Model();
		$cat->load($category);
		
		$news = new Newsevent_Model();
		$news->type = $type;
		$news->category_id = $cat->id;
		$news->newsevent = $cat->type;
		$news->title = $title;
		$news->post = $content;
		$news->posted_by = sess::getUserID();
		$news->time_posted = ActiveRecord::NOW();
		$news->votes = 0;
		
		$news->save(true);

		// Now that we've saved the news item, we can also save the event metadata if applicable
		if ($cat->type == 'event') {
			$meta = new Eventmeta_Model();
			$meta->event_id = $news->id;
			$meta->location = htmlspecialchars(trim($_POST['event_location']), ENT_QUOTES, 'UTF-8');
			$meta->location_area = htmlspecialchars(trim($_POST['event_location_area']), ENT_QUOTES, 'UTF-8');

			if ($_POST['event_all_day']) {
				$meta->all_day = 1;
			} else {
				$meta->all_day = 0;
				
				$from_min = str_pad($_POST['event_start_from_minute'], 2, 0, STR_PAD_LEFT);
				$to_min = str_pad($_POST['event_start_to_minute'], 2, 0, STR_PAD_LEFT);
				
				$start_date = strtotime("{$_POST['event_start_from_date']} {$_POST['event_start_from_hour']}:{$from_min} {$_POST['event_start_from_ampm']}");
				$end_date = strtotime("{$_POST['event_start_to_date']} {$_POST['event_start_to_hour']}:{$to_min} {$_POST['event_start_to_ampm']}");
				
				$meta->start_date = date('Y-m-d H:i:s', $start_date);
				$meta->end_date = date('Y-m-d H:i:s', $end_date);
			}

			$meta->save();
		}
		
		View::respond_to(array('json', 'xml'), function ($format) use($news) {
			echo $news->{"to_$format"}();
		});
	}
	
	/**
	 * Adds or removes a vote on a news/event item
	 */
	public function vote() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$id = $_POST['id'];
		$dir = $_POST['dir'];
		
		if (!is_numeric($id)) {
			return Hub::http_error(400, "Invalid post ID");
		}
		
		$news = ActiveCache::find('Newsevent_Model', $id, 43200)->sql(
			"SELECT * FROM newsevents WHERE id = $id LIMIT 1"
		);
		
		if (!$news->is_loaded()) {
			return Hub::http_error(404, "Post ID not found");
		}
		
		$news->vote($dir);

		View::respond_to(array('json', 'xml'), function ($format) use($news) {
			echo $news->{"to_$format"}();
		});
	}
	
	/**
	 * Adds a comment to the given news/event item
	 */
	public function create_comment() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Unauthorized");
		}
		
		$news_id = null;
		if (isset($_POST['short_news_id'])) {
			$news_id = base_convert($_POST['short_news_id'], 36, 10);
		} elseif (isset($_POST['news_id']) && is_numeric($_POST['news_id'])) {
			$news_id = $_POST['news_id'];
		} else {
			return Hub::http_error(400, "Invalid news ID");
		}
		
		$reply_to = isset($_POST['reply_to']) ? $_POST['reply_to'] : 0;
		$content = htmlspecialchars(trim($_POST['content']), ENT_QUOTES, 'UTF-8');
		
		if (strlen($content) == 0) {
			return Hub::http_error(400, "Missing comment content");
		}
		
		if (!is_numeric($reply_to)) {
			return Hub::http_error(400, "Invalid reply to ID");
		}
		
		$comment = new Newscomment_Model();
		$comment->posted_by = sess::getUserID();
		$comment->news_id = $news_id;
		$comment->parent_id = $reply_to;
		$comment->content = $content;
		$comment->time_posted = ActiveRecord::NOW();
		
		$comment->save();
		
		View::respond_to(array('json', 'xml'), function ($format) use($comment) {
			echo $comment->{"to_$format"}();
		});
	}
	
	/**
	 * Deletes the given comment from a news/event item. Notice that it performs
	 * a soft-delete by flagging the comment as deleted, instead of actually removing
	 * it from MySQL. This is for data mining purposes, and because removing the comment
	 * data would ruin the heirarchical structure of the commenting system.
	 */
	public function delete_comment() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$id = $_POST['id'];
		if (!$id || !is_numeric($id)) {
			return Hub::http_error(401, "Invalid or missing ID");
		}
		
		$comment = ActiveCache::find('Newscomment_Model', $id, 43200)->sql(
			"SELECT * FROM newscomments WHERE id = $id LIMIT 1"
		);
		
		if (!$comment->is_loaded()) {
			return Hub::http_error(404, "Comment not found");
		}
		
		if ($comment->posted_by != sess::getUserID()) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$comment->delete();
		
		View::respond_to(array('json', 'xml'), function () {
			return false;
		});
	}
	
	/**
	 * Performs a simple location search based on a search query. The search list
	 * is stored in the Eventlocation_Model class.
	 */
	public function location_search() {
		$q = trim($_GET['q']);
		if (!$q || strlen($q) == 0) {
			return Hub::http_error(400, "Missing query parameter");
		}
		
		$results = Eventlocation_Model::findByName($_GET['q']);
		
		$data = array();
		foreach ($results as $result) {
			$data[] = array(
				'id' => $result['id'],
				'label' => $result['name'],
				'value' => $result['name']
			);
		}
		
		echo json_encode($data);
	}
}