<?

/**
 * Responsible for managing notifications on the site.
 */
class Notification_Controller extends Controller implements REST {
	public $enable_session = true;
	
	/**
	 * Marks all notifications in the given section as read
	 */
	public function mark_as_read() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$type = trim($_POST['type']);
		if (!$type) {
			return Hub::http_error(401, "Must specify which type of notifications to set as read");
		}
		
		$n = notifications::load();
		if (!$n->markAsRead($type)) {
			return Hub::http_error(401, "Invalid type given or error marking as read");
		}
		
		View::respond_to(array('json', 'xml'), function ($format) use($n) {
			$num = array(
				'news' => $n->numUnread('news'),
				'collaborate' => $n->numUnread('collaborate'),
				'reply' => $n->numUnread('reply')
			);
			
			if ($format == 'json') {
				echo json_encode($num);
			}
		});
	}
	
	/**
	 * Loads all notifications for the logged in user.
	 */
	public function show() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$n = notifications::load();
		
		$msgs = array();
		$msgs['news'] = array_merge($n->get('news'), $n->get('newscomment'));
		$msgs['collaborate'] = $n->get('collaborate');
		$msgs['reply'] = $n->get('reply');
		
		
		View::respond_to(array('xml', 'json'), function ($format) use($msgs) {
			if ($format == 'json') {
				echo json_encode($msgs);
			}
		});
	}
}