<?

/**
 * Responsible for handling all search API endpoints on the site.
 * All searches are performed with the Sphinx search engine.
 */
class Search_Controller extends Controller implements REST {
	public $enable_session = true;
	
	/**
	 * Perform a search through all user statuses
	 */
	public function statuses() {
		$q = trim($_GET['q']);

		if (!$q || strlen($q) < 3) {
			return Hub::http_error(401, "Invalid search, or search too short");
		}
		
		$sphinx = SphinxManager::connection();
		$res = $sphinx->Query($q . '*', 'statuses');
		if ($res['total'] > 0) {
			$statuses = array();
			foreach ($res['matches'] as $match) {
				$status = ActiveCache::find('Status_Model', $match['id'], 43200)->sql(
					"SELECT * FROM statuses WHERE id = {$match['id']} LIMIT 1"
				);
				
				if ($status->is_loaded() && $status->deleted == 0) {
					$statuses[] = $status->limited_object();
				}
			}
			
			View::respond_to(array('json', 'xml'), function ($format) use($statuses) {
				if ($format == 'json') {
					echo json_encode($statuses);
				} elseif ($format == 'xml') {
					echo xml::encode_array($statuses, "search", "result");
				}
			});
		}
	}
	
	/**
	 * Performs a search by searching through all usernames
	 */
	public function users() {
		$q = trim($_GET['q']);

		if (!$q || strlen($q) < 3) {
			return Hub::http_error(401, "Invalid search, or search too short");
		}
		
		$sphinx = SphinxManager::connection();
		$res = $sphinx->Query($q . '*', 'users');
		if ($res['total'] > 0) {
			$users = array();
			foreach ($res['matches'] as $match) {
				$user = ActiveCache::find('User_Model', $match['id'], 43200)->sql(
					"SELECT * FROM users WHERE id = {$match['id']} LIMIT 1"
				);
				
				if ($user->is_loaded()) {
					$users[] = $user->limited_object();
				}
			}
			
			View::respond_to(array('json', 'xml'), function ($format) use($users) {
				if ($format == 'json') {
					echo json_encode($users);
				} elseif ($format == 'xml') {
					echo xml::encode_array($users, "search", "result");
				}
			});
		}
	}
	
	/**
	 * Searches through all question tags
	 */
	public function tags() {
		$q = trim($_GET['q']);
		
		if (!$q || strlen($q) < 3) {
			return Hub::http_error(401, "Invalid search, or search too short");
		}
		
		$sphinx = SphinxManager::connection();
		$res = $sphinx->Query($q . '*', 'tags');
		if ($res['total'] > 0) {
			$questions = array();
			foreach ($res['matches'] as $match) {
				$question_id = $match['attrs']['question_id'];
				$question = ActiveCache::find('Question_Model', $question_id, 43200)->sql(
					"SELECT * FROM questions WHERE id = $question_id LIMIT 1"
				);
				
				if($question->is_loaded() && !$question->deleted) {
					$question->load_tags();
					$question->load_user();
					$questions[] = $question->limited_object();
				}
			}
			
			util::objectSort($questions, 'id');
			
			View::respond_to(array('json','xml'), function ($format) use($questions) {
				if ($format == 'json') {
					echo json_encode($questions);
				} else {
					echo xml::encode_array($questions, 'search', 'result');
				}
			});
		}
	}
}