<?php
include_once $GLOBALS['APPROOT'] . 'application/lib/twitteroauth/twitteroauth.php';

/**
 * Manages site sessions including creation of new users and new sessions, as well
 * as destroying sessions. Also handles Twitter and Facebook authentication.
 */
class Session_Controller extends Controller {
	public $template = 'template/main';
	public $enable_session = true;
	
	/**
	 * Finds the given user, sets the session to log the user in,
	 * and redirects the user.
	 */
	public function new_action() {
		
		/*
		 * If we want to redirect somewhere besides
		 * the site root
		 */
		if(!empty($_GET['r'])) {
			$redirect = $_GET['r'];
		}
		
		try{
			
			$username = reset(explode('@', $_POST['username']));
			
			$user = User_Model::find_user_for_login($username, $_POST['password']);
			$this->session->set_session($user);

			return Hub::redirect('/');
			
		} catch(Exception $e) {
			//echo $e->getMessage();
			return Hub::redirect('/login?fail=true');
			
		}
	}
	
	/**
	 * Logs a user in via Twitter by redirecting to the Twitter OAuth
	 * login page. If the user is already logged in, they will be immediately
	 * redirected back to Campustream.
	 */
	public function twitter_login() {
		$twitter = twitter::api();
		
		$callback = 'http://campustream.com/session/twitter/finish';
		if ($GLOBALS['ISDEV']) {
			$callback = 'http://dev.campustream.com/session/twitter/finish';
		}
		
		$request_token = $twitter->getRequestToken($callback);
		
		$this->session->set('oauth_token', $request_token['oauth_token']);
		$this->session->set('oauth_secret', $request_token['oauth_token_secret']);
		
		switch ($twitter->http_code) {
			case 200:
				$url = $twitter->getAuthorizeURL($this->session->get('oauth_token'));
				Hub::redirect($url);
				break;
			default:
				echo "Error connecting with Twitter, please try again! Error code: {$twitter->http_code}";
		}
	}
	
	/**
	 * Finishes the Twitter login process. The user arrives here after Twitter confirms
	 * their identity and account.
	 */
	public function twitter_finish() {
		if (isset($_REQUEST['oauth_token']) && $this->session->get('oauth_token') !== $_REQUEST['oauth_token']) {
			$this->session->set('oauth_token', null);
			$this->session->set('oauth_secret', null);
			
			echo "Old OAuth token, please try signing in again.";
			return false;
		}
		
		$oauth_token = $this->session->get('oauth_token');
		$oauth_secret = $this->session->get('oauth_secret');
		
		$twitter = new TwitterOAuth($GLOBALS['TWITTER_CONSUMER_KEY'], $GLOBALS['TWITTER_CONSUMER_SECRET'], $oauth_token, $oauth_secret);
		$access_token = $twitter->getAccessToken($_REQUEST['oauth_verifier']);
		
		$this->session->set('oauth_token', false);
		$this->session->set('oauth_secret', false);
		
		$tweeter = $twitter->get('account/verify_credentials');
		
		$usertwitter = ActiveRecord::find('Usertwitter_Model', "SELECT * FROM usertwitter WHERE twitter_id = {$tweeter->id} LIMIT 1");
		if ($usertwitter->is_loaded()) {
			$usertwitter->load_user();
			$this->session->set_session($usertwitter->user);
			
			return Hub::redirect('/');
		} else {
			return Hub::redirect('/login');
		}
	}
	
	/**
	 * Creates a new user via registration.
	 */
	public function create() {
		// Server-side validation
		foreach ($_POST as $key=>$val) {
			$_POST[$key] = trim($val);
			
			if(strlen($_POST[$key]) == 0) {
				if ($key == 'full_name') {
					$_POST['full_name'] = $_POST['username'];
					continue;
				}
				
				return Hub::redirect('/register?fail=' . $key);
			}
		}
		
		if (!preg_match('/[A-Za-z0-9_]+@wpi.edu/i', $_POST['email'])) {
			return Hub::redirect('/register?fail=email');
		}
		
		$username = reset(explode('@', $_POST['email']));
		
		$user = ActiveRecord::find('User_Model', "SELECT * FROM users WHERE username = '$username' LIMIT 1");
		if ($user->is_loaded()) {
			return Hub::redirect('/register?fail=username_taken');
		}
		
		if (strlen($_POST['password']) <=3) {
			return Hub::redirect('/register?fail=password');
		}
		
		if ($_POST['password'] != $_POST['password_confirm']) {
			return Hub::redirect('/register?fail=password_confirm');
		}

		$user = new User_Model();
		$user->username = $username;
		$user->name = $_POST['full_name'];
		$user->email = $_POST['email'];
		$user->password = sha1($_POST['password']);
		$user->join_date = ActiveRecord::NOW();
		$user->confirmed = 0;
		$user->confirm_code = md5(time() + $username + rand(0,1000));
		$user->save();
		
		$r = RedisManager::connection();
		$first_letter = strtolower(substr($user->name, 0, 1));
		$r->sadd("users:by_first_letter:$first_letter", $user->id);
		
		// Send the confirmation email
		email::send_confirmation($user);
		
		$this->session->set_session($user);
		
		return Hub::redirect('/settings/profile');
	}
	
	/**
	 * Rebuilds a user's session based on the given access token.
	 * This is for API authentication.
	 */
	public static function rebuild_using_access_token($access_token) {
		$r = RedisManager::connection();
		$user_id = $r->get("auth:access_token:$access_token");
		
		if (!$user_id) {
			Hub::http_error(403, "Unauthorized");
			die();
		}
		
		$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
			"SELECT * FROM users WHERE id = $user_id LIMIT 1"
		);
		
		if (!$user->is_loaded()) {
			Hub::http_error(404, "User not found");
			die();
		}
		
		$check = self::generate_access_token($user);
		
		if ($check != $access_token) {
			Hub::http_error(403, "Invalid access token");
			die();
		} else {
			return new ImmutableSession($user);
		}
	}
	
	/**
	 * Generates an access token for API authentication.
	 */
	public static function generate_access_token($user) {
		$auth = sha1("KSDFP*(D0-9sdfpiuhD" . $user->username . "FD98sydfkjh*DFhkf" . $user->password . "(*YSDHIUGHSDLF" . $user->join_date);
		$r = RedisManager::connection();
		$r->set("auth:access_token:$auth", $user->id);
		return $auth;
	}
		
	public function destroy()  {
		
		$this->session->destroy();
		
		Hub::redirect( '/' );
		
	}
	
}