<?

/**
 * Responsible for creating, showing, and deleting status messages
 */
class Status_Controller extends Controller implements REST {
	public $enable_session = true;
	public $template = 'template/main';
	
	/**
	 * Loads the public timeline. This is updates from all users.
	 */
	public function public_timeline() {
		$this->output_statuses('statuses:public_timeline');
	}
	
	/**
	 * Loads the given user's timeline. This is all updates from a single user.
	 */
	public function user_timeline() {
		if (!isset($_GET['username'])) {
			return Hub::http_error(401, "Missing required parameter: username");
		}
		
		$username = DatabaseManager::escape(trim($_GET['username']));
		if (strlen($username) === 0) {
			return Hub::http_error(401, "Missing required parameter: username");
		}
		
		$user = ActiveCache::find('User_Model', "name:$username", 43200)->sql(
			"SELECT * FROM users WHERE username = '$username' LIMIT 1"
		);
		
		if (!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		$this->output_statuses("statuses:user_timeline:{$user->id}");
	}
	
	/**
	 * Loads the friend timline of the logged in user. This is updates only
	 * from people the active user follows.
	 */
	public function friend_timeline() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$user = sess::getUser();
		$this->output_statuses("statuses:friend_timeline:{$user->id}");
	}
	
	/**
	 * Outputs the status messages with the given Redis key
	 */
	private function output_statuses($r_key) {
		$r = RedisManager::connection();
		$status_ids = $r->lrange($r_key, 0, Status_Model::$list_max_length);
		
		if (isset($_GET['count']) && is_numeric($_GET['count']) && $_GET['count'] < count($status_ids)) {
			$status_ids = array_slice($status_ids, 0, $_GET['count']);
		}
		
		$comments = false;
		if (isset($_GET['comments']) && $_GET['comments']) {
			$comments = true;
		}
		
		if (count($status_ids) > 0) {
			$statuses = array();
			foreach ($status_ids as $status) {
				$status_id = stream::id($status);
				$type = stream::type($status);
				
				$status = ActiveCache::find('Status_Model', $status_id, 43200)->sql(
					"SELECT * FROM statuses WHERE id = $status_id LIMIT 1"
				);
				
				if (!$status->is_loaded() || $status->deleted) {
					continue;
				}
				
				// HACK - UTC FIX
				$status->post_date = date('Y-m-d H:i:s', strtotime($status->post_date) - (4 * 60 * 60));
				
				if ($comments) {
					$status->load_comments();
					foreach ($status->comments as $i => $comment) {
						// HACK AGAIN, MAYBE SHOULD CENTRALIZE THIS?
						$comment->post_date = date('Y-m-d H:i:s', strtotime($comment->post_date) - (4 * 60 * 60));
						$status->comments[$i] = $comment;
					}
				}
				
				$statuses[] = $status;
			}
			
			View::respond_to(array('json', 'jsonp', 'xml'), function ($format) use($statuses) {
				if ($format == 'json' || $format == 'jsonp') {
					foreach ($statuses as $i=>$status) {
						$status->load_user();
						$statuses[$i] = $status->limited_object();
					}
					
					if ($format == 'jsonp') {
						if (isset($_GET['callback'])) {
							$callback = $_GET['callback'];
						} else {
							$callback = 'campustreamCallback';
						}
						
						echo "$callback(" . json_encode($statuses) . ");";
					} else {
						echo json_encode($statuses);
					}
				} else {
					// implement XML
				}
			});
		} else {
			View::respond_to(array('json', 'jsonp', 'xml'), function ($format) {
				if ($format == 'json') {
					echo json_encode(array());
					return;
				} elseif ($format == 'jsonp') {
					if (isset($_GET['callback'])) {
						$callback = $_GET['callback'];
					} else {
						$callback = 'campustreamCallback';
					}
					
					echo "$callback(" . json_encode(array()) . ");";
				}
			});
		}
	}
	
	/**
	 * Shows a single status message, aka status message permalink. Also
	 * loads and displays some information related to the given status message
	 * depending on its type.
	 */
	public function show($args) {
		$id = null;
		
		if (isset($args['id'])) {
			$id = base_convert($args['id'], 36, 10);
		} elseif (isset($_GET['id']) && is_numeric($_GET['id'])) {
			$id = $_GET['id'];
		} else {
			return Hub::http_error(401, "Bad request");
		}
		
		$status = ActiveCache::find('Status_Model', $id, 43200)->sql(
			"SELECT * FROM statuses WHERE id = $id LIMIT 1"
		);
		
		if (!$status->is_loaded() || $status->deleted != 0) {
			View::respond_to(array('json', 'xml'), function () {
				return Hub::http_error(404, "Status not found");
			});
			View::respond_to('html', function () {
				return Hub::redirect('/');
			});
			
			return false;
		}
		
		$status->load_user();
		$status->load_comments();
		
		View::respond_to(array('json','jsonp','xml'), function ($format) use($status) {
			echo $status->{"to_$format"}();
		});
		
		View::respond_to('html', function () use($status) {
			$status->load_content();
			
			$view = new View('statuses/single');
			$view->status = $status;
			echo $view->render();
		});
	}
	
	/**
	 * Loads all comments for the given status message.
	 */
	public function comments() {
		$id = $_GET['status_id'];
		
		if (!is_numeric($id)) {
			return Hub::http_error(401, "Bad request");
		}
		
		$status = ActiveCache::find('Status_Model', $id, 43200)->sql(
			"SELECT * FROM statuses WHERE id = $id LIMIT 1"
		);
		
		if (!$status->is_loaded()) {
			return Hub::http_error(404, "Status not found");
		}
		
		$status->load_comments();
		
		if (count($status->comments) > 0) {
			for ($i = 0; $i < count($status->comments); $i++) {
				$status->comments[$i]->load_user();
			}
		}
		
		View::respond_to(array('json', 'jsonp', 'xml'), function ($format) use($status) {
			echo $status->{"to_$format"}();
		});
	}
	
	/**
	 * Creates a new status. This should probably be create() instead of update().
	 * Not sure what I was thinking.
	 */
	public function update() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authenticated");
		}
		
		$source = 'site'; // temporary
		$message = trim($_POST['message']);
		
		if (!$message || !strlen($message)) {
			return Hub::http_error(401, "Missing status message");
		}
		
		$type = 'text';
		if (isset($_POST['type'])) {
			$type = $_POST['type'];
		}
		
		$public = true;
		if (isset($_POST['public'])) {
			$public = $_POST['public'];
		}
		
		$user = sess::getUser();
		
		$status = new Status_Model();
		$status->user_id = $user->id;
		$status->source = $source;
		$status->message = htmlspecialchars($message, ENT_QUOTES, 'UTF-8');
		$status->post_date = ActiveRecord::NOW();
		$status->type = $type;
		$status->is_public = $public;
		
		$status->save();
		
		View::respond_to(array('json', 'xml'), function($format) use($status) {
			$status->load_user();
			echo $status->{"to_$format"}();
		});
	}
	
	/**
	 * Deletes the given status update iff the update was posted by the currently
	 * authenticated user.
	 */
	public function delete() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$status_id = $_POST['id'];
		if (!$status_id || !is_numeric($status_id)) {
			return Hub::http_error(401, "Missing or invalid status ID");
		}
		
		$status = ActiveCache::find('Status_Model', $status_id, 43200)->sql(
			"SELECT * FROM statuses WHERE id = $status_id LIMIT 1"
		);
		
		if (!$status->is_loaded()) {
			return Hub::http_error(404, "Status not found");
		}
		
		if ($status->user_id != sess::getUserID()) {
			return Hub::http_error(403, "Not authorized");
		}
		
		$status->delete();
		
		View::respond_to(array('json', 'xml'), function () {
			return true;
		});
	}
}