<?
include_once $GLOBALS['APPROOT'] . 'application/lib/twitteroauth/twitteroauth.php';

/**
 * Handles Campustream's connection with Twitter. This is where the Twitter
 * information is saved when the account is connected for the first time.
 */
class Twitter_Controller extends Controller {
	public $enable_session = true;
	
	/**
	 * Begins the process of connecting a new Twitter account to a Campustream
	 * account.
	 */
	public function new_action() {
		if (!$this->session->get('authenticated')) {
			return Hub::redirect('/login');
		}
		
		$twitter = twitter::api();
		$request_token = $twitter->getRequestToken($GLOBALS['TWITTER_CALLBACK']);
		
		$this->session->set('oauth_token', $request_token['oauth_token']);
		$this->session->set('oauth_secret', $request_token['oauth_token_secret']);
		
		switch ($twitter->http_code) {
			case 200:
				if (isset($_GET['r'])) {
					$this->session->set('redirect', $_GET['r']);
				}
				
				$url = $twitter->getAuthorizeURL($this->session->get('oauth_token'));
				Hub::redirect($url);
				break;
			default:
				echo "Error connecting with Twitter, please try again! Error code: {$twitter->http_code}";
		}
	}
	
	/**
	 * Finishes the process of connecting the Twitter and Campustream accounts. This saves
	 * the user's info into the usertwitter MySQL table.
	 */
	public function create() {
		if (!$this->session->get('authenticated')) {
			return Hub::redirect('/login');
		}
		
		if (isset($_REQUEST['oauth_token']) && $this->session->get('oauth_token') !== $_REQUEST['oauth_token']) {
			$this->session->set('oauth_token', null);
			$this->session->set('oauth_secret', null);
			
			echo "Old OAuth token, please try signing in again.";
			return false;
		}
		
		$oauth_token = $this->session->get('oauth_token');
		$oauth_secret = $this->session->get('oauth_secret');
		
		$twitter = new TwitterOAuth($GLOBALS['TWITTER_CONSUMER_KEY'], $GLOBALS['TWITTER_CONSUMER_SECRET'], $oauth_token, $oauth_secret);
		$access_token = $twitter->getAccessToken($_REQUEST['oauth_verifier']);
		
		$this->session->set('oauth_token', false);
		$this->session->set('oauth_secret', false);
		
		$user = sess::getUser();
		$tweeter = $twitter->get('account/verify_credentials');
		
		$usertwitter = null;
		if ($user->has_twitter()) {
			$usertwitter = $user->twitter();
		} else {
			$usertwitter = new Usertwitter_Model();
			$usertwitter->user_id = $user->id;
			$usertwitter->twitter_id = $tweeter->id;
		}
		
		$usertwitter->oauth_token = $access_token['oauth_token'];
		$usertwitter->oauth_secret = $access_token['oauth_token_secret'];
		
		$usertwitter->username		= $tweeter->screen_name;
		$usertwitter->name 			= $tweeter->name;
		$usertwitter->website 		= $tweeter->url;
		$usertwitter->bio			= $tweeter->description;
		$usertwitter->location		= $tweeter->location;
		$usertwitter->avatar_url	= $tweeter->profile_image_url;
		$usertwitter->geo_enabled	= $tweeter->geo_enabled;
		$usertwitter->utc_offset	= $tweeter->utc_offset;
		
		$usertwitter->profile_background_color     = $tweeter->profile_background_color;
		$usertwitter->profile_text_color           = $tweeter->profile_text_color;
		$usertwitter->profile_link_color           = $tweeter->profile_link_color;
		$usertwitter->profile_sidebar_fill_color   = $tweeter->profile_sidebar_fill_color;
		$usertwitter->profile_sidebar_border_color = $tweeter->profile_sidebar_border_color;
		$usertwitter->profile_background_image_url = $tweeter->profile_background_image_url;
		$usertwitter->profile_background_tile      = 0;
		if($tweeter->profile_background_tile) { $usertwitter->profile_background_tile = 1; }
		
		$usertwitter->save();
		
		$user->twitter = $usertwitter;
		
		if ($this->session->get('redirect')) {
			$redirect = $this->session->get('redirect');
			$this->session->set('redirect', false);
			
			Hub::redirect($redirect);
		} else {
			Hub::redirect('/');
		}
	}
	
	/**
	 * Deletes a user's Twitter info from Campustream effectively deauthorizing the connection.
	 */
	public function destroy() {
		if (!$this->session->get('authenticated')) {
			return Hub::redirect('/login');
		}
		
		$user = sess::getUser();
		
		if (!isset($_GET['s']) || $_GET['s'] != md5('zOmg4tehcoolz!' . $user->username) || !$user->has_twitter()) {
			return Hub::redirect('/settings/services');
		}

		$user->twitter()->delete();
		
		Hub::redirect('/settings/services');
	}
}