<?

/**
 * Handles all interactions with users on Campustream
 */
class User_Controller extends Controller implements REST {
	public $enable_session = true;
	public $template = 'template/main';
	
	/**
	 * Authorizes a user given a username and password. Used by the API
	 * since it returns an access token.
	 */
	public function authorize() {
		$username = $_POST['username'];
		$password = $_POST['password']; // Must be sha1 hashed already
		
		if (!$username || !$password) {
			return Hub::http_error(401, "Bad request");
		}
		
		try {
			$user = User_Model::find_user_for_login($username, $password, true);
			
			$access_token = Session_Controller::generate_access_token($user);
			
			View::respond_to('html', function () use($access_token) {
				echo $access_token;
			});
			
			View::respond_to('json', function () use($access_token) {
				echo json_encode(array('access_token' => $access_token));
			});
			
		} catch (Exception $e) {
			return Hub::http_error(403, "Invalid credentials");
		}
	}
	
	/**
	 * Shows the given user's profile page.
	 */
	public function show($args) {		
		if (isset($args['username'])) {
			$username = $args['username'];
		} elseif (isset($_GET['username'])) {
			$username = $_GET['username'];
		} elseif (isset($_POST['username'])) {
			$username = $_POST['username'];
		} else {
			View::respond_to('html', function () {
				echo "Not found";
			});
			
			View::respond_to(array('json','jsonp','xml'), function () {
				Hub::http_error(401, "Missing username");
			});
			
			return false;
		}
		
		$user = ActiveCache::find('User_Model', "name:$username", 43200)->sql(
			"SELECT * FROM users WHERE username = '$username' LIMIT 1"
		);
		
		if(!$user->is_loaded()) {
			View::respond_to('html', function() {
				echo "User not found.";
			});
			View::respond_to(array('xml','json', 'jsonp'), function() {
				Hub::http_error(404, "User not found");
			});
			
			return false;
		}
		
		$template = $this->template;
		View::respond_to('html', function() use($user, $template) {
			$view = new View('user/show');
			$view->user = $user;
			
			if (sess::isActiveUser($user)) {
				$template->active = 'profile';
			}

			$template->content = $view->render();
			echo $template->render();
		});
		
		View::respond_to(array('xml','json','jsonp'), function($format) use($user) {
			echo $user->{"to_$format"}();
		});
	}
	
	/**
	 * Pretty API endpoint for redirecting to a user's avatar. Used for the API.
	 */
	public function avatar($args) {
		$size = $args['size'];
		$username = $args['username'];
		
		$user = ActiveCache::find('User_Model', "name:$username", 43200)->sql(
			"SELECT * FROM users WHERE username = '$username' LIMIT 1"
		);
		
		if (!$user->is_loaded()) {
			return Hub::http_error(404, "User not found");
		}
		
		Hub::redirect($user->avatar_url($size));
	}
	
	/**
	 * Lists users of the site.
	 */
	public function list_users() {
		sess::require_login();
		
		$view = new View('user/list');
		$this->template->title = "Users";
		$this->template->active = 'users';
		$this->template->content = $view->render();
		echo $this->template->render();
	}
	
	/**
	 * Loads all users whose names start with the given letter.
	 */
	public function show_list() {
		if (!$this->session->get('authenticated')) {
			return Hub::http_error(403, "Unauthorized");
		}

		if (!isset($_POST['letter']) || strlen($_POST['letter']) != 1) {
			return Hub::http_error(401, "Missing or invalid data");
		}
		
		$letter = strtolower($_POST['letter']);
		
		$r = RedisManager::connection();
		$user_ids = $r->smembers("users:by_first_letter:$letter");

		$users = array();
		foreach ($user_ids as $id) {
			$user = ActiveCache::find('User_Model', $id, 43200)->sql(
				"SELECT * FROM users WHERE id = $id LIMIT 1"
			);
			
			if ($user->is_loaded()) {
				$users[] = $user->limited_object();
			}
		}
		
		if (count($users) > 0) {
			util::objectSort($users, 'name');
		}
		
		View::respond_to(array('json', 'xml'), function ($format) use($users) {
			if ($format == 'xml') {
				echo xml::encode_array($users, 'users', 'user');
			} else {
				echo json_encode($users);
			}
		});
	}
	
}