<?php

/**
 * Date helper class used throughout the site.
 */
class date {

	const TIME_OFFSET = 0;

	const ONE_MIN  = 60;
	const TWO_MIN  = 120;
	const ONE_HOUR = 3600;
	const TWO_HOUR = 7200;
	const ONE_DAY  = 86400;
	const TWO_DAY  = 172800;

	const DISPLAY_ONE_MIN  = 'less than a minute ago';
	const DISPLAY_TWO_MIN  = 'about a minute ago';
	const DISPLAY_ONE_HOUR = 'about %d minutes ago';
	const DISPLAY_TWO_HOUR = 'about an hour ago';
	const DISPLAY_ONE_DAY  = 'about %d hours ago';
	const DISPLAY_TWO_DAY  = '1 day ago';
	const DISPLAY_OTHER    = '%d days ago';
	
	// Add support for future dates
	const DISPLAY_FUTURE_ONE_MIN	= 'in a minute';
	const DISPLAY_FUTURE_TWO_MIN	= 'in a few minutes';
	const DISPLAY_FUTURE_ONE_HOUR	= 'in about %d minutes';
	const DISPLAY_FUTURE_TWO_HOUR	= 'in about an hour';
	const DISPLAY_FUTURE_ONE_DAY	= 'in about %d hours';
	const DISPLAY_FUTURE_TWO_DAY	= 'in about 1 day';
	const DISPLAY_FUTURE_OTHER		= 'in %d days';
	
	/**
	 * Given a timestamp, figures out the relative time between now and then
	 *
	 * @param int|string $timestamp the timestamp to compare now with
	 * @return a string of the relative time between the two timestamps
	**/

	public static function relative_time( $timestamp ) {
		
		// Allows this function to take both a unix timestamp and a datetime string.
		if ( is_numeric( $timestamp ) === false ) {
			$timestamp = strtotime( $timestamp );
		}
		
		$delta = time() + self::TIME_OFFSET - $timestamp;
		
		if ($delta > 0) {
			if ( $delta < self::ONE_MIN ) {
				return self::DISPLAY_ONE_MIN;
			} elseif ( $delta < self::TWO_MIN ) {
				return self::DISPLAY_TWO_MIN;
			} elseif ( $delta < self::ONE_HOUR ) {
				return sprintf( self::DISPLAY_ONE_HOUR, floor( $delta / self::ONE_MIN ) );
			} elseif ( $delta < self::TWO_HOUR ) {
				return self::DISPLAY_TWO_HOUR;
			} elseif ( $delta < self::ONE_DAY ) {
				return sprintf( self::DISPLAY_ONE_DAY, floor( $delta / self::ONE_HOUR ) );
			} elseif ( $delta < self::TWO_DAY ) {
				return self::DISPLAY_TWO_DAY;
			} elseif ( $delta > self::TWO_DAY) {
				return sprintf( self::DISPLAY_OTHER, floor( $delta / self::ONE_DAY ) );
			}
		} else {
			// FUTURE DATES
			if ( $delta > -self::ONE_MIN ) {
				return self::DISPLAY_FUTURE_ONE_MIN;
			} elseif ( $delta > -self::TWO_MIN ) {
				return self::DISPLAY_FUTURE_TWO_MIN;
			} elseif ( $delta > -self::ONE_HOUR ) {
				return sprintf( self::DISPLAY_FUTURE_ONE_HOUR, abs(floor( $delta / self::ONE_MIN )) );
			} elseif ( $delta > -self::TWO_HOUR ) {
				return self::DISPLAY_FUTURE_TWO_HOUR;
			} elseif ( $delta > -self::ONE_DAY ) {
				return sprintf( self::DISPLAY_FUTURE_ONE_DAY, abs(floor( $delta / self::ONE_HOUR )) );
			} elseif ( $delta > -self::TWO_DAY ) {
				return self::DISPLAY_FUTURE_TWO_DAY;
			} elseif ( $delta < -self::TWO_DAY) {
				return sprintf( self::DISPLAY_FUTURE_OTHER, abs(floor( $delta / self::ONE_DAY )) );
			}
		}
	}

}