<?
include_once $GLOBALS['APPROOT'] . 'application/lib/postmark/Adapter_Interface.php';
include_once $GLOBALS['APPROOT'] . 'application/lib/postmark/Postmark.php';

/**
 * Email helepr class
 */
class email {
	
	/**
	 * Sends a confirmation email to the given user
	 *
	 * @param $user The user to email
	 */
	public static function send_confirmation(User_Model $user) {
		if (!$user->is_loaded()) { return; }
		
		$subject = "Welcome to Campustream! Please confirm your email.";
		$body = "
			<h1>Welcome to Campustream!</h1>
			<p>Before you can use the site, you need to confirm your email address.  Please visit this link:</p>
			<p><a href=\"http://campustream.com/confirm/do?code={$user->confirm_code}\">http://campustream.com/confirm/do?code={$user->confirm_code}</a></p>
			<p>Thanks! &mdash; the Campustream team</p>
		";
		
		self::enqueue($user, $subject, $body);
	}
	
	/**
	 * Sends a notification email to the given user
	 *
	 * @param $to_user The user to email
	 * @param $msg The message to send
	 * @param $link The link to include in the email
	 */
	public static function send_notification(User_Model $to_user, $msg, $link) {
		if (!$to_user->email_enabled) {
			return;
		}
		
		$msg = sess::getUser()->name . " " . $msg;
		$msg .= "<br />";
		$msg .= "You can view the notification <a href=\"http://campustream.com{$link}\">here</a>.";
		$msg .= "<br /><br />";
		$msg .= "Thanks!<br />";
		$msg .= "The Campustream Team";
		
		$msg .= "<br /><br />";
		$msg .= "Note: if you do not wish to receive notification emails, you may quickly disable them <a href=\"http://dev.campustream.com/settings/profile\">here</a>.";
		
		self::enqueue($to_user, "Campustream Notification", $msg);
	}
	
	/**
	 * Enqueues an email for sending by adding it to a Redis queue
	 */
	private static function enqueue($user, $subject, $body) {
		$r = RedisManager::connection();
		
		$data = array(
			'to' => $user,
			'subject' => $subject,
			'body' => $body
		);
		
		Logger::debug("Enqueuing email to {$user->username}");
		
		$r->rpush('queue:email', serialize($data));
	}
	
	/**
	 * General email sending helper. Used by the email queue worker.
	 * Uses Postmark email service for sending emails.
	 */
	public static function send(User_Model $to, $subject, $body) {
	    
	    $message = "
	    <html>
	    	<head><title>Campustream</title></head>
	    	<body>
	    		$body
	    	</body>
	    </html>
	    ";
	    
	    Logger::email('Sent', "{$to->name} <{$to->email}> - $subject");
	    
	    try {
			Mail_Postmark::compose()
				->addTo($to->email, $to->name)
				->subject($subject)
				->messageHtml($message)
				->send();
		} catch (Exception $e) {
			Logger::email('Error', $e->getMessage());
		}
	}
}

/**
 * Our implementation of Postmark
 */
class Mail_Postmark_Adapter implements Mail_Postmark_Adapter_Interface {
	public static $latestLogEntry;
	
	public static function getApiKey() {
		return "400edfd8-5598-4a39-b513-a4f31543bf8e";
	}
	
	public static function setupDefaults(Mail_Postmark &$mail) {
		$mail->from('noreply@campustream.com', 'Campustream');
	}
	
	public static function log($logData) {
		self::$latestLogEntry = $logData;
	}
}