<?

/**
 * Helper class for managing the site streams
 */
class stream {

	// Define the different stream types
	const STREAM_PUBLIC = 1;
	const STREAM_FRIENDS = 2;
	const STREAM_USER = 3;
	
	// Define the different data types
	const TYPE_STATUS = 'text';
	const TYPE_QUESTION = 'question';
	const TYPE_RESPONSE = 'response';
	const TYPE_NEWSEVENT = 'newsevent';
	
	// Max length for the stream
	const STREAM_LENGTH = 200;
	
	/**
	 * Publish a message to a site stream
	 *
	 * @param $stream The stream to publish to
	 * @param $type What type of status message is it?
	 * @param $id The MySQL ID for the status message
	 */
	public static function publish($stream, $type, $id) {
		$stream_data = array(
			'type' => $type,
			'id' => $id
		);
		
		$stream_data = serialize($stream_data);
		
		$stream_key = null;
		switch ($stream) {
			case self::STREAM_PUBLIC:
				$stream_key = 'statuses:public_timeline';
				break;
			case self::STREAM_FRIENDS:
				$user = sess::getUser();
				$followers = $user->followers();
				$followers[] = $user->id;
				
				$stream_key = array();
				foreach ($followers as $follower) {
					$stream_key[] = "statuses:friend_timeline:$follower";
				}
				
				break;
			case self::STREAM_USER:
				$user = sess::getUser();
				$stream_key = "statuses:user_timeline:{$user->id}";
				break;
		}
		
		if (!$stream_key) {
			return false;
		}
		
		if (is_array($stream_key)) {
			foreach ($stream_key as $key) {
				self::broadcast($key, $stream_data);
			}
		} else {
			self::broadcast($stream_key, $stream_data);
		}
	}
	
	private static function broadcast($stream_key, $data) {
		$r = RedisManager::connection();
		
		$id = self::id($data);
		Logger::debug("publishing to stream $stream_key, id $id");
		$num = $r->lpush($stream_key, $data);
		if ($num > self::STREAM_LENGTH) {
			$r->ltrim($stream_key, 0, self::STREAM_LENGTH);
		}
	}
	
	/**
	 * Given some stream data, return the stream data's ID
	 */
	public static function id($data) {
		$data = unserialize($data);
		return $data['id'];
	}
	
	/**
	 * Given some stream data, return the stream data's type
	 */
	public static function type($data) {
		$data = unserialize($data);
		return $data['type'];
	}

}