<?

/**
 * Represents a single question
 */
class Question_Model extends ActiveRecord {
	public $table_name = 'questions';
	public $columns = array('id', 'user_id', 'category', 'title', 'message', 'post_date', 'last_update', 'vote_count', 'is_public', 'deleted');
	public $cache_keys = array( 'question/#' => 'id', 'questions/user:#' => 'user_id' );
	public $has_many = array('responses', 'tags');
	public $has_one = array('user');
	public $public_columns = array( 'id', 'short_id', 'user_id', 'category', 'title', 'message', 'post_date', 'last_update', 'vote_count', 'user', 'num_responses', 'responses', 'tags' );
	
	public static $categories = array('academic', 'social', 'misc', 'happenings');
	
	/**
	 * Load the user that asked the question
	 */
	public function load_user() {
		if (!$this->user) {
			$user = ActiveCache::find('User_Model', $this->user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = {$this->user_id} LIMIT 1"
			);
			
			if ($user->is_loaded()) {
				$this->add_relationship('user', $user);
			}
		}
	}
	
	/**
	 * Load all responses to this question
	 */
	public function load_responses() {
		if (!$this->responses) {
			$responses = ActiveCache::find_all('Response_Model', "question:{$this->id}", 43200)->sql(
				"SELECT * FROM responses WHERE question_id = {$this->id} ORDER BY vote_count DESC"
			);
			
			$this->add_relationship('responses', $responses);
		}
	}
	
	/**
	 * Return the number of responses to this question
	 */
	public function num_responses() {
		$this->load_responses();
		return count($this->responses);
	}
	
	/**
	 * Load all tags associated with this question
	 */
	public function load_tags() {
		if (!$this->tags) {
			$tags = ActiveCache::find_all('Questiontag_Model', "question:{$this->id}", 43200)->sql(
				"SELECT * FROM questiontags WHERE question_id = {$this->id}"
			);
			
			if (count($tags) > 0) {
				$this->add_relationship('tags', $tags);
			}
		}
	}
	
	/**
	 *  Save the question and post a corresponding status update
	 */
	public function save($is_new = true) {
		// Save to MySQL first
		parent::save();
		
		if (!$is_new) {
			return true;
		}
		
		// and save it to Redis
		$r = RedisManager::connection();		
		$num = $r->lpush("questions:stream:{$this->category}:newest", $this->id);
		if ($num > 200) {
			$r->ltrim("questions:stream:{$this->category}:newest", 0, 200);
		}
		
		// then create the status message for it
		$status = new Status_Model();
		$status->user_id = $this->user_id;
		$status->source = 'site';
		$status->message = $this->title;
		$status->post_date = ActiveRecord::NOW();
		$status->type = 'question';
		$status->content_id = $this->id;
		$status->is_public = $this->is_public;
		$status->save();
	}
	
	/**
	 * Soft-delete this question (for data archiving purposes)
	 */
	public function delete() {
		$this->deleted = 1;
		$this->save(false);
		
		$r = RedisManager::connection();
		$r->lrem("questions:stream:{$this->category}:newest", 0, $this->id);
		
		return true;
	}
	
	public function __get($key) {
		switch ($key) {
			case 'short_id': return base_convert($this->id, 10, 36);
			case 'num_responses': return $this->num_responses();
		}
		
		return parent::__get($key);
	}
}