<?

/**
 * Represents a single comment under a single status
 */
class Statuscomment_Model extends ActiveRecord {
	public $table_name = 'statuscomments';
	public $columns = array('id', 'user_id', 'status_id', 'message', 'post_date', 'deleted');
	public $cache_keys = array( 'statuscomment/#' => 'id', 'statuscomments/user:#' => 'user_id', 'statuscomments/status:#' => 'status_id');
	public $has_many = array();
	public $has_one = array('status', 'user');
	public $public_columns = array( 'id', 'user_id', 'status_id', 'message', 'post_date', 'user' );
	
	/**
	 * Load the user who posted the comment
	 */
	public function load_user() {
		if (!$this->user) {
			$user = ActiveCache::find('User_Model', $this->user_id, 43200)->sql(
				"SELECT * FROM users WHERE id = {$this->user_id} LIMIT 1"
			);
			
			if ($user->is_loaded()) {
				$this->add_relationship('user', $user);
			}
		}
	}
	
	/**
	 * Load the status that this comment was posted under
	 */
	public function load_status() {
		if (!$this->status) {
			$status = ActiveCache::find('Status_Model', $this->status_id, 43200)->sql(
				"SELECT * FROM statuses WHERE id = {$this->status_id} LIMIT 1"
			);
			
			if ($status->is_loaded()) {
				$this->add_relationship('status', $status);
			}
		}
	}
	
	/**
	 * Save this status comment and send a notification
	 */
	public function save() {
		parent::save();
		
		ActiveCache::invalidate_keys_for($this);
		
		$this->load_status();
		
		if ($this->user_id == $this->status->user_id) {
			return;
		}
		
		$n = notifications::load();
		$n->send($this->status->user_id, 'reply', "has replied to your status message.", $this->status->short_id);
		
		$this->status->load_user();
		email::send_notification($this->status->user, "has replied to your status message.", "/s/" . $this->status->short_id);
	}
	
	public function __get($key) {
		switch ($key) {
			case 'user':
				$this->load_user();
				return parent::__get('user');
		}
		
		return parent::__get($key);
	}
}