<?php

/**
 * Represents a single user, which also represents sessions in Hub
 */
class User_Model extends ActiveRecord implements SessionObject {
	
	public $columns = array('id', 'username', 'name', 'bio', 'website', 'college_majors', 'college_minors', 'class_year', 'email', 'password', 'join_date', 'has_avatar', 'avatar_format', 'current_status', 'karma', 'confirmed', 'confirm_code', 'email_enabled');
	public $cache_keys = array( 'user/#' => 'id', 'user/name:#' => 'username' );
	public $has_many = array();
	public $public_columns = array( 'id', 'username', 'name', 'bio', 'website', 'college_majors', 'college_minors', 'class_year', 'join_date', 'current_status', 'follows_count', 'followers_count', 'avatar', 'karma');
	
	private $twitter, $facebook;
	private static $active_user;
  
	public static function get_session_object($user_id) {
		if(!is_numeric($user_id)) {
			return new User_Model;
		}
		
		if (self::$active_user) {
			return self::$active_user;
		}
		
		$user = ActiveCache::find('User_Model', $user_id, 43200)->sql(
			"SELECT * FROM users WHERE id = $user_id LIMIT 1"
		);
		
		self::$active_user = $user;
		
		return $user;
	}
	
	public function get_session_params() {
		return array('user_id' => $this->id);
	}
	
	public function get_session_secret() {
		return md5($this->password);
	}
	
	public function get_session_key() {
		return $this->id;
	}
  
  /**
   * Given a username and password, find the corresponding user. If the username
   * and/or password is invalid, throw an exception.
   */
	public static function find_user_for_login($username, $password, $is_hashed = false) {
		
		$username = DatabaseManager::escape($username);
		$user = ActiveRecord::find('User_Model', "SELECT * FROM users WHERE username = '$username' LIMIT 1");
		
		if (!$is_hashed) {
			$password = sha1($password);
		}
		
		if(!$user->is_loaded()) {
			throw new Exception("Invalid username");
		} elseif($user->password != $password) {
			throw new Exception("Incorrect password");
		} else {
			return $user;
		}
	}
	
	/**
	 * Load and return Facebook information related to this user
	 */
	public function facebook() {
		if ($this->facebook === null) {
			$facebook = ActiveCache::find('Userfacebook_Model', "user:{$this->id}", 43200)->sql(
				"SELECT * FROM userfacebook WHERE user_id = {$this->id} LIMIT 1"
			);
			
			if (!$facebook->is_loaded()) {
				return new Userfacebook_Model();
			} else {
				$this->facebook = $facebook;
			}
		}
		
		return $this->facebook;
	}
	
	/**
	 * Has this user connected their Facebook account?
	 */
	public function has_facebook() {
		return isset($this->facebook()->access_token);
	}
	
	/**
	 * Create a Facebook API instance using thi user's account
	 */
	public function facebook_api() {
		if (!$this->has_facebook()) {
			return false;
		}
		
		$facebook = new Facebook(array(
			'appId' => FACEBOOK_APP_ID,
			'secret' => FACEBOOK_SECRET,
			'cookie' => true
		));
		
		return $facebook;
	}
	
	/**
	 * Prepares a Facebook request
	 */
	public function fb_prepare($data) {
		if (!$this->has_facebook()) {
			return $data;
		}
		
		if (!$data['access_token']) {
			$data['access_token'] = $this->facebook()->access_token;
		}
		
		return $data;
	}
	
	/**
	 * Load and return Twitter information about this user
	 */
	public function twitter() {
		if ($this->twitter === null) {
			$twitter = ActiveCache::find('Usertwitter_Model', "user:{$this->id}", 43200)->sql(
				"SELECT * FROM usertwitter WHERE user_id = {$this->id} LIMIT 1"
			);
			
			if (!$twitter->is_loaded()) {
				return new Usertwitter_Model();
			} else {
				$this->twitter = $twitter;
			}
		}
		
		return $this->twitter;
	}
	
	/**
	 * Has this user connected their Twitter account?
	 */
	public function has_twitter() {
		return isset($this->twitter()->oauth_token);
	}
	
	/** 
	 * Retrieve a Twitter API connection with this user's account
	 */
	public function twitter_api() {
		if (!$this->has_twitter()) {
			return false;
		}
		
		return twitter::api($this);
	}
	
	/**
	 * Retrieve the URL to this users avavar
	 *
	 * @param $size Small, medium or large
	 * @param $placeholder Force returning a placeholder avatar
	 */
	public function avatar_url($size = 'medium', $placeholder = false) {
		if ($this->has_avatar && !$placeholder) {
			return 'http://cdn.campustream.com/images/avatars/' . md5('PRobleM?' . $this->username) . "-$size.{$this->avatar_format}";
		} else {
			return "http://cdn.campustream.com/images/avatars/noavatar-$size.jpg";
		}
	}
	
	/**
	 * Return the absolute path to the avatar directory
	 */
	public function avatar_path($size = 'medium', $format = false) {
		if (!$format) {
			$format = $this->avatar_format;
		}
		
		return $GLOBALS['CDNROOT'] . 'images/avatars/' . md5('PRobleM?' . $this->username) . "-$size.$format";
	}
	
	/* Social graph methods */
	
	/**
	 * Follow the given user
	 */
	public function follow($user) {
	    if ($user instanceof User_Model) {
	    	$user = $user->id;
	    }
	    
	    $flock = FlockDBManager::connection();
	    $flock->add($this->id, 'follows', $user);
	}
	
	/**
	 * Unfollow the given user
	 */
	public function unfollow($user) {
		if ($user instanceof User_Model) {
			$user = $user->id;
		}
		
		$flock = FlockDBManager::connection();
		$flock->remove($this->id, 'follows', $user);
	}
	
	/**
	 * Get all users this user follows
	 */
	public function follows() {
		try {
		    $flock = FlockDBManager::connection();
		    $cursor = $flock->select($this->id, 'follows', null);
		    return iterator_to_array($cursor, false);
		} catch (Exception $e) {
			return array();
		}
	}
	
	/**
	 * Get all users who follow this user
	 */
	public function followers() {
		try {	
		    $flock = FlockDBManager::connection();
		    $cursor = $flock->select(null, 'follows', $this->id);
		    return iterator_to_array($cursor, false);
		} catch (Exception $e) {
			return array();
		}
	}
	
	/**
	 * Is this user followed by the given user?
	 */
	public function is_followed_by($user) {
	    if ($user instanceof User_Model) {
	    	$user = $user->id;
	    }
	    
	    $flock = FlockDBManager::connection();
	    $cursor = $flock->select($user, 'follows', $this->id);
	    return (count($cursor->currentPage()) > 0);
	}
	
	/**
	 * Is this user following the given user?
	 */
	public function is_following($user) {
	    if ($user instanceof User_Model) {
	    	$user = $user->id;
	    }
	    
	    $flock = FlockDBManager::connection();
	    $cursor = $flock->select($this->id, 'follows', $user);
	    return (count($cursor->currentPage()) > 0);
	}
	
	/**
	 * How many users does this user follow?
	 */
	public function follows_count() {
		try {
		    $flock = FlockDBManager::connection();
		    return $flock->size($this->id, 'follows', null);
		} catch (Exception $e) {
			return 0;
		}
	}
	
	/**
	 * How many users follow this user?
	 */
	public function followers_count() {
		try {
		    $flock = FlockDBManager::connection();
		    return $flock->size(null, 'follows', $this->id);
		} catch (Exception $e) {
			return 0;
		}
	}
	
	private function summary() {
		$info = array();
		if (strlen($this->college_majors)) {
			$info[] = "Major(s): {$this->college_majors}";
		}
		if (strlen($this->college_minors)) {
			$info[] = "Minors(s): {$this->college_minors}";
		}
		if ($this->class_year) {
			$info[] = "Class of {$this->class_year}";
		}
		
		return implode(" &mdash; ", $info);
	}
	
	public function __get($key) {
		switch ($key) {
			case 'follows_count': return $this->follows_count();
			case 'followers_count': return $this->followers_count();
			case 'avatar': return $this->avatar_url();
			case 'website_formatted':
				if(mb_strlen($this->website) > 0) {
					$website = '<a class="nav-link" rel="nofollow" target="_blank" href="'.str_replace(array('"', "'"), "", $this->website).'">';
					if(mb_strlen($this->website) > 30) {
						$website .= mb_substr($this->website, 0, 30)."...";
					} else {
						$website .= $this->website;
					}
					$website .= "</a>";
					return $website;
				} else {
					return '';
				}
			case 'summary': return $this->summary();
		}
		
		return parent::__get($key);
	}
	
	public function __set($key, $value) {
		if (!isset($this->$key)) {
			$this->$key = $value;
		}
		
		return parent::__set($key, $value);
	}
}