<?

group('analyze');

desc('Analyze the social graph');
task('social', function ($args) {

	force_execution_on(any_web_server);

	$num = 0;
	if (count($args)) {
		$num = array_shift($args);
	}

	echo "Retrieving all users\n";
	$users = ActiveRecord::find_all("User_Model", "SELECT * FROM users");
	
	if (!count($users)) {
		echo "Error retrieving users\n";
		return false;
	}
	
	$user_follows_stats = array();
	$user_followers_stats = array();
	$user_posts = array();
	
	echo "Getting social graph stats\n";
	foreach ($users as $user) {
		$user_follows_stats[$user->username] = $user->follows_count();
		$user_followers_stats[$user->username] = $user->followers_count();
		
		$result = DatabaseManager::readConnection()->query("SELECT COUNT(*) as count FROM statuses WHERE user_id = {$user->id}");
		if ($result) {
			$data = $result->fetch_assoc();
			$user_posts[$user->username] = $data['count'];
		} else {
			$user_posts[$user->username] = 0;
		}
	}
	
	echo "Sorting results\n";
	arsort($user_follows_stats);
	arsort($user_followers_stats);
	arsort($user_posts);
	
	if ($num > 0) {
		$follows_stats = array_slice($user_follows_stats, 0, $num, true);
		$followers_stats = array_slice($user_followers_stats, 0, $num, true);
	} else {
		$follows_stats = $user_follows_stats;
		$followers_stats = $user_followers_stats;
	}
	
	echo "##############################\n";
	echo "Follows Stats\n";
	echo "username, posts, follows, followed by\n";
	foreach ($follows_stats as $username => $num) {
		echo "$username, {$user_posts[$username]}, $num, {$user_followers_stats[$username]}\n";
	}
	
	echo "##############################\n";
	echo "Follower Stats\n";
	echo "username, posts, follows, followed by\n";
	foreach ($followers_stats as $username => $num) {
		echo "$username, {$user_posts[$username]}, {$user_follows_stats[$username]}, $num \n";
	}
	
	echo "##############################\n";
	echo "Post Stats\n";
	echo "username, posts, follows, followed by\n";
	foreach ($user_posts as $username => $num) {
		echo "$username, {$num}, {$user_follows_stats[$username]}, {$user_followers_stats[$username]}\n";
	}
});

desc("Analyze mutual relationships between users");
task("mutual", function () {

	force_execution_on( any_web_server );
	
	echo "Retrieving all users\n";
	$users = ActiveRecord::find_all("User_Model", "SELECT * FROM users");
	
	if (!count($users)) {
		echo "Error retrieving users\n";
		return false;
	}
	
	$data = array();
	$failed = array();
	
	echo "Analyzing the social graph\n";
	foreach ($users as $user) {
		echo "Checking user {$user->username}\n";
		$data[$user->username] = 0;
		
		foreach ($users as $other_user) {
			try {
				if ($user->is_following($other_user) && $user->is_followed_by($other_user)) {
					$data[$user->username]++;
				}
			} catch (Exception $e) {
				echo "Exception caught for {$user->username}, will try again later\n";
				$failed[] = $user;

				break;
			}
		}
	}
	
	if (count($failed) > 0) {
		echo count($failed) . " users failed, trying again\n";
		foreach ($failed as $user) {
			echo "Checking user {$user->username}\n";
			$data[$user->username] = 0;
			
			foreach ($users as $other_user) {
				try {
					if ($user->is_following($other_user) && $user->is_followed_by($other_user)) {
						$data[$user->username]++;
					}
				} catch (Exception $e) {
					echo "Exception caught again for {$user->username}, giving up.";
					
					break;
				}
			}
		}
	}
	
	echo "##############################\n";
	echo "Mutual Followers\n";
	echo "username, mutual connections\n";
	foreach ($data as $username => $num) {
		echo "$username, $num\n";
	}

});