<?php

class Hub {
	
	public static $activecontroller = NULL;
	public static $response_format  = NULL;
	
	public static $mime_types = array( 'xml' => 'text/xml', 'html' => 'text/html', 'json' => 'application/json', 'rss' => 'application/rss+xml', 'jsx' => 'application/javascript', 'js' => 'text/javascript' );
  
	
	public static function run() {
		
		Hub::setup();
		
		if(!iprules::allowed()) {
			return Hub::http_error(403, "Forbidden");
		}
		
		Hub::determine_response_format();		
		Router::process();
		
	}
	
	// determines the type of response the requestor wants based on the extension
  // of the url. then does some path mangling so it's transparent to the router

  public static function determine_response_format() {
    // try to get the position of the extension, if it exists
    $ext_pos = strrpos( $_GET['hub_uri'], '.' );

    // set the response type as html by default
    if ( $ext_pos === false ) {
      self::$response_format = 'html';
      return;
    }

    // grab everything after the "." in the uri
    self::$response_format = substr( $_GET['hub_uri'], $ext_pos+1 );

    // hack to make .do extensions compatible with this kind of api
    if ( self::$response_format == 'do' ) self::$response_format = 'html';

    // remove the extension from the uri, so the router doesn't have to worry about it
    $_GET['hub_uri'] = substr( $_GET['hub_uri'], 0, $ext_pos );

  }
	
	public static function setup() {
		
		spl_autoload_register(array('Hub', 'auto_load'));
		set_error_handler(array('Hub', 'error_handler'));
		set_exception_handler(array('Hub', 'exception_handler'));
		
	}
	
	public static function redirect($url) {
		header("Location: $url");
	}
	
	public static function send404() {
		header("HTTP/1.0 404 Not Found");
	}
	
	function browserInfo($agent = null) {
	  	// Declare known browsers to look for
	  	$known = array('msie', 'firefox', 'safari', 'webkit', 'opera', 'netscape', 'konqueror', 'gecko');

	  	// Clean up agent and build regex that matches phrases for known browsers
	  	// (e.g. "Firefox/2.0" or "MSIE 6.0" (This only matches the major and minor
	  	// version numbers.  E.g. "2.0.0.6" is parsed as simply "2.0"
	  	$agent = strtolower($agent ? $agent : $_SERVER['HTTP_USER_AGENT']);
	  	$pattern = '#(?<browser>' . join('|', $known) . ')[/ ]+(?<version>[0-9]+(?:\.[0-9]+)?)#';

	  	// Find all phrases (or return empty array if none found)
	  	if (!preg_match_all($pattern, $agent, $matches)) return array();

	  	// Since some UAs have more than one phrase (e.g Firefox has a Gecko phrase,
	  	// Opera 7,8 have a MSIE phrase), use the last one found (the right-most one
	  	// in the UA).  That's usually the most correct.
	  	$i = count($matches['browser'])-1;
	
		return array('name' => $matches['browser'][$i], 'version' => $matches['version'][$i]);
	}
	
	public static function is_ajax() {
		return $_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest';
	}
	
	public static function cache_headers( $unique_id ) {
		header("Last-Modified: ".gmdate("D, d M Y H:i:s", time()-10000)." GMT");
		header("Etag: {$unique_id}");
		
		if ( stripos( $_SERVER['REQUEST_METHOD'], 'HEAD' ) === true ) {
			header('HTTP/1.1 304 Not Modified');
			exit();
		}
		
		header("Cache-Control: maxage=10000");
		header('Expires: ' . gmdate('D, d M Y H:i:s', time()+10000) . ' GMT');
		header('Pragma: public');
	}
	
	public static function auto_load($class) {
		#echo "<br>Loading: $class<br>";
		
		$lib = true;
		if(strpos($class,'_') !== false) $lib = false;
		
		if($lib) {
			
			$classfilename = strtolower($class) . '.php';
			
			if(file_exists($GLOBALS['APPROOT'] . 'core/lib/' . $classfilename )) {
				
				include($GLOBALS['APPROOT'] . 'core/lib/' . $classfilename);
				return;
				
			} elseif(file_exists($GLOBALS['APPROOT'] . 'application/lib/' . $classfilename )) {
				
				include($GLOBALS['APPROOT'] . 'application/lib/' . $classfilename);
				return;
				
			}
			
		} else {
			
			$cparts 		= explode('_', $class);
			$type 			= strtolower($cparts[1]);
			$classfilename 	= strtolower($cparts[0]) . '.php';
			
			if($type == 'controller') {
				
				include($GLOBALS['APPROOT'] . 'application/controllers/' . $classfilename);
				return;
				
			} elseif($type == 'view') {
				
				include($GLOBALS['APPROOT'] . 'application/views/' . $classfilename);
				return;
				
			} elseif($type == 'model') {
				
				include($GLOBALS['APPROOT'] . 'application/models/' . $classfilename);
				return;
				
			}
				
		}
		
		
	}
	
	public static function http_error( $code, $message=false ) {
		
		switch ( $code ) {
			case 400:
			  header( "HTTP/1.1 400 Bad Request" );
			  break;
			case 401:
				header( "HTTP/1.1 401 Unauthorized" );
				break;
			case 403:
				header( "HTTP/1.1 403 Forbidden" );
				break;
			case 404:
				header( "HTTP/1.1 404 Not Found" );
				break;
		}
				
		View::respond_to( 'html', function() {
			
			$view = new View( 'error/404_general' );
						
			if ( Hub::$activecontroller !== null ) {
				Hub::$activecontroller->template->content = $view->render();
				Hub::$activecontroller->template->page_title = "Page could not be found - Twitpic";
				echo Hub::$activecontroller->template->render();
			} else {
				echo $view->render();
			}
			
		});
		
		View::respond_to( array('json','jsonp','xml'), function($format) use ($code, $message) {
			
			$response = new stdClass;

			$response->code    = $code;
			$response->message = $message;

			echo View::factory( "error/error.{$format}" )->set( "response", $response )->render();
			
		});
		
		return Hub::shutdown();
		
	}
	
	public static function exception_handler( $exception ) {
		header('HTTP/1.1 500 Internal Server Error');
		Logger::debug("--> ERROR: " . $exception->getMessage());
		include $GLOBALS['APPROOT'] . 'www/50x.html';
	}
	
	public static function error_handler($errno, $errstr, $errfile, $errline) {
		//Logger::debug("--> ERROR: $errstr");
		return true;
	}
	
	public static function shutdown() {
		// future cleanup code- ideally, all requests should end here...
		exit;
	}
	
		
}