<?php

class ActiveCache {
	
	private $run_sql    = false;
 	private $find_all   = false;
 	private $result_set = null;
	private $object     = null;
	private static $mem_cache	= array();
	
	public static function find( $class, $cache_key, $expire=0 ) {
		
		$memcache = MemcacheManager::connection();
		
		$table_name = ActiveRecord::model_to_table_name( $class );
		$cache_key  = substr($table_name, 0, -1) . '/' . $cache_key;
		
		Logger::debug( "looking up cache key $cache_key");
		
		if (!empty(self::$mem_cache[$cache_key])) {
			Logger::debug( "using local cache key $cache_key" );
			return new ActiveCache( $class, $cache_key, $expire, self::$mem_cache[$cache_key]);
		} elseif ( ( $result = $memcache->get( $cache_key ) ) !== false ) {
			return new ActiveCache( $class, $cache_key, $expire, $result );
		} else {
			return new ActiveCache( $class, $cache_key, $expire );
		}
		
	}
	
	public static function find_all( $class, $cache_key, $expire=0 ) {
		
		$memcache = MemcacheManager::connection();
		
		$table_name = ActiveRecord::model_to_table_name( $class );
		$cache_key  = $table_name . '/' . $cache_key;
		
		Logger::debug( "looking up cache key $cache_key");
		
		
		if (!empty(self::$mem_cache[$cache_key])) {
			Logger::debug( "using local cache key $cache_key" );
			return new ActiveCache( $class, $cache_key, $expire, self::$mem_cache[$cache_key]);
		} elseif ( ( $result = $memcache->get( $cache_key ) ) !== false ) {
			return new ActiveCache( $class, $cache_key, $expire, $result, true );
			Logger::debug( "found cache key $cache_key");
		} else {
			return new ActiveCache( $class, $cache_key, $expire, null, true );
		}
		
	}
	
	public static function invalidate_keys_for( ActiveRecord $object ) {
		$memcache = MemcacheManager::connection();
				
		foreach ( $object->cache_keys as $cache_key => $replace ) {
			if ( $replace !== null ) $cache_key = str_replace( '#', $object->{$replace}, $cache_key );
			$memcache->delete( $cache_key );
			Logger::debug( "deleting $cache_key" );
		}
	}
	
	public function __construct( $class, $cache_key, $expire=0, $result=null, $find_all=false ) {
		$this->class = $class;
		$this->cache_key = $cache_key;
		
		if ( $expire == 0 ) {
			$this->expire = 0;
		} else {
			$this->expire = $expire + time();
		}
		// used to keep track if being called w/ find_all or find
		$this->find_all = $find_all;
		if ( $result === null ) {
			$this->run_sql = true;
			return; 
		}
		
		$this->result_set = $result;
		
	}
	
	public function sql( $clause ) {
		// only run the sql if nothing was found in the cache
		if ( $this->run_sql === true ) {
			
			$memcache = MemcacheManager::connection();
			
			if ( $this->find_all === false ) {
				$this->result_set = ActiveRecord::find( $this->class, $clause );
				if ( $this->result_set->is_loaded() === false ) return $this->result_set; // we don't want to cache an empty object 
			} else {
				$this->result_set = ActiveRecord::find_all( $this->class, $clause );
			}
			
			Logger::debug( "setting cache key $this->cache_key");
			$memcache->set( $this->cache_key, $this->result_set, $this->expire);
			
		}
		
		self::$mem_cache[$this->cache_key] = $this->result_set;
		
		// subscribe to save() and delete() events, which will call
		// ActiveCache->notify and invalidate cache keys automatically.
		
		if ( is_array( $this->result_set ) ) {
			foreach( $this->result_set as $result ) {
				$result->cache_keys[ $this->cache_key ] = null;
				$result->subscribe( $this );
			}
		} else {
			$this->result_set->cache_keys[ $this->cache_key ] = null;
			$this->result_set->subscribe( $this );
		}
		
		$result_set = $this->result_set;
		$this->result_set = null;
		
		
		return $result_set;
	
	}
	
	public function notify( ActiveRecord $object ) {
		ActiveCache::invalidate_keys_for( $object );
	}
}