<?php

class ActiveQueue {
	
	const FINISHED = 3;
	const RETRY    = 4;
	const FAILED   = 5;
	
	const MAX_RETRIES = 5;
	
	const OPT_RETRY_QUEUE = 'retry_queue';
	
	public static $kestrel = null;
	
	private $options = array(ActiveQueue::OPT_RETRY_QUEUE => false);
	
	private $queue_name;
	
	public static function kestrel($server='localhost', $port="22133") {
		
		if (self::$kestrel === null) {
		
			self::$kestrel = new Memcached();
			self::$kestrel->setOption( Memcached::OPT_CONNECT_TIMEOUT, 3000 );
			self::$kestrel->setOption( Memcached::OPT_COMPRESSION, false );
			self::$kestrel->addServer( $server, $port, 1 );
			
		}
		
		return self::$kestrel;
		
	}
	
	public function setOption($option, $value) {
		$this->options[$option] = $value;
	}
	
	public function __construct( $name ) {
		$this->queue_name = $name;
	}
	
	public function push( $payload, $metadata=array() ) {
		
		$item = json_encode( array( get_class( $payload ), $payload->id, 0, $metadata ) );
		self::kestrel()->set($this->queue_name, $item, 0 );
	}
	
	public function process( $seconds, $block ) {
		
		$start = time();
		
		$kestrel = self::kestrel();
		
		while ( $seconds >= (time() - $start) ) {
			
			$head = $kestrel->get("{$this->queue_name}/t=2000/open");
			
			if ( $head === false ) {
				// Queue is empty.
				sleep(2);
				continue;
			}
			
			list( $class, $id, $attempts, $metadata ) = json_decode( $head );
			
			if ( $attempts > self::MAX_RETRIES ) {
				$kestrel->get( "{$this->queue_name}/close" );
				continue;
			}
			
			$attempts++;
			
			if ($class !== "stdClass") {
				$object = ActiveRecord::find( $class, $id );
			
				// Payload is invalid, probably due to slave lag
				// Remove it from the top and reinsert it.
				if ( $object->is_loaded() === false ) {
					$kestrel->get( "{$this->queue_name}/close" );
					sleep(2);
					$item = json_encode( array( $class,$id,$attempts,$metadata) );
					$kestrel->set($this->queue_name, $item, 0 );
					continue;
				}
			
			} else {
				$object = new stdClass;
				$object->id = $id;
			}
						
			$status = $block( $object, $metadata );
			
			if ( $status === ActiveQueue::FAILED ) {
				$kestrel->get( "{$this->queue_name}/close" );
			} elseif ( $status === ActiveQueue::RETRY ) {
				$kestrel->get( "{$this->queue_name}/close" );
				
				if ($this->options[self::OPT_RETRY_QUEUE] === true) {
					$kestrel->set($this->queue_name . "_retry", json_encode(array($class,$id,$attempts,$metadata)), 60 );
				} else {
					$kestrel->set($this->queue_name, json_encode(array($class,$id,$attempts,$metadata)), 0 );
				}
				
			} elseif ( $status === ActiveQueue::FINISHED ) {
				$kestrel->get( "{$this->queue_name}/close" );
			}
				
		}	
	}	
}
