<?php

class Session {
	
	private $prefix = '';
	private $domain = '';
	private $sess_object = '';
	
	function __construct() {
		
		require($GLOBALS['APPROOT'] . 'application/config/sessions.php');
		
		if(!empty($SESSION_HANDLER) && strlen($SESSION_HANDLER) > 0) {
			ini_set('session.save_handler',  $SESSION_HANDLER);
			ini_set('session.gc_maxlifetime', 86400);
			ini_set('session.save_path', $SESSION_SAVE_PATH);
		}
		
		$this->prefix = $SESSION_PREFIX;
		$this->domain = $SESSION_DOMAIN;
		$this->sess_object = $SESSION_OBJECT;
		
		session_set_cookie_params( time() + 86400, '/', $this->domain);
		session_name('hub'); // increment the session version if you want to kill old sessions
		session_start();
		
		if ( isset( $_COOKIE['hub_auth1'] ) && ! $this->get('authenticated') ) {
			$this->rebuild_session();
		}
	}
	
	public function rebuild_session() {
		
		$cookie = SignedCookie::unserialize('hub_auth1');

		$sess_key = DatabaseManager::escape( $cookie['sess_key'] );
		$class = $this->sess_object;
		$sess_obj = $class::get_session_object($sess_key);
		
		if ( ! $sess_obj->is_loaded() ) {
			// someone is messing with the cookie or it's corrupted, so let's erase it.
			setcookie( 'hub_auth1', false, time()-42000, '/', $this->domain );
			return false;
		}
		
		$secrets = array( 'sess_secret' => $sess_obj->get_session_secret() );
		
		if ( SignedCookie::verify($cookie, $secrets) === false ) {
			// someone is messing with the cookie or it's corrupted, so let's erase it.
			setcookie( 'hub_auth1', false, time()-42000, '/', $this->domain );
			return false;
		}
		
		$this->set_session($user);
		
	}
	
	public function set_session(SessionObject $obj, $cookie=true) {
		
		if(!$obj->is_loaded()) {
			return false;
		}

		// Turn this session into an authenticated one
		$this->set( 'authenticated', true );
		$this->set('sess_key', $obj->get_session_key());
		
		if($cookie) {
			$params  = $obj->get_session_params();
			$secrets = array( 'sess_secret' => $obj->get_session_secret() );
			
			SignedCookie::create( 'hub_auth1', $params, $secrets );
		}
		
		return true;
		
	}
	
	public function get_session_object() {
		$class = $this->sess_object;
		return $class::get_session_object($this->get('sess_key'));
	}
	
	public function set($key, $value) {
		$this->_set( $this->prefix . '_' . $key ,$value );
	}
	
	public function get($key) {
		return $this->_get( $this->prefix . '_' . $key );
	}
	
	private function _set($key, $value) {
		$_SESSION[$key] = $value;
	}
	
	private function _get($key) {
		return $_SESSION[$key];
	}
			
	public function destroy() {

		$_SESSION = array();

		if(isset($_COOKIE[session_name()])) {
			setcookie(session_name(), '', time()-42000, '/');
		}
		
		setcookie( 'hub_auth1', false, time()-42000, '/', $this->domain );

		session_destroy();

	}
	
}