<?php

class SignedCookie {
	
	public static $default_values = array( 'nonce' => '', 'timestamp' => '', 'token' => '' );
	public static $last_signature_data = null;
	
	public static function create( $cookie_name, $params, $secrets=array() ) {
	
		$params['token']     = $GLOBALS['SIGNED_COOKIE_TOKEN'];
		$params['timestamp'] = time();
		$params['nonce']     = mt_rand( 0, 10000000 ); // reserved for future use
		
		$signature = self::signature( $params, $secrets );
		
		$cookie_value = self::$last_signature_data . "signature=$signature";

		setcookie( $cookie_name, $cookie_value, time() + 1209600, '/', $GLOBALS['SESSION_DOMAIN'] );
		
	}
	
	public static function unserialize( $cookie_name ) {
		
		$cookie = array();
		parse_str( $_COOKIE[$cookie_name], $cookie );
		
		// merge with default values, so we know for sure that they exist
		$cookie = array_merge( self::$default_values, $cookie );
		
		return $cookie;
		
	}
	
	public static function verify( $cookie, $secrets=array() ) {
		
		$untrusted_signature = $cookie['signature'];
		unset( $cookie['signature'] );
		
		$signature = self::signature( $cookie, $secrets );
		
		return ( $untrusted_signature === $signature );
		
	}
	
	public static function signature( $data, $secrets=array() ) {
		
		// sort data alphabetically by key
		ksort( $data );
		// turn an array() in key=value&key1=value w/ trailing &
		$data_string = http_build_query( $data ) . '&';
		
		// since sorting and building the value is a potentially expensive
		// operation, lets store it so it can be accessed temporarily until
		// this method is run again
		self::$last_signature_data = $data_string;
		
		// add our default secret to the array
		$secrets['secret'] = $_GLOBALS['SIGNED_COOKIE_SECRET'];
		
		// do the same for the secrets array
		ksort( $secrets );
		$secret_string = http_build_query( $secrets );
		
		$signature = sha1( $data_string . $secret_string );
		
		return $signature;
		
	}
	
}