(function (window) {
	var campustream = {},
		cs = campustream;
		
	if (!('console' in window)) {
		window.console = (function () {
			return {
				log: function () {},
				info: function () {},
				error: function () {}
			};
		}());
	}
	
	window.redirect = function (dest) {
		window.location.href = dest;
	};

	var util = (function () {
		var url_regex = {
			home: /^\/?$/,
			user: /^\/?user\/([A-Za-z0-9_]+)\/?$/,
			user_friendships: /^\/?user\/([A-Za-z0-9_]+)\/(followers|follows)\/?$/,
			user_list: /^\/?users\/?$/,
			user_search: /^\/?users\/search\/?$/,
			collaborate: /^\/?collaborate\/?$/,
			collaborate_section: /^\/?collaborate\/([A-Za-z]+)\/?$/,
			collaborate_question: /^\/?collaborate\/q\/([A-Za-z0-9]+)\/?$/,
			settings: /^\/?settings\/(profile|services)\/?$/,
			confirm_code: /^\/?confirm\/wait\/?$/,
			news: /^\/?(news_events|news|events)\/?$/,
			news_create: /^\/?news\/new\/?$/,
			news_category: /^\/?(news|event)\/([A-Za-z0-9_]+)\/?$/,
			news_single: /^\/?ne\/([A-Za-z0-9]+)\/?$/
		},
		page_type = null;
		
		return {
			get_page_type : function () {
				if (page_type) {
					return page_type;
				}
				
				var	path = window.location.pathname,
					type;
				for (type in url_regex) {
					if (path.match(url_regex[type])) {
						page_type = type;
						return type;
					}
				}
				
				return false;
			},
			
			get_url_tokens : function () {
				var	path = window.location.pathname,
					type = this.get_page_type();
				
				if (type !== false) {
					return path.match(url_regex[type]).slice(1);
				}
				
				return [];
			},
			
			is_dev_site: function () {
				return (window.location.host.split('.')[0] === 'dev');
			},
			
			base_convert: function (number, frombase, tobase) {
				return parseInt(number + '', frombase | 0).toString(tobase | 0);
			}
		};
		
	}());
	
	var urls = (function () {
		var url_regex = /(?=[^"])((http|https):\/\/(([A-Z0-9\-]+\.)?([A-Z0-9\-]+)\.([A-Z]{2,4})(#|\?|\/)?([a-zA-Z0-9\?\=\-\/]+)))/gi,
			services = {
				twitpic: {
					type: 'photo',
					search: /(?:(?:https?\:\/\/)?(?:www\.)?)?twitpic\.com\/(?!(?:place|photos|events)\/)([a-zA-Z0-9\?\=\-]+)/g,
					replace: function (m1, m2) {
						return 'http://twitpic.com/show/mini/' + m2;
					}
				},
				imgur: {
					type: 'photo',
					search: /(?:http|https):\/\/imgur.com\/([A-Za-z0-9]+)(\.(png|gif|jpg))/g,
					replace: function (m1, m2, m3) {
						return 'http://imgur.com/' + m2 + 's' + m3;
					}
				}
			};
		
		return {
			find: function (text) {
				return text.match(url_regex);
			},
			
			parse_services: function (text) {
				var found = [],
					replacement;
				$.each(services, function (service, rx) {
					if (text.match(rx.search)) {
						$.each(text.match(rx.search), function (i, match) {
							replacement = match.replace(rx.search, rx.replace);
							switch (rx.type) {
							case 'photo':
								found.push({
									type: 'photo',
									image: replacement,
									link: match
								});
								break;
							}
						});
					}
				});
				
				return found;
			},
			
			linkify_text: function (text) {
				return text.replace(url_regex, function (m1) {
					return '<a target="_blank" rel="nofollow" href="' + m1 + '">' + m1 + '</a>';
				});
			}
		};
	}());
	
	var API = (function () {
		var url = '/',
			iframe_url,
			iframe,
			callbacks = {},
			xd_ready = true,
			queue = [];
			
		(function () {
			iframe = $("#xd-receiver").get(0).contentWindow;
			
			$("#xd-receiver").bind('load', function () {
				xd_ready = true;
				if (queue.length > 0) {
					$.each(queue, function (i, msg) {
						API.send_message(msg);
					});
				}
			});
		}());
			
		if (window.location.host.split('.')[0] === 'dev') {
			iframe_url = 'http://apidev.campustream.com/xd_receiver.html';
		} else {
			iframe_url = 'http://api.campustream.com/xd_receiver.html';
		}
		
		// Set up API query receiver
		$.receiveMessage(function (e) {
			var data = $.deparam(e.data, true),
				callback_id,
				result;

			callback_id = data.callback_id;
			result = data.result;
			
			if (!callback_id) {
				return;
			}
			
			console.log("API result:", result);
			
			callbacks[callback_id](result);
			delete callbacks[callback_id];
		});
		
		return {
			post: function (endpoint, args, callback) {
				this.query(endpoint, 'POST', args, callback);
			},
			
			get: function (endpoint, args, callback) {
				this.query(endpoint, 'GET', args, callback);
			},
			
			query: function (endpoint, method, args, callback) {
				var callback_id = new Date().getTime(),
					message = {
						endpoint: endpoint,
						method: method,
						args: args,
						current_url: (window.location.href.split('#')[0]),
						callback_id: callback_id
					};
				
				callbacks[callback_id] = callback || function () {};

				if (!xd_ready) {
					queue.push(message);
				} else {
					this.send_message(message);
				}
			},
			
			send_message: function (message) {
				console.log("API query:", message);
				$.postMessage(message, iframe_url, iframe);
			}
		};
	}());
	
	var View = (function () {
		var templates = {};
		
		templates.status = 
			'<div id="status_{{status_id}}" class="status-wrap">' +
				'<div class="status-avatar">' +
					'<a href="/user/{{username}}"><img src="{{avatar_url}}" /></a>' +
				'</div>' +
				'<div class="status-body">' +
					'<div class="status">' +
						'<a class="status-user" href="/user/{{username}}">{{name}}</a> {{message}}' +
					'</div>' +
					'<div class="status-meta">' +
						'{{action}} <a href="/s/{{short_id}}">{{post_time}}</a> &bull; ' +
						'<span class="link" onclick="cs.statuses.show_comments({{status_id}})">' +
							'<span id="num-comments-{{status_id}}">{{num_comments}}</span> comments' +
						'</span>' +
						'{{extra_meta}}' +
					'</div>' +
					'<div class="status-comments-wrap" style="display:none"></div>' +
				'</div>' +
				'<div style="clear:both;"></div>' +
			'</div>';

		templates.comment = 
			'<div class="comment-wrap">' +
				'<div class="comment-avatar">' +
				    '<a href="/user/{{username}}"><img src="{{avatar_url}}" /></a>' +
				'</div>' +
				'<div class="comment-body">' +
				    '<div class="comment">' +
				    	'<a class="comment-user" href="/user/{{username}}">{{name}}</a> {{message}}' +
				    '</div>' +
				'</div>' +
				'<div style="clear:both"></div>' +
			'</div>';
			
		templates.comment_form =
			'<div class="comment-form">' +
				'<input type="hidden" value="{{status_id}}" class="comment-status-id" />' +
				'<textarea id="comment_input_{{status_id}}">Leave a comment...</textarea>' +
				'<div class="comment-submit-button">' +
					'<input type="button" class="comment-submit fancy-button-blue" value="Comment" />' +
				'</div>' +
			'</div>';
			
		templates.question_stream =
			'<div class="question-stream-wrap">' +
				'<div class="question-stream-photo">' +
					'<img src="{{avatar_url}}" />' +
				'</div>' +
				'<div class="question-stream-content">' +
					'<h1><a href="/collaborate/q/{{question_shortid}}">{{question_title}}</a></h1>' +
					'<div class="question-tags">' +
						'{{question_tags}}' +
					'</div>' +
					'<p class="question-meta">' +
					'	{{num_responses}} responses &bull; asked by <a href="/user/{{username}}">{{user_name}}</a> {{post_time}}' +
					'</p>' +
				'</div>' +
				'<div style="clear:both"></div>' +
			'</div>';
			
		templates.question_search = 
			'<div id="question-search-wrap">' +
				'<input type="text" id="question-search" />' +
				'<input type="button" id="question-search-submit" class="fancy-button-blue" value="Search" />' +
			'</div>';
			
		templates.user_list =
			'<div class="user-list-item">' +
				'<div class="user-list-avatar">' +
					'<img src="{{user_avatar}}" />' +
				'</div>' +
				'<div class="user-list-info">' +
					'<h1><a href="/user/{{username}}">{{user_name}}</a></h1>' +
					'<h2><a href="mailto:{{username}}@wpi.edu">{{username}}</a></h2>' +
					'<p>{{user_bio}}</p>' +
					'<div class="user-list-meta">' +
						'<table>' +
							'<tr>' +
								'<td>Class of {{class_year}}</td>' +
								'<td>Followers: {{followers}}, Following: {{following}}</td>' +
							'</tr>' +
							'<tr>' +
								'<td>Majors: {{college_majors}}</td>' +
								'<td>Minors: {{college_minors}}</td>' +
							'</tr>' +
						'</table>' +
					'</div>' +
				'</div>' +
				'<div style="clear:both"></div>' +
			'</div>';
			
		templates.dialog = 
			'<div id="dialog-shadow"></div>' +
			'<div id="dialog-wrap">' +
				'<div id="dialog-title">{{title}}</div>' +
				'<div id="dialog-body">{{content}}</div>' +
				'<div id="dialog-actions">{{actions}}</div>' +
			'</div>';
		
		return {
			factory: function (type) {
				var viewObj,
					template;
				
				if (!templates[type]) {
					throw "Template not defined";
				}
				
				template = templates[type];
				
				viewObj = (function () {
					var values = {};
					
					return {
						set: function (i, val) {
							values[i] = val;
							return this;
						},
						
						render: function () {
							var HTML = template,
								regexp;
							
							$.each(values, function (search, replace) {
								regexp = new RegExp("{{" + search + "}}", "g");
								
								// crude test to see if it's a jQuery element
								if (replace.jquery) {
									replace = replace.appendTo('<div />').parent().html();
								}
								
								HTML = HTML.replace(regexp, replace);
							});
							
							return HTML;
						}
					};
				}());
				
				return viewObj;
			},
			
			plural: function (singular, num) {
				return singular + (num === 1 ? '' : 's');
			}
		};
	}());
	
	var User = (function () {
		var user = window.cs_user || {};

		return {
			get: function (prop) {
				return prop ? user[prop] : user;
			},
			
			isActiveUser: function (comp) {
				if (typeof comp === 'number') {
					return comp == user.id;
				} else if (typeof comp === 'string') {
					return comp == user.username;
				}
				
				return false;
			}
		};
	}());
	
	cs.ui = (function () {
		
		return {
			init: function () {
				console.log('ui module initialized');
				
				if (util.get_page_type() === 'settings') {
					this.iphone_inputs();
				}
				
				this.init_generic_events();
			},
			
			init_generic_events: function () {
			
				// User is following & clicks unfollow
				$(".follow-button > a.following")
					.live('mouseover mouseout', function (event) {
						if (event.type === 'mouseover') {
							$(this).addClass('unfollow').html('Unfollow');
						} else {
							$(this).removeClass('unfollow').html('Following');
						}
					})
					.live('click', function () {
						var	user,
							ele = $(this);
							
						if ($(this).parent().attr('id').length > 0) {
							user = $(this).parent().attr('id').split('_')[1];
						} else {
							user = util.get_url_tokens()[0];
						}

						API.post('friendships/destroy', {username: user}, function (data) {
							ele
								.removeClass('following')
								.removeClass('unfollow')
								.addClass('follow')
								.html('Follow');
							
							$(".followers-count").html(data.followers_count);
						});
					});
					
				$(".follow-button > a.follow").live('click', function () {
					var user,
						ele = $(this);
						
						if ($(this).parent().attr('id').length > 0) {
							user = $(this).parent().attr('id').split('_')[1];
						} else {
							user = util.get_url_tokens()[0];
						}
						
					API.post('friendships/create', {username: user}, function (data) {
						ele
							.removeClass('follow')
							.addClass('following')
							.html('Following');
							
						$(".followers-count").html(data.followers_count);
					});
				});
			},
			
			iphone_inputs: function () {
				$('.service-hooks :checkbox').iphoneStyle();
				
				// Bind onchange so we can update our settings
				$('.service-hooks :checkbox').bind('change', function () {
					var postData = {
						service: $(this).attr('id').split('-')[0],
						type: $(this).attr('id').split('-')[1],
						enable: $(this).is(':checked'),
						form_auth: $("#form_auth").attr('value')
					};
					
					API.post('settings/service_hooks', postData, function() {
						console.log("Settings saved");
					});
				});
				
				$("#notification-emails").iphoneStyle();
				$("#notification-emails").bind('change', function () {
					var postData = {
						form_auth: $("#email-form-auth").attr('value'),
						enable: $(this).is(":checked")
					};
					
					API.post('settings/email', postData, function () {
						console.log("Settings saved");
					});
				});
			}
		};
	}());
	
	cs.statuses = (function () {

		return {
			init: function () {
				console.log('statuses module initialized');
				
				this.init_global_events();
				
				var page = util.get_page_type();
				
				switch (page) {
				case 'home':
					this.init_home();
					this.load_home();
					break;
				case 'user':
					this.init_user();
					this.load_user();
					break;
				}
			},
			
			init_global_events: function () {
				$("#status-submit-button").bind('click', function () {
					var status, public;
					
					status = $.trim($("#status").attr('value'));
					if (status.length === 0) {
						return;
					}
					
					public = $("#status-submit [name=status-public]").attr('value');
					if (!public) {
						public = 0;
					} else {
						public = 1;
					}
					
					$("#status").attr('value', '');
					
					API.post('statuses/update', {message: status, public: public}, function (data) {
						var page = util.get_page_type(),
							statusOutput, streamType = cs.statuses.stream_type();
						
						switch (page) {
						case 'home':
							if ((streamType === 'public' || !streamType) && !public) {
								return;
							}
							statusOutput = $(cs.statuses.format_status(data));
							statusOutput.css({display: 'none'});
							$("#status-stream").prepend(statusOutput);
							statusOutput.slideDown();
							break;
						case 'user':
							if (streamType === 'recent' || !streamType) {
								statusOutput = $(cs.statuses.format_status(data));
								statusOutput.css({display: 'none'});
								$("#status-stream").prepend(statusOutput);
								statusOutput.slideDown();
							}
							
							break;
						}
					});
				});
				
				$(".comment-form > textarea").live('focus', function () {
					if ($(this).attr('value') === 'Leave a comment...') {
						$(this)
							.attr('value', '')
							.css({height: '60px', color: '#333333'});
					}
				});
				
				$(".comment-form > textarea").live('blur', function () {
					if ($.trim($(this).attr('value')) === '') {
						$(this)
							.attr('value', 'Leave a comment...')
							.css({height: '18px', color: '#555555'});
					}
				});
				
				$(".comment-submit").live('click', function () {
					var status_id = $(this)
						.parent().parent()
						.find('input.comment-status-id').attr('value');
						
						cs.statuses.submit_comment(status_id);
				});
			},
			
			init_home: function () {
				$("#nav-friend").bind('click', function () {
					window.location.hash = '#stream=friend';
					cs.statuses.load_home();
				});
				
				$("#nav-public").bind('click', function () {
					window.location.hash = "#stream=public";
					cs.statuses.load_home();
				});
			},
			
			load_home: function () {
				var stream = this.stream_type();
				
				if (!stream || stream === 'public') {
					stream = 'public';
					$("#stream-title").html('Recent Activity from Everyone');
					$("#nav-friend").removeClass('active');
					$("#nav-public").addClass('active');
				} else {
					$("#stream-title").html('Recent Activity from Your Friends');
					$("#nav-friend").addClass('active');
					$("#nav-public").removeClass('active');
				}
				
				$("#status-stream").empty();
				
				if (!$("#stream-loading").is(":visible")) {
					$("#stream-loading").fadeIn('fast');
				}
					
				API.get('statuses/' + stream + '_timeline', {count: 30}, function (data) {
					$("#stream-loading").hide();
					cs.statuses.output_statuses('#status-stream', data);
				});
			},
			
			init_user: function () {
				$("#nav-recent").bind('click', function () {
					window.location.hash = '#stream=user';
					cs.statuses.load_user();
				});
			},
			
			load_user: function () {
				var stream = this.stream_type(),
					user = util.get_url_tokens()[0],
					name = $("#user-sidebar-bio > h1").html(),
					endpoint;
				
				if (!stream || stream === 'user') {
					$("#stream-title").html('Recent Activity from ' + name);
					$("#nav-recent").addClass('active');
					$("#nav-wall").removeClass('active');
					endpoint = 'statuses/user_timeline';
				} else {
					$("#stream-title").html(user + "'s Wall");
					$("#nav-recent").removeClass('active');
					$("#nav-wall").addClass('active');
					endpoint = 'statuses/user_wall';
				}
				
				$("#status-stream").empty();
				
				if (!$("#stream-loading").is(":visible")) {
					$("#stream-loading").fadeIn('fast');
				}
				
				API.get(endpoint, {username: user, count: 30}, function (data) {
					$("#stream-loading").hide();
					cs.statuses.output_statuses('#status-stream', data);
				});
			},
			
			output_statuses: function (dest, statuses) {
				var statusOutput;
				
				if (!statuses || statuses.length === 0) {
					this.output_no_statuses(dest);
					return;
				}
				
				if($("#no-statuses").is(":visible")) {
					$("#no-statuses").slideUp('fast');
				}
				
				$.each(statuses, function (i, status) {
					cs.statuses.add_status(dest, status);
				});
			},
			
			output_no_statuses: function (dest) {
				$("#no-statuses").fadeIn('slow');
			},
			
			add_status: function (target, status) {
				var statusOutput = cs.statuses.format_status(status);
				$(target).append(statusOutput);
			},
			
			format_status: function (status) {
				var HTML = View.factory('status')
						.set('status_id', status.id)
						.set('short_id', util.base_convert(status.id, 10, 36))
						.set('avatar_url', status.user.avatar.replace('medium', 'small'))
						.set('username', status.user.username)
						.set('name', status.user.name)
						.set('post_time', $.timeago(status.post_date))
						.set('num_comments', status.num_comments);
				
				status.message = urls.linkify_text(status.message);
				
				switch (status.type) {
				case 'question':
					HTML.set(
						'message',
						'asked, <a href="/collaborate/q/' + 
						util.base_convert(status.content_id, 10, 36) + '">' +
						status.message + '</a>'
					);
					
					HTML.set('action', 'Asked');
					
					break;
				case 'response':
					HTML.set(
						'message',
						'replied <a href="/collaborate/q/' + 
						util.base_convert(status.content_id, 10, 36) + '">' +
						'to a question' + '</a> and said, "' +
						status.message + '"'
					);
					
					HTML.set('action', 'Replied');
					
					break;
				case 'newsevent':
					HTML.set(
						'message',
						'posted, <a href="/ne/' +
						util.base_convert(status.content_id, 10, 36) + '">' +
						status.message + '</a>'
					);
					
					HTML.set('action', 'Posted');
					
					break;
				default:
					HTML.set('message', status.message);
					HTML.set('action', 'Posted');
				}
				
				if (User.isActiveUser(status.user.id)) {
					HTML.set(
						'extra_meta',
						' &bull; <span class="link" onclick="cs.statuses.delete_status(' + status.id + ')">Delete</span>'
					);
				} else {
					HTML.set('extra_meta', '');
				}
				
				return HTML.render();
			},
			
			show_comments: function (status_id) {
				var comment_wrap = $("#status_" + status_id + " .status-comments-wrap");
				
				if (comment_wrap.is(":visible")) {
				    comment_wrap.slideUp();
				} else {
				    comment_wrap.slideDown();
				}
				
				if (comment_wrap.html().length !== 0) {	
					return;
				}
				
				API.get('statuses/comments', {status_id: status_id}, function (status) {
					cs.statuses.inject_comments(status);
				});
			},
			
			
			inject_comments: function (status) {
				var	comment_wrap = $("#status_" + status.id + " .status-comments-wrap"),
					comment_form = View.factory('comment_form')
						.set('status_id', status.id)
						.render(),
					comment_html;
					
				comment_wrap.prepend(comment_form);
				
				if (!status.comments || status.comments.length === 0) {
					return;
				}
				
				$.each(status.comments, function (i, comment) {
					cs.statuses.inject_comment(comment);
				});
			},
			
			inject_comment: function (comment) {
			    comment_html = View.factory('comment')
			    		.set('username', comment.user.username)
			    		.set('avatar_url', comment.user.avatar.replace('medium', 'small'))
			    		.set('name', comment.user.name)
			    		.set('message', comment.message)
			    		.render();
			    		
			    $("#status_" + comment.status_id + " .status-comments-wrap").append(comment_html);
			},
			
			submit_comment: function (status_id) {
				var textarea = $("#comment_input_" + status_id);
				
				comment = $.trim(textarea.attr('value'));
				textarea
					.attr('value', 'Leave a comment...')
					.css({height: '18px', color: '#555555'});
						
				if (comment.length === 0 || comment === 'Leave a comment...') {
					return;
				}
				
				API.post('comments/update', {message: comment, status_id: status_id}, function (comment) {
					cs.statuses.inject_comment(comment);
					$("#num-comments-" + status_id).html(Number($("#num-comments-" + status_id).html()) + 1);
				});
			},
			
			delete_status: function (status_id) {
				if (!status_id || typeof status_id != "number") {
					return;
				}
				
				API.post('statuses/delete', {id: status_id}, function () {
					$("#status_" + status_id).slideUp();
				});
			},
			
			stream_type: function () {
				var data = $.deparam.fragment();
				
				if (data.stream) {
					return data.stream;
				} else {
					return false;
				}
			}
		};
	}());
	
	cs.collaborate = (function () {
	
		return {
			init: function (page) {
				console.log("collaborate module initialized");
				
				switch (page) {
				case 'collaborate':
					break;
				case 'collaborate_section':
					this.init_events();
					this.load_questions();
					break;
				case 'collaborate_question':
					this.init_question_page();
					break;
				}
			},
			
			init_events: function () {
				$("#collaborate-post").live('focus', function () {
					if ($(this).attr('value') === 'Ask a question!') {
						$(this)
							.attr('value', '')
							.animate({height: '150px'})
							.css({color: '#333333'});
					}
				});
				
				$("#collaborate-post").live('blur', function () {
					if ($.trim($(this).attr('value')) === '') {
						$(this)
							.attr('value', 'Ask a question!')
							.animate({height: '50px'})
							.css({color: '#555555'});
					}
				});
				
				$("#collaborate-submit").live('click', function () {
					var title = $.trim($("#collaborate-title").attr('value')),
						message = $.trim($("#collaborate-post").attr('value')),
						tags = $.trim($("#collaborate-tags").attr('value')),
						category = util.get_url_tokens()[0],
						public = $(".collaborate-submit [name=question-public]").attr('value');
					
					if (title.length === 0 || message.length === 0) {
						return;
					}
					
					API.post(
						'collaborate/create', 
						
						{
							category: category, 
							title: title,
							message: message, 
							tags: tags,
							public: public
						}, 
						
						function (question) {
							window.location.href = '/collaborate/q/' + question.short_id;
						}
					);
				});
				
				$("#collaborate-navbar > li").live('click', function () {
				  var sort = $(this).attr('id').substr(4).replace('-', '_');
				  window.location.hash = "#sort=" + sort;
				  
				  cs.collaborate.load_questions(true);
				});
				
				$("#question-search-submit").live('click', function () {
					cs.collaborate.do_search();
				});
				
				$("#question-search").live('keydown', function (e) {
					if (e.keyCode === 13) {
						cs.collaborate.do_search();
					}
				});
				
				$(".question-tag").live('click', function () {
					var activeTab = $("#collaborate-navbar .active").attr('id').split('-')[1],
						search = $(this).html();
					
					// tab isn't the search tab... need to switch
					if (activeTab !== 'search') {
						cs.collaborate.load_search();
					}
					
					$("#question-search").attr('value', search);
					cs.collaborate.do_search();
				});
			},
			
			init_question_page: function () {
				// Response submit
				$("#collaborate-submit").live('click', function () {
					var	response = $.trim($("#question-respond-input").attr('value')),
						broadcast = $(".collaborate-submit [name=question-public]:checked").attr('value'),
						short_id = util.get_url_tokens()[0];
						
					if (response.length === 0) {
						return;
					}
					
					API.post('collaborate/respond', {short_id: short_id, message: response, public: broadcast}, function (response) {
						window.location.reload();
					});
				});
				
				// Question deletion
				$(".delete-question").live('click', function () {
				  var id = $(this).attr('id').split('-')[0];
				  
				  API.post('collaborate/delete', {id: id}, function () {
				  	window.location.href = '/collaborate';
				  });
				});
				
				// Response voting
				$(".vote-up").live('click', function () {
					var response_id = $(this).parent().parent().attr('id').split("-")[1];
					
					var that = this;
					API.post('collaborate/vote', {dir: 'up', id: response_id}, function (response) {
						var down = $(that).parent().find('.vote-down'),
							up = $(that),
							count = up.parent().find('.vote-count'),
							details = up.parent().parent().find('.voting-details'),
							details_html = "";
						
						up.toggleClass('active');
						if (down.hasClass('active')) {
							down.removeClass('active');
						}
						
						if (up.hasClass('active')) {
							count.addClass('active');
						} else {
							count.removeClass('active');
						}
						
						count.html(response.vote_count);
						
						details_html = 
						response.up_votes + " up " + View.plural('vote', response.up_votes) +
						", " + response.down_votes + " down " + View.plural('vote', response.down_votes);
						
						details.html(details_html);
					});
				});
				
				$(".vote-down").live('click', function () {
					var response_id = $(this).parent().parent().attr('id').split("-")[1];
					
					var that = this;
					API.post('collaborate/vote', {dir: 'down', id: response_id}, function (response) {
						var down = $(that),
							up = $(that).parent().find('.vote-up'),
							count = up.parent().find('.vote-count'),
							details = up.parent().parent().find('.voting-details'),
							details_html = "";;
						
						down.toggleClass('active');
						if (up.hasClass('active')) {
							up.removeClass('active');
						}
						
						if (down.hasClass('active')) {
							count.addClass('active');
						} else {
							count.removeClass('active');
						}
						
						count.html(response.vote_count);
						
						details_html = 
						response.up_votes + " up " + View.plural('vote', response.up_votes) +
						", " + response.down_votes + " down " + View.plural('vote', response.down_votes);
						
						details.html(details_html);
					});
				});
				
				// Linkify and embedded services asynchronously
				this.format_content();
			},
			
			load_questions: function (clear) {
				var opts = $.deparam.fragment(),
					section = util.get_url_tokens()[0],
					page = opts.page || 1,
					sorting = opts.sort || 'last_updated';
					
				$("#nav-" + sorting.replace("_", "-")).parent().find('.active').removeClass('active');
				$("#nav-" + sorting.replace("_", "-")).addClass('active');
				
				if (sorting === 'search') {
					return cs.collaborate.load_search();
				}
					
				if (clear) {
				   	$("#question-stream").empty();
				}

				API.get('collaborate/list', {section: section, page: page, sorting: sorting}, function (questions) {
					$("#stream-loading").hide();
					
					if (questions.length === 0) {
						$("#no-statuses").fadeIn();
						return;
					} else {
						$("#no-statuses").hide();
					}
					
					$.each(questions, function (i, question) {
						var tags = "";
							
						if (question.tags && question.tags.length > 0) {
						    $.each(question.tags, function (i, tag) {
						    	tags += '<span class="question-tag">' + tag.tag + '</span> ';
						    });
						}
							
						var HTML = View.factory('question_stream')
							.set('avatar_url', question.user.avatar.replace('medium', 'small'))
							.set('question_shortid', question.short_id)
							.set('question_title', question.title)
							.set('num_responses', question.num_responses)
							.set('user_name', question.user.name)
							.set('username', question.user.username)
							.set('post_time', $.timeago(question.post_date))
							.set('question_tags', tags)
							.render();
								
						$("#question-stream").append(HTML);
					});
				});
			},
			
			load_search: function () {
				$("#nav-search").parent().find('.active').removeClass('active');
				$("#nav-search").addClass('active');
				$("#no-statuses").hide();
				$("#stream-loading").hide();
				$("#question-stream").empty();
				
				var HTML = View.factory('question_search').render();
				$("#question-stream").html(HTML).append('<div id="search-results"></div>');
			},
			
			do_search: function () {
				var search = $.trim($("#question-search").attr('value'));
				
				if (search.length === 0) {
					return;
				}
				
				$("#search-results").empty();
				
				API.get('search/tags', {q: search}, function (questions) {
					$.each(questions, function (i, question) {
						var tags = "";
							
						if (question.tags && question.tags.length > 0) {
						    $.each(question.tags, function (i, tag) {
						    	tags += '<span class="question-tag">' + tag.tag + '</span> ';
						    });
						}
							
						var HTML = View.factory('question_stream')
							.set('avatar_url', question.user.avatar.replace('medium', 'small'))
							.set('question_shortid', question.short_id)
							.set('question_title', question.title)
							.set('num_responses', question.num_responses)
							.set('user_name', question.user.name)
							.set('username', question.user.username)
							.set('post_time', $.timeago(question.post_date))
							.set('question_tags', tags)
							.render();
								
						$("#search-results").append(HTML);
					});
				});
			},
			
			format_content: function () {
				setTimeout(function () {
					var question = $("#question-content").html(),
						question_services = urls.parse_services(question);
						
					if (question_services.length > 0) {
						$("#question-attachments").show();
						$.each(question_services, function (i, service) {
							cs.collaborate.inject[service.type](service, $("#question-media"));
						});
					}
					
					$("#question-content").html(urls.linkify_text(question));
					
					$.each($(".response-content > p"), function () {
						var content = $(this).html(),
							services = urls.parse_services(content),
							that = this;
							
						if (services.length > 0) {
							$(this).parent().parent().find('.response-attachments').show();
							$.each(services, function (i, service) {
								cs.collaborate.inject[service.type](service, $(that).parent().parent().find('.response-media'));
							});
						}
						
						$(this).html(urls.linkify_text(content));
					});
				}, 1);
			},
			
			inject: {
				photo: function (data, target) {
					var html = '<a target="_blank" href="' + data.link + '"><img src="' + data.image + '" /></a> ';
					target.append(html);
				}
			}
		};
	}());
	
	cs.users = (function () {
		
		return {
			init: function (page) {
				console.log("users module initialized");
				
				switch (page) {
				case 'user_list': this.init_user_list();
				}
			},
			
			init_user_list: function () {					
				$('.search-letter').live('click', function () {
					cs.users.load_list($(this).html());
				});
				
				this.load_list();
			},
			
			load_list: function (new_letter) {
				var letter = (new_letter || ($.deparam.fragment().letter || 'a')).toLowerCase();
				
				$("#user-list").empty();
				$("#stream-loading").fadeIn();
				
				$("#search-letter-" + letter).parent().find('.active').removeClass('active');
				$("#search-letter-" + letter).addClass('active');
				
				API.post('users/list', {letter: letter}, function (users) {
					// update the window hash
					window.location.href = $.param.fragment(window.location.href, {letter: letter});
					
					if (users.length == 0) {
						$("#stream-loading").hide();
						$("#no-statuses").fadeIn();
						return;
					}
					
					$.each(users, function (i, user) {
						var HTML = View.factory('user_list')
							.set('user_avatar', user.avatar)
							.set('user_name', user.name)
							.set('username', user.username)
							.set('user_bio', user.bio)
							.set('class_year', user.class_year || 'N/A')
							.set('college_majors', user.college_majors || 'N/A')
							.set('college_minors', user.college_minors || 'N/A')
							.set('followers', user.followers_count)
							.set('following', user.follows_count)
							.render();
							
						$("#user-list").append(HTML);
					});
					
					$("#no-statuses").hide();
					$("#stream-loading").hide();
				});
			}
		}
	}());
	
  cs.news = (function () {
    var page;
    
    return {
      init: function (_page) {
        console.log("news module initialized");
        var self = this;
        
        page = _page;
        
        $("#news-sidebar-submit").live('click', function () {
          window.location.href = '/news/new';
        });
        
        $(".vote-up").live('click', function () {
        	var news_id = $(this).parent().parent().attr('id').split('_')[1];
        	self.vote(news_id, 'up');
        });
        
        $(".vote-down").live('click', function () {
        	var news_id = $(this).parent().parent().attr('id').split('_')[1];
        	self.vote(news_id, 'down');
        });

        if (page === 'news_create') {
        	this.init_news_creator();
        }
        
        if (page === 'news_single') {
        	this.init_news_single();
        }
      },
      
      init_news_single: function () {
      	var self = this;
      	
      	$(".comment-reply-button > input").live('click', function () {
      		var id = util.get_url_tokens()[0],
      		reply_to = $(this).parent().parent().attr('id').split('-')[2],
      		content = $(this).parent().parent().find('.news-comment-reply').attr('value');
      		
      		self.post_comment(id, reply_to, content);
      	});
      	
      	$(".news-expand-reply").live('click', function () {
      		$(this).parent().parent().find('.news-comment-reply-to').slideToggle();
      	});
      	
      	$(".news-toggle-children").live('click', function () {
      		$(this).parent().parent().children('.news-child-comments').toggle();
      	});
      	
      	$(".news-delete-comment").live('click', function () {
					var id = $(this).parent().parent().attr('id').split('-')[2];
					
					API.post('news/delete_comment', {id: id}, function () {
						window.location.reload();
					});
      	});
      },
      
      init_news_creator: function () {
      	var type = 'link';
      	
      	$("#news-submit-text").osimoeditor({
      	  editor_path: '/js/osimoeditor/',
      	  width: '500px',
      	  styles: {
      	  	width: '510px'
      	  }
      	});
      	
      	$("#user-navbar li").live('click', function () {
      	  type = $(this).attr('id').split('-')[1];
      	  
      	  $(".news-submit-wrap").hide();
      	  $("#news-submit-" + type + "-wrap").show();
      	  
      	  $(this).parent().find('.active').removeClass('active');
      	  $(this).addClass('active');
      	});
      	
      	$("#news-submit-category .category").live('click', function () {
      		$(".category").removeClass('active');
      		$(this).addClass('active');
      		
      		if ($(this).hasClass('category-event')) {
      			$("#event-extra").show();
      		} else {
      			$("#event-extra").hide();
      		}
      	});
      	
      	$("#event-location").autocomplete({
      		minLength: 3,
      		source: function (request, response) {
      			API.get('news/location_search', {q: request.term}, function (resp) {
      				response( resp );
      			});
      		}
      	});
      	
      	$("#event-date-to, #event-date-from").datepicker({
      		minDate: 0,
      		showOtherMonths: true,
      		selectOtherMonths: true
      	});
      	
      	$("#event-time-allday").live('change', function () {
      		if ($(this).is(":checked")) {
      			$("#event-time-from-hour").attr('disabled', true);
      			$("#event-time-from-minute").attr('disabled', true);
      			$("#event-time-from-ampm").attr('disabled', true);
      			
      			$("#event-to").hide();
      		} else {
      			$("#event-time-from-hour").attr('disabled', false);
      			$("#event-time-from-minute").attr('disabled', false);
      			$("#event-time-from-ampm").attr('disabled', false);
      			
      			$("#event-to").show();
      		}
      	});
      	
      	$("#submit-news").live('click', function () {
      		var title, content, category;
      		
      		title = $("#news-submit-" + type + "-wrap .title").attr('value');
      		content = $("#news-submit-" + type + "-wrap .content").attr('value');
      		category = $("#news-submit-category .active");
      		if (category.length) {
      			category = category.attr('id').split('-')[1];
      		} else {
      			return;
      		}
      		
      		var data = {
      			title: title,
      			content: content,
      			category: category,
      			type: type
      		};
      		
      		// Extra event metadata
      		if ($("#news-submit-category .active").hasClass('category-event')) {
      			
      			data.event_location = $("#event-location").attr('value');
      			data.event_location_area = $("#event-location-room").attr('value');
						
						if ($("#event-time-allday").is(":checked")) {
							data.event_all_day = 1;
						} else {
							data.event_start_from_hour = $("#event-time-from-hour").attr('value');
							data.event_start_from_minute = $("#event-time-from-minute").attr('value');
							data.event_start_from_ampm = $("#event-time-from-ampm").attr('value');
							data.event_start_from_date = $("#event-date-from").attr('value');
							
							data.event_start_to_hour = $("#event-time-to-hour").attr('value');
							data.event_start_to_minute = $("#event-time-to-minute").attr('value');
							data.event_start_to_ampm = $("#event-time-to-ampm").attr('value');
							data.event_start_to_date = $("#event-date-to").attr('value');
						}
      		}
      		
      		API.post('news/create', data, function (news) {
      			window.location.href = '/ne/' + news.short_id + "/?new=1";
      		});
      	});
      },
      
      post_comment: function (id, reply_to, comment) {
      	var data = {short_news_id: id, reply_to: reply_to, content: comment};
      	API.post('news/create_comment', data, function (resp) {
      		window.location.reload(); // I'm lazy for now... maybe make this inject in future?
      	});
      },
      
      vote: function (id, dir) {
      	API.post('news/vote', {id: id, dir: dir}, function (response) {
      		var root = $("#news_" + id),
      		down = root.find('.vote-down'),
					up = root.find('.vote-up'),
					count = root.find('.vote-count');
					
					if (dir == 'down') {
						down.toggleClass('active');
						if (up.hasClass('active')) {
						  up.removeClass('active');
						}
						
						if (down.hasClass('active')) {
						  count.addClass('active');
						} else {
						  count.removeClass('active');
						}					
					} else {
						up.toggleClass('active');
						if (down.hasClass('active')) {
							down.removeClass('active');
						}
						
						if (up.hasClass('active')) {
							count.addClass('active');
						} else {
							count.removeClass('active');
						}
					}
					
					count.html(response.votes);
      	});
      }
    };
  }());
	
	cs.notifications = (function () {
		
		return {
			init: function () {
				console.log("notifications module initialized");
				
				$("#notification-count").live('click', function () {
					$("#notification-count-wrap").hide('slide', {direction: 'down'}, 100, function () {
						$("#notification-wrap").show('slide', {direction: 'down'}, 300);
					});
				});
				
				$("#notification-close").live('click', function () {
					$("#notification-wrap").hide('slide', {direction: 'down'}, 300, function () {
						$("#notification-count-wrap").show('slide', {direction: 'down'}, 100);
					});
				});
				
				$(".notification-mark-read").live('click', function () {
					var type = $(this).attr('id').split('-')[0];
					
					cs.notifications.markAsRead(type);
				});
			},
			
			markAsRead: function (type) {
				API.post('notifications/mark_as_read', {type: type}, function (resp) {
					var num = Number(resp.news) + Number(resp.collaborate) + Number(resp.reply);
					
					$("#notification-count").html(num);
					if (num == 0) {
						$("#notification-count").removeClass('unread');
					}
					
					$("#" + type + "-mark-read").fadeOut();
					$("#" + type + "-mark-read").parent().find('ul > li')
						.removeClass('notification-unread')
						.addClass('notification-read');
				});
			}
		}
	}());
	
	window.campustream = campustream;
	window.cs = campustream;
	
	$(document).ready(function () {
		var page = util.get_page_type();
		console.log("Page type is: " + page);
		cs.ui.init();
		cs.statuses.init();
		cs.notifications.init();
		
		switch (page) {
		case 'collaborate':
		case 'collaborate_question':
		case 'collaborate_section':
			cs.collaborate.init(page);
			break;
		case 'user_list':
			cs.users.init(page);
			break;
		case 'news':
		case 'news_create':
		case 'news_category':
		case 'news_single':
		  cs.news.init(page);
		  break;
		}
	});

}(window));